package net.metadiversity.diversity.navigator.connection;

import java.util.*;
//import javax.xml.parsers.*;
//import org.xml.sax.*;
//import org.xml.sax.helpers.*;
import org.w3c.dom.*;

import java.io.File;

//import net.metadiversity.diversity.navigator.io.*;
import net.metadiversity.diversity.navigator.util.Crypt;

import org.apache.log4j.Logger;

/*  all informations about connections are stored in an xml-file "connection.xml"
 *  located in user-home directory
 *
 */
public class ConnectionsManager 
{
    static Logger logger = Logger.getLogger( ConnectionsManager.class.getName() );
    static String configFileName;
    /*
    final static String CONFIG_FILE_NAME            = "diversity_navigator.connections.xml";    
    final static String CONFIG_FILE_NAME_DTD        = "diversity_navigator.connections.dtd";    
    final static String CONNECTION_ROOT_TAG         = "CONNECTIONS";
    final static String CONNECTION_TAG              = "connection";
    final static String CONNECTION_NAME_TAG         = "name";
    final static String CONNECTION_DESCRIPTION_TAG  = "description";
    final static String CONNECTION_DRIVER_TAG       = "driver";
    final static String CONNECTION_URL_TAG          = "url";
    final static String CONNECTION_USER_TAG         = "user";
    final static String CONNECTION_PWD_TAG          = "pwd";
    */
    // Used from DN Version 0.97.xx
    final static String CONFIG_FILE_NAME            = "diversity_navigator.services.xml";    
    final static String CONFIG_FILE_NAME_DTD        = "diversity_navigator.services.dtd";    
    final static String CONNECTION_ROOT_TAG         = "SERVICES";
    final static String CONNECTION_TAG              = "service";
    final static String CONNECTION_NAME_TAG         = "resource";
    final static String CONNECTION_DESCRIPTION_TAG  = "description";
    final static String CONNECTION_DRIVER_TAG       = "driver";
    final static String CONNECTION_URL_TAG          = "accesspoint";
    final static String CONNECTION_USER_TAG         = "user";
    final static String CONNECTION_PWD_TAG          = "pwd";
    
    private static LinkedHashMap connections;
    
    final static Crypt crypt = new Crypt();
    
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public ConnectionsManager()
    {
        // read file from user.home directory
	String path = System.getProperty( "user.home" );
        configFileName = path + "/.diversity_navigator/" + CONFIG_FILE_NAME;
	connections = new LinkedHashMap();                
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public static ConnectionInfo getConnectionInfoByName( String connectionName )
    {
        ConnectionInfo ci = null;
        try
        {
            if( connections == null )
            {
                getConnections();
            }
            ci = ( ConnectionInfo ) connections.get( connectionName );
            logger.info( "get connection info: " + connectionName );     
        }
        catch( java.lang.Exception ex )        
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );     
        }
        return ci;
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public static LinkedHashMap getConnections() throws Exception 
    {
        // read file from user.home directory
        if( configFileName == null )
        {
            String path = System.getProperty( "user.home" );
        }    
        // test if connection-file exist        
	File f = new java.io.File( configFileName );
	if( ! f.exists() ) 
        {
             logger.error( "Can't read file: " + configFileName ); 
             return new LinkedHashMap();
        }
	connections = new LinkedHashMap();                
        // get XML factory
        try
        {
            javax.xml.parsers.DocumentBuilderFactory factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
            factory.setValidating( true );        // enable Validating with DTD
            javax.xml.parsers.DocumentBuilder docBuilder = factory.newDocumentBuilder();
            docBuilder.setErrorHandler( new XmlSaxErrorHandler() ); 
            org.w3c.dom.Document doc = docBuilder.parse( configFileName );
            Element root = doc.getDocumentElement();
            NodeList nodeList = root.getElementsByTagName( CONNECTION_TAG );
            for( int i = 0; i < nodeList.getLength(); i++ ) 
            {
                Element connNode = ( Element ) nodeList.item( i );
                String name = "";
                String description = "";
                String driver = "";
                String url = "";
                String user = "";
                String pwd = "";
                org.w3c.dom.Node node;
                name    = ( (Element) connNode.getElementsByTagName( CONNECTION_NAME_TAG ).item(0)).getFirstChild().getNodeValue();
                // description is maybe empty
                description = "";
                Element e = (Element) connNode.getElementsByTagName( CONNECTION_DESCRIPTION_TAG ).item(0);
                if( e != null )
                {    
                    node  = e.getFirstChild();
                    if( node != null )
                    {   
                        description = node.getNodeValue();
                    }
                }    
                driver  = ( (Element) connNode.getElementsByTagName( CONNECTION_DRIVER_TAG ).item(0)).getFirstChild().getNodeValue();
                url     = ( (Element) connNode.getElementsByTagName( CONNECTION_URL_TAG ).item(0)).getFirstChild().getNodeValue();
                user    = ( (Element) connNode.getElementsByTagName( CONNECTION_USER_TAG ).item(0)).getFirstChild().getNodeValue();
                // get password
                node = ( (Element) connNode.getElementsByTagName( CONNECTION_PWD_TAG ).item(0)).getFirstChild();                
                if( node == null)
                {
                    pwd = "";
                }
                else 
                {   
                    pwd = node.getNodeValue();
                    if( pwd.compareTo( "" ) != 0 )
                    {
                        pwd = crypt.decrypt( pwd );
                    }
                }    
                // set data
                connections.put( name, new ConnectionInfo( name, description, driver, url, user, pwd ) );                
            }
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        return connections;
    }
    //--------------------------------------------------------------------------
    // Add Connection 
    //--------------------------------------------------------------------------
    public static void addConnection( String name, String description, String driver, String url, String user, String pwd )
    {
        try
        {
            LinkedHashMap connections = getConnections();
            connections.put( name, new ConnectionInfo( name, description, driver, url, user, pwd ) );
            saveConnections( connections );
        }
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    //--------------------------------------------------------------------------
    // Change Connection 
    //--------------------------------------------------------------------------
    public static void changeConnection( String name, String description, String driver, String url, String user, String pwd )
    {
        try
        {
            LinkedHashMap connections = getConnections();
            if( connections.containsKey( name ) )
            {
                connections.put( name, new ConnectionInfo( name, description, driver, url, user, pwd ) );
                saveConnections( connections );
            }
            else
            {    
                logger.fatal( "Can't change connection: " + name );
            }    
        }
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    //--------------------------------------------------------------------------
    // Delete Connection 
    //--------------------------------------------------------------------------
    public static void deleteConnection( String name )
    {
        try
        {
            LinkedHashMap connections = getConnections();
            connections.remove( name );
            saveConnections( connections );
        }
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
    }
    //--------------------------------------------------------------------------
    // Save Connection 
    //--------------------------------------------------------------------------
    private static void saveConnections( LinkedHashMap connections )
    {
        try
        {
            logger.info( "Write file: " + configFileName );
            javax.xml.parsers.DocumentBuilderFactory factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
            javax.xml.parsers.DocumentBuilder docBuilder = factory.newDocumentBuilder();
            docBuilder.setErrorHandler( new XmlSaxErrorHandler() ); 
            
            factory.setValidating( true );        // enable Validating with DTD
                       
            org.w3c.dom.Document doc = docBuilder.newDocument();
            Element rootElement     = doc.createElement( CONNECTION_ROOT_TAG );
            
            doc.appendChild( rootElement );

            Iterator iter = connections.keySet().iterator();
            while( iter.hasNext() )
            {
                ConnectionInfo ci = (ConnectionInfo) connections.get( iter.next() );
            
                Element conElement      = doc.createElement( CONNECTION_TAG );
                Element nameElement     = doc.createElement( CONNECTION_NAME_TAG );
                Element descElement     = doc.createElement( CONNECTION_DESCRIPTION_TAG );
                Element driverElement   = doc.createElement( CONNECTION_DRIVER_TAG );
                Element urlElement      = doc.createElement( CONNECTION_URL_TAG );
                Element userElement     = doc.createElement( CONNECTION_USER_TAG );
                Element pwdElement      = doc.createElement( CONNECTION_PWD_TAG );

                nameElement.appendChild(    doc.createTextNode( ci.getName() ) );
                descElement.appendChild(    doc.createTextNode( ci.getDescription() ) );
                driverElement.appendChild(  doc.createTextNode( ci.getDriver() ) );
                urlElement.appendChild(     doc.createTextNode( ci.getUrl() ) );
                userElement.appendChild(    doc.createTextNode( ci.getUser()  ) );
                 // we will write all passwords encrypted
                String pwd = ci.getPwd();
                if( pwd == null )
                {
                    pwd = "";
                }
                if( pwd.compareTo( "" ) == 0 )
                {    
                    pwdElement.appendChild(     doc.createTextNode(  pwd ) );
                }
                else
                {    
                    pwdElement.appendChild(     doc.createTextNode( crypt.encrypt( pwd ) ) );
                }
                
                conElement.appendChild( nameElement );    
                conElement.appendChild( descElement );    
                conElement.appendChild( driverElement );    
                conElement.appendChild( urlElement );    
                conElement.appendChild( userElement );    
                conElement.appendChild( pwdElement );    
                
                rootElement.appendChild( conElement );
            }
            
            
            javax.xml.transform.Transformer t = javax.xml.transform.TransformerFactory.newInstance().newTransformer();
            //
            // Properties see: javax.xml.transform.OutputKeys
            //
            t.setOutputProperty( "method", "xml" );
            t.setOutputProperty( "standalone" , "no" );
            t.setOutputProperty( "indent" , "yes" );
            t.setOutputProperty( "doctype-system" , CONFIG_FILE_NAME_DTD );
            // write to file
            t.transform( new javax.xml.transform.dom.DOMSource( doc ),  new javax.xml.transform.stream.StreamResult( new java.io.FileOutputStream( configFileName ) ));
            
                                    
        }
        catch( Exception ex ) 
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }            
    }
    //--------------------------------------------------------------------------
    // Errorhandler
    //--------------------------------------------------------------------------
    public static class XmlSaxErrorHandler implements org.xml.sax.ErrorHandler
    {        
        
        public void warning( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException 
        {
            // System.out.println( exception.toString() );
            logger.warn( buildMsg( ex ) );
        }
        public void error( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException 
        {
            // System.out.println( exception.toString() );
            logger.error( buildMsg( ex ) );
        }
        public void fatalError( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException
        {
            // System.out.println( exception.toString() );
            logger.fatal( buildMsg( ex ) );            
        }
        // format error message
        private String buildMsg( org.xml.sax.SAXParseException ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            String msg = configFileName + "\n Line: " + ex.getLineNumber() + " Column: " + ex.getColumnNumber() + "\n" + ex.toString();            
            return msg;
        }
    }
}

