/*
 * CharacterStateEditorModel.java
 *
 * Created on 13. Februar 2004, 19:35
 */

package net.metadiversity.diversity.navigator.gui.delta_editor;

import net.metadiversity.diversity.navigator.db.delta_editor.CharacterStateDescriptionEntry;
import net.metadiversity.diversity.navigator.gui.DnOptionPane;
import org.apache.log4j.Logger;
/**
 *
 * @author  didi
 */
public class CharacterStateEditorModel implements javax.swing.table.TableModel
{
    static Logger logger = Logger.getLogger( CharacterStateEditorModel.class.getName() );
        
    final static int CS         = 0;
    final static int MODIFIER   = 1;
    final static int X          = 2;
    final static int TEXT       = 3;
    final static int NOTES      = 4;
    // only one field for comments
    // if "TE" we use Text-file, else use Notes-field as comment.
    // Text + Notes --> Comments
    final static boolean onlyUseOneNotesTextFieldFlag = true;
    // getX() is a double value, use Double or Text 
    final static boolean useNumbersAsText = true;
    
    javax.swing.event.EventListenerList listenerList;
    DeltaEditorChangeEvent deltaEditorChangeEvent;
    
    java.util.Vector states = null;
    java.lang.String characterName = null;            
    //------------------------------------------------------------------
    public CharacterStateEditorModel()
    {
        states = new java.util.Vector();
        this.characterName = "Character Name";                                
        listenerList = new javax.swing.event.EventListenerList();
    }
    //------------------------------------------------------------------
    //------------------------------------------------------------------
    public CharacterStateEditorModel(java.lang.String characterName)
    {
        states = new java.util.Vector();
        this.characterName = characterName;                                
        listenerList = new javax.swing.event.EventListenerList();
    }
    //------------------------------------------------------------------
    //------------------------------------------------------------------
    public void addCharacterStateDescriptionEntry( CharacterStateDescriptionEntry csde )
    {
        states.add( csde );
    }
    //------------------------------------------------------------------
    //------------------------------------------------------------------
    public boolean removeCharacterStateDescriptionEntry( String cs )
    {
        CharacterStateDescriptionEntry csde;
        for( int i = 0; i < states.size(); i++ )
        {
            csde = (CharacterStateDescriptionEntry) states.get( i );
        
            if( csde.getCS().compareTo( cs ) == 0 )
            {
                //  ask for delete
                if( csde.getModifier().trim().length() > 0 ||
                    csde.getNotes().trim().length() > 0 ||
                    csde.getText().trim().length() > 0 
                  )
                {
                    int ret = net.metadiversity.diversity.navigator.gui.DnOptionPane.showConfirmDialog( null, "Delete this entry" );
                    if( ret == 0 )
                    {    
                        // states.remove( csde );
                        removeCSDE( csde );
                        return true;
                    }
                    return false;
                }
                else
                {
                    // states.remove( csde );                    
                    removeCSDE( csde );                    
                }
                break;
            }
        }
        return true;
    }
    //--------------------------------------------------------------------------    
    //--------------------------------------------------------------------------
    private void removeCSDE( CharacterStateDescriptionEntry csde )
    {
        // remove from Database
        
        // remove from GUI
        states.remove( csde );
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int getColumnCount() 
    {
        if( onlyUseOneNotesTextFieldFlag )
            return 4;
        else
            return 5;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int getRowCount() 
    {
        if( states == null )
            return 0;
        return states.size();
    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public Object getValueAt( int row, int col ) 
    {
        if( onlyUseOneNotesTextFieldFlag )
        {
            switch( col ) 
            {
                case CS :       return (( CharacterStateDescriptionEntry ) states.get( row ) ).getCS();
                case MODIFIER : return (( CharacterStateDescriptionEntry ) states.get( row ) ).getModifier();
                case X :        if( useNumbersAsText )
                                {
                                    double val = (( CharacterStateDescriptionEntry ) states.get( row ) ).getX();
                                    
                                    java.text.NumberFormat nf = java.text.NumberFormat.getInstance();                                    
                                    if( nf instanceof java.text.DecimalFormat) 
                                    {
                                        ((java.text.DecimalFormat) nf).setDecimalSeparatorAlwaysShown( false );
                                    }
                                    String str = nf.format( val );
                                    return str;
                                }
                                return new Double( (( CharacterStateDescriptionEntry ) states.get( row ) ).getX() );
                                // ??? comments
                case TEXT :     
                case NOTES :    
                                CharacterStateDescriptionEntry csde = ( CharacterStateDescriptionEntry ) states.get( row );
                                if( csde.isTextElement() )
                                {
                                    return csde.getText();
                                }
                                else
                                {
                                    return csde.getNotes();
                                }
                default:        return null;
            }            
        }
        else
        {    
            switch( col ) 
            {
                case CS :       return (( CharacterStateDescriptionEntry ) states.get( row ) ).getCS();
                case MODIFIER : return (( CharacterStateDescriptionEntry ) states.get( row ) ).getModifier();
                case X :        if( useNumbersAsText )
                                {
                                    double val = (( CharacterStateDescriptionEntry ) states.get( row ) ).getX();
                                    
                                    java.text.NumberFormat nf = java.text.NumberFormat.getInstance();                                    
                                    if( nf instanceof java.text.DecimalFormat) 
                                    {
                                        ((java.text.DecimalFormat) nf).setDecimalSeparatorAlwaysShown( false );
                                    }
                                    String str = nf.format( val );
                                    return str;
                                }
                                return new Double( (( CharacterStateDescriptionEntry ) states.get( row ) ).getX() );
                case TEXT :     return (( CharacterStateDescriptionEntry ) states.get( row ) ).getText();
                case NOTES :    return (( CharacterStateDescriptionEntry ) states.get( row ) ).getNotes();
                default:        return null;
            }
        }
    }
    //------------------------------------------------------------------
    public Class getColumnClass( int columnIndex ) 
    {
        switch( columnIndex )
        {
            case CS :       return String.class;
            case MODIFIER : return String.class;
            case X :        if( useNumbersAsText )
                                return String.class;
                            else
                                return Double.class;
            case TEXT :     return String.class;
            case NOTES :    return String.class;
            default:        return null;
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getColumnName( int col ) 
    {
        if( onlyUseOneNotesTextFieldFlag )
        {
            switch( col )
            {
                case CS :       return "Character State";
                case MODIFIER : return "Modifier";
                case X :        return "X";
                case TEXT :     return "Comments";
                default:        return null;
            }
        }
        else
        {
            switch( col )
            {
                case CS :       return "Character State";
                case MODIFIER : return "Modifier";
                case X :        return "X";
                case TEXT :     return "Text";
                case NOTES :    return "Notes";
                default:        return null;
            }
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public boolean isCellEditable( int row, int col ) 
    {
        if( col == 0 )
            return false;
        else
            return true;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setValueAt( Object obj, int row, int col ) 
    {
        logger.debug( "set value at ROW : " + row  + " COL: " + col + " OBJ: " + obj ); 

        CharacterStateDescriptionEntry csde = (CharacterStateDescriptionEntry) states.get( row );
        if( onlyUseOneNotesTextFieldFlag )
        {
            switch( col )
            {
                case CS :       break;
                case MODIFIER : csde.setModifier( (String) obj ); break;
                case X :        if( useNumbersAsText )
                                {
                                    java.text.NumberFormat nf = java.text.NumberFormat.getInstance();                                    
                                    if( nf instanceof java.text.DecimalFormat) 
                                    {
                                        ((java.text.DecimalFormat) nf).setDecimalSeparatorAlwaysShown( false );
                                    }
                                    try
                                    {
                                        csde.setX( nf.parse( (String) obj ).doubleValue() ); 
                                    }
                                    catch( java.text.ParseException ex )
                                    {
                                        String msg = "Invalid number format";
                                        logger.error( msg );
                                        DnOptionPane.showMessageDialog( null, msg, "Error" , DnOptionPane.ERROR_MESSAGE );                                     
                                    }
                                }
                                else
                                {
                                    csde.setX( ((Double) obj).doubleValue() );
                                }
                                break;
                                // ??? comments
                case TEXT :     
                case NOTES :    
                                if( csde.isTextElement() )
                                {                    
                                    csde.setText( (String) obj ); 
                                }
                                else
                                {
                                    csde.setNotes( (String) obj );
                                }
                                break;                                   
                default:        break;
            }
        }
        else
        {
            switch( col )
            {
                case CS :       break;
                case MODIFIER : csde.setModifier( (String) obj ); break;
                case X :        if( useNumbersAsText )
                                {
                                    java.text.NumberFormat nf = java.text.NumberFormat.getInstance();                                    
                                    if( nf instanceof java.text.DecimalFormat) 
                                    {
                                        ((java.text.DecimalFormat) nf).setDecimalSeparatorAlwaysShown( false );
                                    }
                                    try
                                    {
                                        csde.setX( nf.parse( (String) obj ).doubleValue() ); 
                                    }
                                    catch( java.text.ParseException ex )
                                    {
                                        logger.error( "Invalid number format" );
                                    }
                                }
                                else
                                {
                                    csde.setX( ((Double) obj).doubleValue() );
                                }
                                break;
                case TEXT :     csde.setText( (String) obj ); break;
                case NOTES :    csde.setNotes( (String) obj ); break;
                default:        break;
            }
        }
        deltaEditorChangeEvent = new DeltaEditorChangeEvent( this, DeltaEditorChangeEvent.UPDATE_CHARACTER_DESCRIPTION_STATE, row, col );            
        fireChangeListener();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------    
    public void addTableModelListener(javax.swing.event.TableModelListener tableModelListener) 
    {
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void removeTableModelListener(javax.swing.event.TableModelListener tableModelListener) 
    {
    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------                
    public CharacterStateDescriptionEntry getCharacterStateDescriptionEntry( int i )
    {
        return (CharacterStateDescriptionEntry) states.get( i );
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void addDeltaEditorChangeListener( DeltaEditorChangeListener listener ) 
    {
        listenerList.add( DeltaEditorChangeListener.class , listener );
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void removeDeltaEditorChangeListener( DeltaEditorChangeListener listener ) 
    {
        listenerList.remove( DeltaEditorChangeListener.class , listener );
    }
    //--------------------------------------------------------------------------
    // Notify all listeners that have registered interest for
    // notification on this event type.  The event instance 
    // is lazily created using the parameters passed into 
    // the fire method.
    //--------------------------------------------------------------------------
    protected void fireChangeListener() 
    {
        // Guaranteed to return a non-null array
        Object[] listeners = listenerList.getListenerList();
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for( int i = listeners.length-2; i >= 0; i -= 2 ) 
        {
            if( listeners[i] == DeltaEditorChangeListener.class ) 
            {
                // Lazily create the event:
                if( deltaEditorChangeEvent == null )
                {
                    deltaEditorChangeEvent = new DeltaEditorChangeEvent( this );
                }
                (( DeltaEditorChangeListener ) listeners[ i + 1 ] ).stateChanged( deltaEditorChangeEvent );
            }
        }
    }    
}
