/*
 * UbtCollectionProjectPlugin.java
 *
 * Created on 18. November 2003, 16:15
 */

package net.metadiversity.diversity.navigator.projectPlugin;

import org.apache.log4j.Logger;

import net.metadiversity.diversity.navigator.connection.ConnectionInfo;
//import net.metadiversity.diversity.navigator.gui.ProjectQueryPanel;
import net.metadiversity.diversity.navigator.gui.CollectionTransectGisVisualisation;
import net.metadiversity.diversity.navigator.gui.DnOptionPane;
import net.metadiversity.diversity.navigator.gui.delta_editor.DeltaEditor;
import net.metadiversity.diversity.navikey.ui.NaviKey;

/**
 *
 * @author  Dieter Neubacher
 */
public class UbtCollectionProjectPlugin extends Plugin
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3256718477052949556L;
    static Logger logger = Logger.getLogger( UbtCollectionProjectPlugin.class.getName() );
    
    /** Creates a new instance of UbtCollectionProjectPlugin */
    public UbtCollectionProjectPlugin()
    {
        super( "BIOTA S04 Lichens", "Diversity Workbench" );

        String collData = "BIOTA Lichen Collections";
        String descData = "BIOTA Lichen Descriptions";
        
        addProjectDatabase( null, collData );
        addProjectDatabase( null, descData );
        
        java.sql.Connection conDesc = null;
        java.sql.Connection conColl = null;
        
        String diversity_accepted_names_cache_database = "Diversity AcceptedNames Cache";
        addProjectDatabase( null, diversity_accepted_names_cache_database );

        java.sql.Connection connectionAcceptedNames = null;        
        ConnectionInfo ciDesc = null;        
        ConnectionInfo ciColl = null;        
        ConnectionInfo ciAccp = null;        
        try
        {
            ciDesc = net.metadiversity.diversity.navigator.connection.ConnectionsManager.getConnectionInfoByName( descData );
            conDesc = ciDesc.getConnection();
            logger.info( "Connect to: " + ciDesc.getUrl() );
            
            ciColl = net.metadiversity.diversity.navigator.connection.ConnectionsManager.getConnectionInfoByName( collData );
            conColl = ciColl.getConnection();
            logger.info( "Connect to: " + ciColl.getUrl() );            

            ciAccp = net.metadiversity.diversity.navigator.connection.ConnectionsManager.getConnectionInfoByName( diversity_accepted_names_cache_database );
            logger.info( "Connect to: " + ciAccp.getUrl() );
            connectionAcceptedNames = ciAccp.getConnection();
            
        }
        catch( java.sql.SQLException ex )
        {
             logger.error( ex.getMessage(), ex.fillInStackTrace() );
             DnOptionPane.showMessageDialog( null, ex.getMessage(), "Error" , DnOptionPane.ERROR_MESSAGE );
             return;             
        }
        catch( Exception ex )
        {
             logger.error( ex.getMessage(), ex.fillInStackTrace() );
             DnOptionPane.showMessageDialog( null, ex.getMessage(), "Error" , DnOptionPane.ERROR_MESSAGE );
             return;             
        }
        if( conColl == null )
        {
            DnOptionPane.showMessageDialog( null, "can't connect to " + ciColl.getUrl(), "Error" , DnOptionPane.ERROR_MESSAGE );
            return;
        }
        if( conDesc == null )
        {
            DnOptionPane.showMessageDialog( null, "can't connect to " + ciDesc.getUrl(), "Error" , DnOptionPane.ERROR_MESSAGE );
            return;
        }
        String tablePrefix = "BIOTAlichendescr";
                
        if( ciDesc.getDatabaseName().indexOf( "BIOTAlichendescr" ) >= 0 )
        {
            tablePrefix =  "BIOTAlichendescr"; 
        }
        else if( ciDesc.getDatabaseName().indexOf( "LIASlight" ) >= 0 )
        {
            tablePrefix =  "LIASlight"; 
        }
        else if( ciDesc.getDatabaseName().indexOf( "BIOTA_S04" ) >= 0 )
        {
            tablePrefix =  "BIOTA_S04"; // ??? No Column StateID  in CS       
        }
        
        setMapPanel( new CollectionTransectGisVisualisation( conDesc, conColl, tablePrefix ) );
        
        net.metadiversity.diversity.navigator.db.ub.UbtCollectionMainFrame dataForm = new net.metadiversity.diversity.navigator.db.ub.UbtCollectionMainFrame( ciColl );
        setDataFormPanel( dataForm );

        if( connectionAcceptedNames == null )
        {
             DnOptionPane.DnShowErrorDialog( "Can't connect to AcceptedNamesCache database");
             return;
        }
        // setup DeltaEditor for description database
        
        logger.info( "init DeltaEditor" );
        DeltaEditor p = new DeltaEditor( conDesc, null, connectionAcceptedNames, tablePrefix, true );
        this.setGridViewPanel( p );                

        // setup NaviKey
        
        boolean withNaviKeyFlag = true;
        
        if( withNaviKeyFlag )
        {
            String specs = p.getSpecsString();
            String chars = p.getCharsString();
            String items = p.getItemsString();

            logger.info( "init Navikey" );
            NaviKey navikey = new NaviKey( (java.awt.Container) getIdentificationPanel(), specs, chars, items );
            navikey.init();

            this.pack();
            this.setVisible( true );
        }        
    }
}
