/**
 * InfoFrame.java
 *  Displays information about the current version of NaviKey
 * @author Michael Bartley
 * @date 7/1/99
 * @author Dieter Neubacher
 */

package net.metadiversity.diversity.navikey.ui;

public class InfoFrame extends javax.swing.JFrame
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3258409547327419444L;
    private javax.swing.JButton closebutton;

    private String VERSION = "v. 4.06";
    private String BUILD_DATE = "2006-05-22";
    private String FONT_COLOR = "000000";
    
    public InfoFrame()
    {
        super( "About NaviKey v. 4.06" );
        
        getContentPane().setLayout( new java.awt.BorderLayout() );
        
        closebutton = new javax.swing.JButton( "Close" );
        closebutton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                closeButtonActionPerformed( evt );
            }
        });
        
        getContentPane().add( java.awt.BorderLayout.SOUTH, closebutton );
                
        String info = "<html>" +
/* !!! Didi: Can't change the font-family. !!!         
*        			  "<span style=\"font-family:'Times New Roman'\">" + 
*            		  "<b><h1>NaviKey " + VERSION + "</h1></b> " +
*                      "</span>" +
*
*       			  "<span style=\"font-family:'Arial'\">" + 
*
* Working:
*         			 "<p style='font-family:Arial; font-size:15px; color:#ff0000;'>1. Zeile<br>2. Zeile</p>" +
*/                      
        			 "<body style='font-family:Arial; font-size:12px; color:#000000;'>" +
        
            		  "<b><h1>NaviKey " + VERSION + "</h1></b> " +
                      "(<a href=\"http://www.gnu.org/licenses/licenses.html\">" + 
                      "GNU General Public Licence<a>)" +
                     "</p>" + 
         			 "<p>" +
                     BUILD_DATE + 
                     "</p>" + 
         			 "<p>" +
                      "This software is based on an earlier version (NaviKey v. 2.3 by Michael Bartley and Noel Cross, Harvard University Herbarium, Boston, USA) and has been further developed in the frame of the " +
                      "BIOTA Africa (<u>http://www.biota-africa.de</u>) project by Dieter Neubacher and Gerhard Rambold (University of Bayreuth, Germany). Email any comments about the program to navikey@uni-bayreuth.de." + 
                      
                      "<p>NaviKey " + VERSION + " is a tool for querying descriptive data and has been implemented as" +
                      "<ul>" + 
                      "<li>a Java applet providing the contents of " +
                        "DELTA (<u>http://www.delta-intkey.com</u>) flat files;" + 
                      "<li>a stand-alone Java application for accessing " +
                        "DELTA flat files or " +
                        "<i>DiversityDescription</i> (<u>http://www.diversitycampus.net/Workbench/download.html</u>) PostgreSQL databases, respectively;" +
                      "<li>a module (\"plug-in\") of the " +
                        "<i>Diversity Navigator</i> (<u>http://www.diversitynavigator.net</u>) Java database client for accessing " +
                        "DELTA flat files or " +
                        "<i>DiversityDescription</i> PostgreSQL databases, respectively." +
                      "</ul>" +
                      "Regular version updates of the NaviKey applet and stand-alone NaviKey application are provided on the " +
                        "NaviKey Home side (<u>http://www.navikey.net</u>)" +
                      "<p>" +    
                      "The user interface contains four main panels:" +
                      "<ul>" + 
                      "<li>top left - Character panel: selection of the characters;" + 
                      "<li>top right - Character states or numeric data panel: preselection of states of the selected character (to finalize the query, the &lt;Select&gt; button is to be used);" + 
                      "<li>bottom left - Query criteria panel: display of previous character state selections;" + 
                      "<li>bottom right - Matching items panel: display of matching items (selecting any item will bring up a complete description of that item).</ul>" + 
                      "<p>NaviKey checkbox matching options are:<ul>" + 
                      "<li><b>Use \"Best\" algorithm</b><br>To be implemented.</li>" + 
                      "<li><b>Use character dependencies</b><br>To be implemented.</li>" +
                      "<li><b>Restrict view on used characters and character states of remaining items</b>" +
                       "<br>When checked, NaviKey only displays those characters and states (in the two top main panels) that will reduce the number of item matches; characters or states which are not applicable to the remaining items (in the bottom right main panel) or apply to all of the items in the same form are no longer displayed. If a character is selected, for which more than one state applies, multiple state selection may be performed using the &lt;Ctrl&gt; (or corresponding) button (see also \"Enable multiple selection of character states\"). " +
                       "In the case of numeric characters, the top right panel provides fields for input of numeric values. After the input of either a single measurement or a range of values, the selection of the &lt;Add&gt; button will finalize the query.</li>" +

                // "<li><b>Exclude items with undefined states in selected characters</b>" +
                "<li><b>Retain items unrecorded for the selected characters</b>" +

                      "<br>Checking the box enables the \"Direct identification\" mode with the undefined states included; i.e., NaviKey also displays those items (in the bottom right panel), for which the selected character(s) have (so far) not been specified in the database. When unchecked, it only displays the items with at least one of the selected characters being specified. This behaviour largely corresponds with the \"analysis/data retrieval mode\" in CSIRO-DELTA Intkey and in the Identify module of <i>DiversityDescriptions</i></li>" +

                // "<li><b>Enable multiple selection of character states (with OR)</b>" +
                "<li><b>Retain items matching at least one selected state of resp. characters</b>" +
                      
                      "<br>When checked, NaviKey selects those items matching by at least one character state of the selected characters, otherwise only those items matching directly the selected combination of character states.</li>" +
                
                      "<li><b>Use extreme interval validation</b><br>When checked, the whole range between the extreme values (if present in the item data set) is used in numerical queries, otherwise only the range between minimum and maximum values (if present in the item data set) is considered.</li>" +
                      "<li><b>Use overlapping interval validation</b>" +
                       "<br>When checked, overlapping between queried and the specified numerical intervals is matching, otherwise not.</li></ul>" +
                       "</p>" + 
                      "</body>" + 
                      "</html>";  
        
        javax.swing.JEditorPane ta;
        ta = new javax.swing.JEditorPane( "text/html", info );
        ta.setEditable( false );
        ta.setMargin( new java.awt.Insets( 10,10,10,10) );
        ta.setCaretPosition( 0 ); // Moveto first Position
        javax.swing.JScrollPane sp = new javax.swing.JScrollPane( ta );
        sp.setHorizontalScrollBarPolicy( javax.swing.JScrollPane.HORIZONTAL_SCROLLBAR_NEVER );
        getContentPane().add( java.awt.BorderLayout.CENTER, sp );
    }
    
    private void closeButtonActionPerformed( java.awt.event.ActionEvent evt )
    {
        setVisible( false );
        dispose();
    }
}
