/*
 * ImportExportOptionPanel.java
 *
 * Created on 12. September 2003, 14:03
 */

package net.metadiversity.diversity.navigator.options;

import org.apache.log4j.Logger;
/**
 *
 * @author  Dieter Neubacher
 */
public class ImportExportOptionPanel extends javax.swing.JPanel
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 4121417315025040439L;

    static Logger logger = Logger.getLogger( ImportExportOptionPanel.class.getName() );
  
    // posible encodings are:
    // java.nio.charset.Charset.availableCharsets()
    //
    // "US-ASCII"         "Seven-bit ASCII, a.k.a. ISO646-US, a.k.a. the Basic Latin block of the Unicode character set"
    // "ISO-8859-1"       "ISO Latin Alphabet No. 1, a.k.a. ISO-LATIN-1"
    // "UTF-8"            "Eight-bit Unicode Transformation Format"
    // "UTF-16BE"         "Sixteen-bit Unicode Transformation Format, big-endian byte order"
    // "UTF-16LE"         "Sixteen-bit Unicode Transformation Format, little-endian byte order"
    // "UTF-16"           "Sixteen-bit Unicode Transformation Format, byte order specified by a mandatory initial byte-order mark (either order accepted on input, big-endian used on output)"

    final String encodings [] = { "UTF-8", "UTF-16", "UTF-16BE", "UTF-16LE", "US-ASCII", "ISO-8859-1" };
    final String seperators[] = { ";", "TAB", "" };
    final String stringDelimiters [] = { "\"", "" };
    
    // use ComboBoxModel's
    javax.swing.DefaultComboBoxModel encodingModel;
    javax.swing.DefaultComboBoxModel seperatorModel;
    javax.swing.DefaultComboBoxModel stringDelimiterModel;
    
    // selected values
    
    String selectedSeperator;
    String selectedDelimiter;
    String selectedEncoding;
    
    
    /** Creates new form ImportExportOptionPanel */
    public ImportExportOptionPanel()
    {
        encodingModel = new javax.swing.DefaultComboBoxModel( encodings );
        seperatorModel = new javax.swing.DefaultComboBoxModel( seperators );
        stringDelimiterModel = new javax.swing.DefaultComboBoxModel( stringDelimiters );
        
        // init default settings
        selectedSeperator = (String) seperatorModel.getElementAt( 0 );
        selectedDelimiter = (String) stringDelimiterModel.getElementAt( 0 );
        selectedEncoding  = (String) encodingModel.getElementAt( 0 );
        
        initComponents();
    }
    
    /**
     * @return selected seperator
     *
     */    
    public String getSeperator()
    {
        return selectedSeperator;
    }
    /**
     * @return String delimiter
     */    
    public String getStringDelimiter()
    {
        return selectedDelimiter;
    }
    /**
     * @return selected encoding
     *
     */    
    public String getEncoding()
    {
        return selectedEncoding;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        jLabelHeadLine = new javax.swing.JLabel();
        jPanelOptions = new javax.swing.JPanel();
        jLabelSeperator = new javax.swing.JLabel();
        jComboBoxSeperator = new javax.swing.JComboBox();
        jLabelStringDelimiter = new javax.swing.JLabel();
        jComboBoxStringDelimiter = new javax.swing.JComboBox();
        jLabelFileEncoding = new javax.swing.JLabel();
        jComboBoxFileEncoding = new javax.swing.JComboBox();
        jPanelButtons = new javax.swing.JPanel();
        jButtonOk = new javax.swing.JButton();
        jButtonAccept = new javax.swing.JButton();
        jButtonCancel = new javax.swing.JButton();

        setLayout(new java.awt.BorderLayout());

        jLabelHeadLine.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelHeadLine.setText("Import / Export Options");
        add(jLabelHeadLine, java.awt.BorderLayout.NORTH);

        jPanelOptions.setLayout(new java.awt.GridLayout(3, 2));

        jLabelSeperator.setText("Seperator: ");
        jPanelOptions.add(jLabelSeperator);

        jComboBoxSeperator.setEditable(true);
        jComboBoxSeperator.setModel( seperatorModel );
        jPanelOptions.add(jComboBoxSeperator);

        jLabelStringDelimiter.setText("String delimiter:");
        jPanelOptions.add(jLabelStringDelimiter);

        jComboBoxStringDelimiter.setEditable(true);
        jComboBoxStringDelimiter.setModel( stringDelimiterModel );
        jPanelOptions.add(jComboBoxStringDelimiter);

        jLabelFileEncoding.setText("File encoding: ");
        jPanelOptions.add(jLabelFileEncoding);

        jComboBoxFileEncoding.setModel(encodingModel);
        jPanelOptions.add(jComboBoxFileEncoding);

        add(jPanelOptions, java.awt.BorderLayout.CENTER);

        jButtonOk.setMnemonic('O');
        jButtonOk.setText("Ok");
        jButtonOk.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                jButtonOkActionPerformed(evt);
            }
        });

        jPanelButtons.add(jButtonOk);

        jButtonAccept.setMnemonic('A');
        jButtonAccept.setText("Accept");
        jButtonAccept.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                jButtonAcceptActionPerformed(evt);
            }
        });

        jPanelButtons.add(jButtonAccept);

        jButtonCancel.setMnemonic('C');
        jButtonCancel.setText("Cancel");
        jButtonCancel.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                jButtonCancelActionPerformed(evt);
            }
        });

        jPanelButtons.add(jButtonCancel);

        add(jPanelButtons, java.awt.BorderLayout.SOUTH);

    }//GEN-END:initComponents

    private void jButtonCancelActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jButtonCancelActionPerformed
    {//GEN-HEADEREND:event_jButtonCancelActionPerformed
        // set default values
        seperatorModel.setSelectedItem( seperatorModel.getElementAt( 0 ) );
        stringDelimiterModel.setSelectedItem( stringDelimiterModel.getElementAt( 0 ) );
        encodingModel.setSelectedItem( encodingModel.getElementAt( 0 ) );
        
        setVisible( false );
        
    }//GEN-LAST:event_jButtonCancelActionPerformed

    private void jButtonAcceptActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jButtonAcceptActionPerformed
    {//GEN-HEADEREND:event_jButtonAcceptActionPerformed
        // set selection as default
        selectedSeperator = (String) seperatorModel.getSelectedItem();
        selectedDelimiter = (String) stringDelimiterModel.getSelectedItem();
        selectedEncoding  = (String) encodingModel.getSelectedItem();

        setVisible( false );
        
    }//GEN-LAST:event_jButtonAcceptActionPerformed

    private void jButtonOkActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_jButtonOkActionPerformed
    {//GEN-HEADEREND:event_jButtonOkActionPerformed
        selectedSeperator = (String) seperatorModel.getSelectedItem();
        selectedDelimiter = (String) stringDelimiterModel.getSelectedItem();
        selectedEncoding  = (String) encodingModel.getSelectedItem();
        
        logger.debug( "Seperator: " + selectedSeperator + " Delimiter: " + selectedDelimiter + " Encoding: " + selectedEncoding );

        setVisible( false );
        
    }//GEN-LAST:event_jButtonOkActionPerformed
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[])
    {
        javax.swing.JFrame frame = new javax.swing.JFrame();
        ImportExportOptionPanel o = new ImportExportOptionPanel();        
        o.setVisible( true );
        frame.getContentPane().add( o );
        // frame.setSize( new java.awt.Dimension( 200, 200 ) );
        frame.pack();
        frame.setVisible( true );
        
        System.out.println( "select options" );
        
        System.out.println( o.getSeperator() );
        System.out.println( o.getStringDelimiter() );
        System.out.println( o.getEncoding() );        
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabelHeadLine;
    private javax.swing.JButton jButtonCancel;
    private javax.swing.JLabel jLabelFileEncoding;
    private javax.swing.JPanel jPanelOptions;
    private javax.swing.JComboBox jComboBoxStringDelimiter;
    private javax.swing.JLabel jLabelStringDelimiter;
    private javax.swing.JComboBox jComboBoxSeperator;
    private javax.swing.JComboBox jComboBoxFileEncoding;
    private javax.swing.JButton jButtonOk;
    private javax.swing.JPanel jPanelButtons;
    private javax.swing.JButton jButtonAccept;
    private javax.swing.JLabel jLabelSeperator;
    // End of variables declaration//GEN-END:variables
        
}
