/*
 * CollectionObservatory.java
 *
 * Created on 24. September 2003, 17:08
 */

package net.metadiversity.diversity.navigator.util;


// import java.lang.*;
// import java.lang.Math.*;

// import java.util.*;
// import javax.xml.parsers.*;

// import org.xml.sax.*;
// import org.xml.sax.helpers.*;
// import org.w3c.dom.*;

// import java.io.File;

import org.apache.log4j.Logger;

/**
 *
 * @author  Dieter Neubacher
 */
public class CollectionObservatory
{
    static Logger logger = Logger.getLogger( CollectionObservatory.class.getName() );
    
    final static int GRAD   = 0;
    final static int MINUTE = 1;
    final static int SECOND = 2;
    
    // observatories search mode for a position
    int observatorySearchMode;
    final static int INSITE_OBSERVATORY   = 0;
    final static int CLOSEST_OBSERVATORY  = 1;
    
    // SOUTH, EAST coordinates     
    java.util.Vector ObservatoryData;
    // Observatory size, wee look for a square with [ distance - pos, pos + distance ]
    // for both directions.
    double distance;

    static double earthRadiusSouthNorth = 6356.775; // [km]
    static double earthRadiusEastWest   = 6378.137; // °quator [km]
    
    // read Observatory definition from config-file 
    static String configFileName;
    final static boolean USE_XML_CONFIG_FILE = true;
    //--------------------------------------------------------------------------
    // internal class for Observatory data.
    //--------------------------------------------------------------------------
    public class Observatory    
    {
        String name;
        double south;
        double east;
        // ??? String referencePoint;
        String description; 
        
        // Reference Point
        final static int Center     = 0;
        final static int NorthWest  = 1;
        final static int NorthEast  = 2;
        final static int SouthEast  = 3;
        final static int SouthWest  = 4;                
        
        public Observatory( String name, double south, double east )
        {
            this.name   = name;
            this.south  = south;
            this.east   = east;
            description = "";
        }
        public double getSouthPos()
        {
            return south;
        }
        public double getEastPos()
        {
            return east;
        }
        public java.lang.String getName()
        {
            return name;
        }
        public java.lang.String getDescription()
        {
            return description;
        }
        public java.lang.String getReferencePoint()
        {
            return name;
        }
    }
    //--------------------------------------------------------------------------
    // Constructor: Creates a new instance of CollectionObservatory
    //--------------------------------------------------------------------------
    public CollectionObservatory()
    {        
        // default distance
        distance = 16.1 / (360 * 60 * 60); // --> size = 1 KM
        observatorySearchMode = CLOSEST_OBSERVATORY;
                
        ObservatoryData = new java.util.Vector();
        
        if( USE_XML_CONFIG_FILE )
        {
            // read file from user.home directory
            String path = System.getProperty( "user.home" );
            configFileName = path + "/.diversity_navigator/diversity_navigator.collection.observatories.xml";

            // get XML factory
            try            
            {
                /*
                <Observatory>
                    <EastPos>17° 50' 56.5" E</EastPos>
                    <SouthPos>25° 20' 32.8" S</SouthPos>
                    <!-- ??? -->
                    <Corner>(NorthWest corner of the Northern Camp and NorthEast corner of the Southern Camp) (die 2 Obs. liegen bei einander)</Corner>
                    <Name>(08/09)</Name>
                    <Description>Namibia, Niko 377 (Niko North)</Description>
                </Observatory>
                */
                
                javax.xml.parsers.DocumentBuilderFactory factory = javax.xml.parsers.DocumentBuilderFactory.newInstance();
// ???                factory.setValidating( true );        // enable Validating with DTD
                javax.xml.parsers.DocumentBuilder docBuilder = factory.newDocumentBuilder();
                docBuilder.setErrorHandler( new XmlSaxErrorHandler() ); 
                org.w3c.dom.Document doc = docBuilder.parse( configFileName );
                org.w3c.dom.Element root = doc.getDocumentElement();
                org.w3c.dom.NodeList nodeList = root.getElementsByTagName( "Observatory" );
                for( int i = 0; i < nodeList.getLength(); i++ ) 
                {
                    org.w3c.dom.Element connNode = ( org.w3c.dom.Element ) nodeList.item( i );
                    
                    String eastPos = "";
                    String southPos = "";
                    String referencePoint = "";
                    String name = "";
                    String description = "";
                    
                    org.w3c.dom.Node node;
                    
                    eastPos         = ( (org.w3c.dom.Element) connNode.getElementsByTagName( "EastPos"  ).item(0)).getFirstChild().getNodeValue();
                    southPos        = ( (org.w3c.dom.Element) connNode.getElementsByTagName( "SouthPos" ).item(0)).getFirstChild().getNodeValue();
                    referencePoint  = ( (org.w3c.dom.Element) connNode.getElementsByTagName( "ReferencePoint").item(0)).getFirstChild().getNodeValue();
                    name            = ( (org.w3c.dom.Element) connNode.getElementsByTagName( "Name"     ).item(0)).getFirstChild().getNodeValue();
                    // description is maybe empty
                    description = "";
                    org.w3c.dom.Element e = (org.w3c.dom.Element) connNode.getElementsByTagName( "Description" ).item(0);
                    if( e != null )
                    {    
                        node  = e.getFirstChild();
                        if( node != null )
                        {   
                            description = node.getNodeValue();
                        }
                    }    
                    // set data
                    ObservatoryData.add( new Observatory( name,   getPos( southPos ), getPos( eastPos ) ) );
                }
            }
            catch( Exception ex )
            {
                logger.error( ex.getMessage(), ex.fillInStackTrace() );
            }
        }
        else
        {
            // init Observatory data
            ObservatoryData.add( new Observatory( "01",   getPos( "18° 18' 06.4''S" ), getPos( "19° 15' 24.6''E" ) ) );
            ObservatoryData.add( new Observatory( "02",   getPos( "18° 18' 06.4''S" ), getPos( "19° 15' 30.0'' E" ) ) );
            ObservatoryData.add( new Observatory( "03",   getPos( "19° 04' 58.3'' S" ), getPos( "18° 54' 48.3'' E" ) ) );
            ObservatoryData.add( new Observatory( "04",   getPos( "21° 30' 37.7''" ), getPos( "16° 43' 45.2''" ) ) );
            ObservatoryData.add( new Observatory( "05",   getPos( "21° 35' 48.7''" ), getPos( "16° 56' 41.2''" ) ) );
            ObservatoryData.add( new Observatory( "06",   getPos( "22° 01' 09.0\"S" ), getPos( "17° 03' 44.9\"E" ) ) );
            ObservatoryData.add( new Observatory( "08/09", getPos( "25° 20' 32.8''S" ), getPos( "17° 50' 56.5''" ) ) );
            ObservatoryData.add( new Observatory( "10" , getPos( "26° 24' 04.2''S" ), getPos( "18° 00' 17.5''E" ) ) );
            ObservatoryData.add( new Observatory( "11" , getPos( "26° 23' 26.31''S" ), getPos( "17° 59' 43.7''E" ) ) );
            ObservatoryData.add( new Observatory( "12/13/14/15" , getPos( "27° 41' 07.9''S" ), getPos( "17° 48' 5.3''E" ) ) );
            ObservatoryData.add( new Observatory( "16" , getPos( "22° 18' 54.0''S " ), getPos( "14° 28' 09.1'E" ) ) );
            ObservatoryData.add( new Observatory( "17" , getPos( "26° 45' 39.3''S" ), getPos( "20° 36' 50.5''E" ) ) );
            ObservatoryData.add( new Observatory( "18/19" , getPos( "28° 14' 08.4''S" ), getPos( "17° 01' 32.4''E" ) ) );
            ObservatoryData.add( new Observatory( "20" , getPos( "28° 18° 07.6'' S" ), getPos( "16° 57' 50.4'' E" ) ) );
            ObservatoryData.add( new Observatory( "21" , getPos( "28° 36'44.5\" S" ), getPos( "16° 39' 52.4'' E" ) ) );
            ObservatoryData.add( new Observatory( "22" , getPos( "30° 11' 43.18''S" ), getPos( "17° 33' 13.6'' E" ) ) );
            ObservatoryData.add( new Observatory( "24" , getPos( "30° 23' 41.1''S " ), getPos( "18° 17' 10.1'' E" ) ) );
            ObservatoryData.add( new Observatory( "25" , getPos( "30° 23' 43.4\" S" ), getPos( "18° 17' 32.5\"E" ) ) );
            ObservatoryData.add( new Observatory( "26" , getPos( "31° 17' 08.6\" S" ), getPos( "18° 36' 07.3\"E" ) ) );
            ObservatoryData.add( new Observatory( "27" , getPos( "31° 17' 09.2\" S" ), getPos( "18° 36' 01.3\"E" ) ) );
            ObservatoryData.add( new Observatory( "28" , getPos( "31° 27' 39.5\" S" ), getPos( "18° 26' 58.5\" E" ) ) );
            ObservatoryData.add( new Observatory( "29" , getPos( "32° 36' 03.1\" S" ), getPos( "18° 18' 20.0\" E" ) ) );
            ObservatoryData.add( new Observatory( "31" , getPos( "33° 29'21.6\" S" ), getPos( "18° 34' 48.4\"E" ) ) );
            ObservatoryData.add( new Observatory( "32" , getPos( "33° 25' 52.0\" S" ), getPos( "19° 01' 50.5\"E" ) ) );
            ObservatoryData.add( new Observatory( "33" , getPos( "34° 16' 09.2\" S" ), getPos( "18° 23' 32.0\"E" ) ) );
            // ObservatoryData.add( new Observatory( "" , getPos( "" ), getPos( "" ) ) );                
        }
        
    }
    //--------------------------------------------------------------------------
    // Set distance in GRAD
    //--------------------------------------------------------------------------
    public void setDistance( double distGrad )
    {        
        distance = distGrad;
    }
    //--------------------------------------------------------------------------
    double oneKmInSecundsNorthSouth( double southPosition )
    {
        // static double earthRadiusSouthNorth = 6356.775; // [km]
        // static double earthRadiusEastWest   = 6378.137; // °quator [km]
        return java.lang.Math.abs( 360.0 * 60.0 * 60.0 / ( earthRadiusSouthNorth * java.lang.Math.PI * 2.0 ) );
    }
    //--------------------------------------------------------------------------
    double oneKmInSecundsEastWest( double southPosition )
    {
        // radius at source Position
        double length = earthRadiusSouthNorth * java.lang.Math.PI * 2.0 *  java.lang.Math.cos( southPosition * java.lang.Math.PI / 180.0 );
        return java.lang.Math.abs( 360.0 * 60.0 * 60.0 / length );
    }
    //--------------------------------------------------------------------------
    // Format: gg mm ss
    //--------------------------------------------------------------------------    
    private double getPos( String coordinate ) throws java.lang.NumberFormatException
    {
        double grad     = 0.0;
        double minute   = 0.0;
        double second   = 0.0;        
        double sign     = 1.0;
        
        if( coordinate.indexOf( '-' ) >= 0 )
        {
            coordinate = coordinate.replace( '-', ' ' );
            sign = -1.0;
        }
        if( coordinate.indexOf( '+' ) >= 0 )
        {
            coordinate = coordinate.replace( '+', ' ' );
            sign = 1.0;
        }
        // Grad "ue" utf8 \uC2B0 , UTF 00B0
        char GRAD = '\u00B0';
        /*
        Character g = new  Character( 'ü' );        
        System.out.println( "GRAD: " + GRAD );
        System.out.println( coordinate );
         */
        logger.info( coordinate );
        java.util.StringTokenizer tokenizer = new java.util.StringTokenizer( coordinate, GRAD + "`?'\" \t", false );
        String token = null;
        if( tokenizer.hasMoreTokens() )
        {
            token = tokenizer.nextToken().trim();
            // System.out.println( token );
            grad = java.lang.Double.parseDouble( token );
        }    
        if( tokenizer.hasMoreTokens() )
        {
            token = tokenizer.nextToken().trim();
            // System.out.println( token );
            minute = java.lang.Double.parseDouble( token );
        }    
        if( tokenizer.hasMoreTokens() )
        {
            token = tokenizer.nextToken().trim();
            // System.out.println( token );
            second = java.lang.Double.parseDouble( token );
        }    
        
        return grad + minute / 60.0 + second / 3600.0;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public java.lang.String getObservatory( String southPos, String eastPos )
    {        
        // find Observatory for this position
        return getObservatory( getPos( southPos ), getPos( eastPos ) );
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public java.lang.String getObservatory( double southPos, double eastPos )
    {
        String observatory = "none";        
        // find Observatory for this position        
        switch( observatorySearchMode )
        {
            case INSITE_OBSERVATORY:                
            // Search for an observartory with includes the position
            for( int i = 0; i < ObservatoryData.size(); i++ )
            {
                Observatory obs = ( Observatory ) ObservatoryData.get( i );

                if( java.lang.Math.abs( southPos - obs.getSouthPos()) < distance && java.lang.Math.abs( eastPos - obs.getEastPos()) < distance )
                {                
                    return obs.getName(); // Observatory found
                }
            }         
            break;
            case CLOSEST_OBSERVATORY:
            // Search for the closest observartory for position
            double minDistanze = 10000.0;    
            for( int i = 0; i < ObservatoryData.size(); i++ )
            {
                Observatory obs = ( Observatory ) ObservatoryData.get( i );
                
                double dist = ( (southPos - obs.getSouthPos()) * (southPos - obs.getSouthPos()) );
                dist += ( (eastPos - obs.getEastPos()) * (eastPos - obs.getEastPos()) );
                dist = java.lang.Math.sqrt( dist );
                if( dist < minDistanze )                    
                {    
                    minDistanze = dist;           
                    observatory = obs.getName();
                }
            }                         
            break;
        }
        return observatory;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public java.lang.Object[] getObservatoryNames()
    {        
        // list fo all names
        java.util.Vector nameVector = new java.util.Vector();
        
        for( int i = 0; i < ObservatoryData.size(); i++ )
        {
            Observatory t = ( Observatory ) ObservatoryData.get( i );
            nameVector.add( t.name );
        }                
        return nameVector.toArray();
    }    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args)
    {
        // init
        
        CollectionObservatory ct = new CollectionObservatory();
        
        String pos;
        
        pos = "-17° 15` 20''";
        System.out.println( "Pos: " +  pos + " --> " + ct.getPos( pos ) );        
        
        pos = "+17° 15` 20''";
        System.out.println( "Pos: " +  pos + " --> " + ct.getPos( pos ) );        
        
        pos = "+ 17° 0` 2``";
        System.out.println( "Pos: " +  pos + " --> " + ct.getPos( pos ) );                
        // 26
        String t = ct.getObservatory( ct.getPos( "31° 17' 08.6\" S" ), ct.getPos( "18° 36' 07.3\"S" ) );
        System.out.println( "Observatory: " + t  );
        // 27
        t = ct.getObservatory( ct.getPos( "31° 17' 09.2\" S" ), ct.getPos( "18° 36' 01.3\"E" ) );        
        System.out.println( "Observatory: " + t  );        
        t = ct.getObservatory( ct.getPos( "30° 13' 25.1\"" ), ct.getPos( "17° 33' 18.8\"" ) );
        System.out.println( "Observatory: " + t  );        
        
        double p = 0.0;
        while( p < 90.0 )
        {
            System.out.println( "North:  " + p + " " + ct.oneKmInSecundsNorthSouth( p ) );
            System.out.println( "East:   " + p + " " + ct.oneKmInSecundsEastWest( p ) );
            p += 3.0;
            // System.out.println( java.lang.Math.cos( p * java.lang.Math.PI / 180.0 ) );
        }
    }
    
    
    // Errorhandler
    public static class XmlSaxErrorHandler implements org.xml.sax.ErrorHandler
    {        
        
        public void warning( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException 
        {
            // System.out.println( exception.toString() );
            logger.warn( buildMsg( ex ) );
        }
        public void error( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException 
        {
            // System.out.println( exception.toString() );
            logger.error( buildMsg( ex ) );
        }
        public void fatalError( org.xml.sax.SAXParseException ex ) throws org.xml.sax.SAXException
        {
            // System.out.println( exception.toString() );
            logger.fatal( buildMsg( ex ) );            
        }
        // format error message
        private String buildMsg( org.xml.sax.SAXParseException ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
            String msg = configFileName + "\n Line: " + ex.getLineNumber() + " Column: " + ex.getColumnNumber() + "\n" + ex.toString();            
            return msg;
        }
    }
    
}
