/*
 * Crypt.java
 *
 * Created on 21. M�rz 2003, 16:38
 */

package net.metadiversity.diversity.navigator.util;

import java.lang.String;
import javax.crypto.*;
//import java.security.*;
//import java.security.Key.*;

import org.apache.log4j.Logger;

/**
 *
 * @author  didi
 */
public class Crypt
{
    static Logger logger = Logger.getLogger( Crypt.class.getName() );
    
    KeyGenerator keygen;
    SecretKey desKey;
    Cipher desCipher;     // Create the cipher
    
    final static String code = "0123456789ABCDEF";
    final static String keyFileName = "/.diversity_navigator/diversity_navigator.key";
    
    /** Creates a new instance of Crypt */
    public Crypt()
    {        
        // To create a DES key, we have to instantiate a KeyGenerator for
        // DES. We do not specify a provider, because we do not care about a
        // particular DES key generation implementation. Since we do not
        // initialize the KeyGenerator, a system- provided source of randomness
        // will be used to create the DES key:

        try
        {            
            String path = System.getProperty( "user.home" );
            java.io.File file = new java.io.File( path + keyFileName );
            if( file.exists() )
            {
                // read Key from file
                java.io.FileInputStream fis = new java.io.FileInputStream( file );             
                java.io.ObjectInputStream ois = new java.io.ObjectInputStream(  fis );
                desKey = (SecretKey) ois.readObject();
                ois.close();             
            }
            else
            {
                // create an new key 
                keygen = KeyGenerator.getInstance("DES");
                desKey = keygen.generateKey(); 
                // and store this key into a file   
                java.io.FileOutputStream fos = new java.io.FileOutputStream( file );             
                java.io.ObjectOutputStream oos = new java.io.ObjectOutputStream(  fos );
                oos.writeObject( desKey );
                oos.close();             
            }
            desCipher = Cipher.getInstance( "DES/ECB/PKCS5Padding" );         
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getStackTrace().toString() );
            logger.error( ex.getLocalizedMessage() );
//            ex.printStackTrace();
        }   
    }

    public String encrypt( String str )
    {
        try
        {
            // We use the generated desKey from above to initialize
            // the Cipher object for encryption:
            // Initialize the cipher for encryption
            desCipher.init( Cipher.ENCRYPT_MODE, desKey );   // Our cleartext
            byte[] cleartext  = str.getBytes();              // Encrypt the cleartext
            byte[] ciphertext = desCipher.doFinal( cleartext );                 
            return toHexString( ciphertext );
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getStackTrace().toString() );
            logger.error( ex.getLocalizedMessage() );
//            ex.printStackTrace();
            return null;
        }   
    }
    
    private String toHexString( byte [] b )
    {
        StringBuffer output = new StringBuffer();        
        int     i = b.length;      
        int     pos = 0;
        int     val;
        while( i > 0 )
        {   // convert to HEX
            val = (int) b[ pos ];
            val = val & 0xF0;
            val =  val >> 4;
            output.append( code.charAt( val ) );
            val = (int) b[ pos++ ];
            val = val & 0x0F;            
            output.append( code.charAt( val ) );
            i--;
        }
        return output.toString();
    }
    //--------------------------------------------------------------------------
    private String hexToString( String s )
    {
        int len = s.length();
        byte [] b = new byte[ len / 2 ];
        int  ch;
        int  val;
        int pos = 0;
        for( int i = 0; i < len;  )
        {
            val = code.indexOf( s.charAt( i++ ) );
            ch = (byte) val;
            ch = ch << 4;
            val = code.indexOf( s.charAt( i++ ) );
            ch = ch | ( 0x0F & val );
            b[ pos++ ] = (byte) ch;
        }
        return new String( b );
    }
    //--------------------------------------------------------------------------
    private byte [] hexToByte( String s )
    {
        int len = s.length();
        byte [] b = new byte[ len / 2 ];
        int  ch;
        int  val;
        int pos = 0;
        for( int i = 0; i < len;  )
        {
            val = code.indexOf( s.charAt( i++ ) );
            ch = (byte) val;
            ch = ch << 4;
            val = code.indexOf( s.charAt( i++ ) );
            ch = ch | ( 0x0F & val );
            b[ pos++ ] = (byte) ch;
        }
        return b;
    }
    
    public String decrypt( String str )
    {
        try
        {
            // We use the generated desKey from above to initialize
            // the Cipher object for decryption:
            // Decrypt the ciphertext
            desCipher.init( Cipher.DECRYPT_MODE, desKey );
            /* Don't works well with all kindes of string (maybe character-coding)
            String sss = hexToString( str );
            byte[] ciphertext = sss.getBytes();
             */            
            byte[] ciphertext = hexToByte( str );
            byte[] cleartext  = desCipher.doFinal( ciphertext );         
            return new String( cleartext );  
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getStackTrace().toString() );
            logger.error( ex.getLocalizedMessage() );
//            ex.printStackTrace();
            return null;
        }   
    }
    
    public String decrypt( byte[] ciphertext )
    {
        try
        {
            // We use the generated desKey from above to initialize
            // the Cipher object for decryption:
            // Decrypt the ciphertext
            desCipher.init( Cipher.DECRYPT_MODE, desKey );
            byte[] cleartext  = desCipher.doFinal( ciphertext );         
            return new String( cleartext );  
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getStackTrace().toString() );
            logger.error( ex.getLocalizedMessage() );
//            ex.printStackTrace();
            return null;
        }   
    }        
    
    public void test()
    {
        try
        {

            // Initialize the cipher for encryption
            desCipher.init( Cipher.ENCRYPT_MODE, desKey );     // Our cleartext
            byte[] cleartext = "DiversityNavigator".getBytes();     // Encrypt the cleartext        
            System.out.println( new String( cleartext ) );
            
            // Initialize the same cipher for decryptCryption
            byte[] ciphertext = desCipher.doFinal( cleartext );                 
            System.out.println( new String( ciphertext ) );
            
            // Decrypt the ciphertext
            desCipher.init( Cipher.DECRYPT_MODE, desKey );     
            byte[] cleartext1 = desCipher.doFinal( ciphertext );         
            System.out.println( new String( cleartext ) );
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getStackTrace().toString() );
            logger.error( ex.getLocalizedMessage() );
//            ex.printStackTrace();
        }   
    }
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args)
    {
        Crypt c = new Crypt();
        // c.test();
        
        String in  = "Dieter";
        String out = c.encrypt( in );
        System.out.println( in + " --> " + out );
        in = c.decrypt( out );
        System.out.println( out + " --> " + in );        
        System.exit( 0 );
    }    
}
