package net.metadiversity.diversity.navikey.delta;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * <pre>
 * This class reads parts of a DELTA specs file as described in
 * Definition of the Delta Format
 * 14 March, 1995
 * Dallwitz & Paine
 *
 * Currently deals with:
 *   *CHARACTER TYPES
 *
 * Still need to parse:
 *  *IMPLICIT VALUES
 *  *DEPENDENT CHARACTERS
 *
 * The following can be calculated from the Chars and Items files:
 *  *NUMBER OF CHARACTERS
 *  *MAXIMUM NUMBER OF STATES
 *  *MAXIMUM NUMBER OF ITEMS
 *  *NUMBERS OF STATES
 *
 * Not sure what to do with:
 *  *SHOW <name>-- specifications. #TIME#DATE
 *  *SPECIAL STORAGE
 *  *MANDATORY CHARACTERS  (Not needed for reading purposes)
 * </pre>
 * @author Noel Cross
 * @author Michael Bartley
 */

public class SpecsFileReader
{
    StringBuffer    itemCharacterTypeLines;
    StringBuffer    itemCharacterDependencyLines;
    //
    Vector          itemCharacterTypes;
     
    String CHARACTER_TYPES          = "*CHARACTER TYPES";
    String DEPENDENT_CHARACTERS     = "*DEPENDENT CHARACTERS";
    
    public SpecsFileReader()
    {
        itemCharacterDependencyLines = new StringBuffer();
        itemCharacterTypeLines       = new StringBuffer();
        itemCharacterTypes           = new Vector();        
    }
    public Vector getItemCharacterTypes()
    {
        return itemCharacterTypes;
    }
    /**
     * Read in the file
     */
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void read( java.lang.String fileName )
    {
        java.io.BufferedReader bufferdReader = null;
        try
        {
            System.out.println( "SpecsFileReader(), read UTF-8 file: " + fileName );
            // bufferdReader = new java.io.BufferedReader( new java.io.FileReader( fileName ) );
            bufferdReader = new java.io.BufferedReader( new java.io.InputStreamReader( new java.io.FileInputStream( fileName ), "UTF-8" ) );            

            read( bufferdReader );
            bufferdReader.close();
        }
        catch( java.io.IOException ex )
        {
            System.out.println( fileName + " not found" );
            return;
        }        
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void read( java.io.BufferedReader specsReader )
    {
        read( specsReader, false, false );
    }
    void read( java.io.BufferedReader specsReader, boolean removeSpecsCommentFlag, boolean removeSpecsStatesCommentFlag )
    {
        //read in the file a line at a time
        String line = "";
        
        int count = 0;
        
        boolean newSectionFlag;
        boolean itemCharacterTypesFlag = false;
        boolean itemCharacterDependencyFlag = false;
        
        while( line != null )
        {
            count++;
            try
            {
                line = specsReader.readLine();
                if( line != null )
                {
                    line = line.trim();
                    if( line.indexOf( "*" ) == 0 )
                    {
                        newSectionFlag = true;
                        itemCharacterTypesFlag = false;
                    }
                    else
                    {
                        newSectionFlag = false;
                    }
                    //Descriptor definition type lines look like this:
                    //    *CHARACTER TYPES 1,TE 4,RN 6,RN
                    //    25,RN 27,RN 28,RN 29,RN
                    
                    //Check for itemCharacter type header
                    // if( line.indexOf( "*CHARACTER TYPES" ) == 0 )
                    if( line.indexOf( CHARACTER_TYPES ) == 0 )
                    {
                        itemCharacterTypesFlag = true;
                    }
                    if( itemCharacterTypesFlag )
                    {
                        itemCharacterTypeLines.append( " " ).append( line );
                    }
                    //Descriptor definition  dependency lines look like this:
                    //   *DEPENDENT CHARACTERS 5,2:6 20,2:21-22                    
                    //Check for itemCharacter dependency header
                    if( line.indexOf( DEPENDENT_CHARACTERS ) == 0 )
                    {
                        itemCharacterDependencyFlag = true;
                    }
                    if(itemCharacterDependencyFlag)
                    {
                        itemCharacterDependencyLines.append( " " ).append( line );
                    }                    
                }
            } 
            catch( IOException e )
            {
                //System.out.println("Error reading file " + fileName + "at line " + count);
                System.out.println(e);
            }
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void makeItemCharacterTypes()
    {        
        if( itemCharacterTypeLines == null )
            return;
        if( itemCharacterTypeLines.length() < CHARACTER_TYPES.length() )
            return;
       
        String allLines = itemCharacterTypeLines.substring( 17 ); // ???
        
        // System.out.println("allLines: " + allLines);
        
        //split the rest into itemCharacterId,itemCharacterType pairs
        StringTokenizer stAll = new StringTokenizer(allLines);
        while(stAll.hasMoreTokens())
        {
            String str = stAll.nextToken();
            //split up each pair into id, name values
            StringTokenizer st = new StringTokenizer(str, ",");
            String idStr = st.nextToken();
            String type = st.nextToken();
            //System.out.println("Id: " + idStr + " type: " + type);
            //hendle ranges
            StringTokenizer st2 = new StringTokenizer(idStr, "-");
            if(st2.countTokens() == 2)
            {
                Integer start = new Integer(st2.nextToken());
                Integer end = new Integer(st2.nextToken());
                System.out.println("Start:"+start+" End:"+end +" Type:"+type);
                for(int j=start.intValue(); j<=end.intValue(); j++)
                {
                    itemCharacterTypes.addElement( new ItemCharacterType( new Integer(j), type) );
                }
            }
            else
            {
                Integer id = new Integer(st2.nextToken());
                itemCharacterTypes.addElement( new ItemCharacterType(id, type));
            }
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    // ??? NOT USED
    public void __makeItemCharacterDependencies()
    {
        /*
        String allLines = new String();
        for(int i = 0; i < itemCharacterDependencyLines.size(); i++)
        {
            allLines = allLines + " " + ((String)itemCharacterDependencyLines.elementAt(i));
        }
        //chop off the "*DEPENDENT CHARACTERS" text
        allLines = allLines.substring(22, allLines.length());
        */
        String allLines = itemCharacterDependencyLines.substring( 22 ); // ???
        //System.out.println("allLines: " + allLines);
        
    /*
    //split the rest into itemCharacterId,itemCharacterDependency pairs
    StringTokenizer stAll = new StringTokenizer(allLines);
    while(stAll.hasMoreTokens()) {
      String str = stAll.nextToken();
      System.out.println("Unparsed String: " + str);
      //split up each pair into id, name values
      StringTokenizer st = new StringTokenizer(str, ",");
      String idStr = st.nextToken();
      String type = st.nextToken();
      //System.out.println("Id: " + idStr + " type: " + type);
      Integer id = new Integer(idStr);
      itemCharacterTypes.addElement(new ItemCharacterType(id, type));
    }
     */
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void print()
    {
        System.out.println("itemCharacter Types:");
        for(int i = 0; i < itemCharacterTypes.size(); i++)
        {
            ItemCharacterType ict = (ItemCharacterType)itemCharacterTypes.elementAt(i);
            ict.print();
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public static void main(String args[])
    {
        if(args.length != 1)
        {
            System.out.println("Usage: SpecsFileReader <specs file name>");
            System.exit(0);
        }
        SpecsFileReader sf = new SpecsFileReader();
        sf.read(args[0]);
        sf.makeItemCharacterTypes();
        //sf.makeItemCharacterDependencies();
        sf.print();
    }
}
