/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Title:        Checklist
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      University Bayreuth
 * @author Dieter Neubacher
 * @version 1.0
 */

package net.metadiversity.diversity.navigator.checklists.dataimport;

// import java.net.URL;          // F�r Datenbank-URL
// import java.sql.*;            // JDBC
// import java.lang.*;
import java.lang.String;
import java.lang.StringBuffer;
// import java.io.FileReader;
// import java.io.StringReader;
// import java.io.BufferedReader;

import java.util.StringTokenizer;
import java.util.Vector;
// import javax.swing.*;

import net.metadiversity.diversity.navigator.checklists.taxon.*;
// import net.metadiversity.diversity.navigator.checklists.database.DbGetty.*;

public class ParseImportLine
{
    final static String      errString = " (????) ";
    
    //-----------------------------------
    // tags to delimiter input data
    //-----------------------------------    
    final static String CONCEPT_NOTE_TAG            = "#CN#";
    final static String TAXONOMIC_REFERENCE_TAG     = "#TR#";
    final static String FLORISTIC_REFERENCES_TAG    = "#FR#";
    final static String TYPUS_INFORMATION_TAG       = "#TI#";
    final static String GEOGRAPHICAL_UNIT_TAG       = "#GU#";
    final static String NOTES_TAG                   = "#NT#";
    //-----------------------------------
    // IdentificationQualifier extentions
    //-----------------------------------
    final static String [] IdentificationQalifiers = { "cf.", "aff." };
        
    final static String IDENT_QUALI_GENUS           = "gen.";
    final static String IDENT_QUALI_SPECIES         = "sp.";
    final static String IDENT_QUALI_SSP             = "ssp.";
    final static String IDENT_QUALI_VARITY          = "var.";
    final static String IDENT_QUALI_FORMA           = "f.";
    final static String IDENT_QUALI_UNDEFINED       = "";
    
    
    public String      token;
    public TaxonData   taxon;


    final static int      codeIndex       = 0;
    final static int      genusIndex      = 1;
    final static int      speciesIndex    = 2;
    final static int      firstDesIndex   = 3;
    final static int      authorIndex     = 4;
    final static int      subSpeciesIndex = 5;
    final static int      varIndex        = 6;
    final static int      formIndex       = 7;
    final static int      conceptNoteIndex= 8;
    final static int      litTaxonIndex   = 9;
    final static int      litFlorIndex    = 10;
    final static int      typusIndex      = 11;
    final static int      locationIndex   = 12;
    final static int      commentIndex    = 13;

    // Parse
    boolean       errorFlag;
    boolean       writeXmlFileFlag;
    boolean       firstUnitFlag;
    StringBuffer  error;
    StringBuffer  outPutLine;
    StringBuffer  xmlChecklistData;
    StringBuffer  htmlChecklistData;
    StringBuffer  htmlChecklistHeaderData;
    StringBuffer  multiLocation;

    String        synonymLastName;
    
    int parsePos;
    int genusPos;
    int speciesPos;
    int subSpeciesPos;
    int varietyPos;
    int formPos;
    int firstDescriptorPos;
    int authorPos;
    int conceptNotePos;
    int literatureTaxonPos;
    int literatureFlorPos;
    int typusPos;
    int locationPos;
    int infoPos;

    //------------------
    // check EditAreaId
    //------------------
    net.metadiversity.diversity.navigator.checklists.database.DbGetty getty;
    
    public void setXmlOutputFlag( boolean flag )
    {    
        writeXmlFileFlag = flag;
    }
    private void setErrorMark()
    {
        // System.out.println( "Error" );
        error.append( errString );
        errorFlag = true;
    }

    void setTaxonField( String str, int index )
    {
        // System.out.println( "String: " + str + "  Index: " + index );
        
        switch( index )
        {
        case codeIndex :

                        break;
        case genusIndex :
                        taxon.setGenus( str );
                        break;
        case speciesIndex :
                        taxon.setSpecies( str );
                        break;
        case firstDesIndex :
                        taxon.setFirstDescription( str );
                        break;
        case authorIndex :
                        // System.out.println( ">" + str +"<" );
                        // Error in Author-Field
                        
                        if( str.indexOf( "var." ) >= 0 )
                            setErrorMark();
                        if( str.indexOf( "ssp." ) >= 0 )
                            setErrorMark();
                        /* is valied token in AuthorName
                        if( str.startsWith( "f." ) )
                            setErrorMark();
                        if( str.indexOf( " f." ) > 0 )
                            setErrorMark();
                        */
                        if( str.indexOf( CONCEPT_NOTE_TAG ) >= 0 )
                            setErrorMark();
                        if( str.indexOf( TAXONOMIC_REFERENCE_TAG ) >= 0 )
                            setErrorMark();
                        if( str.indexOf( FLORISTIC_REFERENCES_TAG ) >= 0 )
                            setErrorMark();
                        if( str.indexOf( NOTES_TAG ) >= 0 )
                            setErrorMark();
                        if( str.indexOf( TYPUS_INFORMATION_TAG ) >= 0 )
                            setErrorMark();
                        if( str.indexOf( GEOGRAPHICAL_UNIT_TAG ) >= 0 )
                            setErrorMark();                        
                        
                        taxon.setAuthor( str );                        
                        
                        break;
        case subSpeciesIndex :
                        if( taxon.getSubSpecies().length() != 0 )
                        {    
                            setErrorMark();
                        }    
                        else
                        {    
                            taxon.setSubSpecies( str );
                            taxon.setInfraspecific( str );    
                        }  
                        break;
        case varIndex :
                        if( taxon.getVariety().length() != 0 )
                        {    
                            setErrorMark();
                        }    
                        else
                        {    
                            taxon.setVariety( str );
                            taxon.setInfraspecific( str );    
                        }  
                        break;
        case formIndex :
                        if( taxon.getForm().length() != 0 )
                        {    
                            setErrorMark();
                        }    
                        else
                        {    
                            taxon.setForm( str );
                            taxon.setInfraspecific( str );    
                        }  
                        break;
        case conceptNoteIndex :
            
                        if( str.startsWith( CONCEPT_NOTE_TAG ) )
                        {
                            taxon.setConceptNote( str.substring( CONCEPT_NOTE_TAG.length(), str.length() ) );
                        }
                        else
                        {
                            setErrorMark();
                        }    
                        break;
             
        case litTaxonIndex :                
                        if( str.startsWith( TAXONOMIC_REFERENCE_TAG ) )
                        {
                            taxon.setLiteratureTaxon( str.substring( TAXONOMIC_REFERENCE_TAG.length(), str.length() ) );
                        }
                        else
                        {
                            setErrorMark();
                        }    
                        break;
        case litFlorIndex :                        
                        if( str.startsWith( FLORISTIC_REFERENCES_TAG ) )
                        {
                            taxon.setLiteratureFlor( str.substring( FLORISTIC_REFERENCES_TAG.length(), str.length() ) );                            
                        }    
                        else
                        {
                            setErrorMark();
                        }    
                        break;
        case typusIndex :
                        if( str.startsWith( TYPUS_INFORMATION_TAG ) )
                        {
                            taxon.setTypus( str.substring( TYPUS_INFORMATION_TAG.length(), str.length() ) );
                        }    
                        else
                        {
                            setErrorMark();
                        }    
                        break;
        case locationIndex :
                        if( str.startsWith( GEOGRAPHICAL_UNIT_TAG ) )
                        {
                            // System.out.println( "Location: " + str );
                            String EdtAreaID = str.substring( GEOGRAPHICAL_UNIT_TAG.length(), str.length() ).trim();
                            
                            if( net.metadiversity.diversity.navigator.checklists.Properties.checkAreaId && getty.testId( EdtAreaID ) != true )
                            {
                                setErrorMark();
                            }
                            multiLocation.append(  "(" + EdtAreaID  + ") " );
                            taxon.setLocation( multiLocation.toString() );
                        }    
                        else
                        {
                            setErrorMark();
                        }    
                        break;
        case commentIndex :
                        if( str.startsWith( NOTES_TAG ) )
                        {
                            taxon.setComment( str.substring( NOTES_TAG.length(), str.length() ) );
                        }    
                        else
                        {
                            setErrorMark();
                        }    
                        break;
        }
    }

    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public ParseImportLine()
    {
        // flags = new TaxonFlags();
        taxon = new TaxonData();
	error = new StringBuffer();
        outPutLine = new StringBuffer();
        // for xml-output
        xmlChecklistData        = new StringBuffer();
        htmlChecklistData       = new StringBuffer();
        htmlChecklistHeaderData = new StringBuffer();
        writeXmlFileFlag = true;
        // Unit
        firstUnitFlag = true;
        // last name for Synonym Marker
        synonymLastName = "";
        if( true )
        {    
            getty = new net.metadiversity.diversity.navigator.checklists.database.DbGetty( "hongkong.didi.dn", "getty", "didi", "" );
        }    
    }

    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    void clearPos()
    {
        parsePos = 0;
        genusPos = 0;
        speciesPos = 0;
        subSpeciesPos = 0;
        varietyPos = 0;
        formPos = 0;
        firstDescriptorPos = 0;
        authorPos = 0;
        conceptNotePos = 0;
        literatureFlorPos = 0;
        literatureTaxonPos = 0;
        typusPos = 0;
        locationPos = 0;
        infoPos = 0;
    }

    String getXmlChecklistData()
    {
        return xmlChecklistData.toString();
        /*
        if( firstUnitFlag )
            return xmlChecklistData.toString();
        else
            return xmlChecklistData.toString() + "</Unit>";
         */
    }
    boolean isStartUnit()
    {   
        return taxon.isStartUnit();
    }    
    TaxonData getTaxon()
    {
        return taxon;
    }
    
    String getHtmlChecklistData()
    {
        StringBuffer html = new StringBuffer();
        
        html.append( "<html>\n" );
        html.append( "<head>\n" );
        html.append( "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n" );
        html.append( "</head>\n" );
        html.append( "<body>\n" );        
        html.append( htmlChecklistHeaderData.toString() );
        html.append( "<BASEFONT color=#000000>\n" );        
        // Write comment in header line                    
        html.append( htmlTableHeader() );
        html.append( htmlChecklistData.toString() );
        
        html.append( "</table>\n" );
        html.append( "</body>\n" );
        html.append( "</html>\n" ); 
        
        return html.toString();
    }    
    //--------------------------------------------------------
    // Parse the input data, if no error detected retuen true
    //--------------------------------------------------------
    public String getErrorText()
    {
        return error.toString();
    }

    public boolean parseAndConvertToXmlData( String data )
    {
        boolean ret;
        writeXmlFileFlag = true;
        ret = parseData( data  );
        writeXmlFileFlag = false;
        return ret;
    }    
    //----------------------------------------------------------------------
    // check for IdentificationQualifier
    //----------------------------------------------------------------------
    private boolean checkAndSetIdentificationQualifier( String token, String extention )
    {
        for( int i=0; i < IdentificationQalifiers.length; i++ )
        {                    
            if( token.compareTo( IdentificationQalifiers[ i ] ) == 0 )
            {    
                taxon.setIdentificationQualifier( token + " " + extention );
                return true;
            }
        }    
        return false;
    }                
    private boolean checkIdentificationQualifier( String token )
    {
        for( int i=0; i < IdentificationQalifiers.length; i++ )
        {                    
            if( token.compareTo( IdentificationQalifiers[ i ] ) == 0 )
            {    
                return true;
            }
        }    
        return false;
    }                
    //----------------------------------------------------------------------
    // parse a checklist line entry
    //----------------------------------------------------------------------    
    public boolean parseData( String data )
    {
        taxon = new TaxonData();
        multiLocation = new StringBuffer();        
        clearPos();
	errorFlag  = false;        
        String rankIdentificationQualifier = "";
        
	if( data == null || data.length() == 0 )
	{
	    errorFlag = true;
	    return errorFlag;
	}
        /* ???
        data.replace( '\t', ' ' );
        data.replace( '\r', ' ' );
        data.replace( '\n', ' ' );
	data.trim();
         */
        data = data.replace( '\t', ' ' ).replace( '\r', ' ' ).replace( '\n', ' ' ).trim();

	error = new StringBuffer();
        outPutLine = new StringBuffer();

        //------------------------------------------
	// File header contains Location information
	//------------------------------------------

	if( data.startsWith( "##" ) )
	{
	    int pos = data.lastIndexOf( "**" );
	    if( pos > 0 )
	    {
		String code = data.substring( pos );
		String location = "";
		for( int i=0; i < code.length(); i++ )
		{
			if( Character.isWhitespace(  code.charAt( i ) ) )
			{
			    int endpos = i;
			    location = code.substring( i+1 );
			    code = code.substring( 0, i );
			    break;
			}
		}
		error.append( "Location: " + code + "-->" + location );
	    }
	    return errorFlag;
	}
        //----------------------------------------------------------------------
        // Header in XML-Format
        //----------------------------------------------------------------------
	if( data.trim().startsWith( "<" ) || data.trim().startsWith( "]" ) )     // xml tag or end of CDATA
	{
            if( ! firstUnitFlag )
            {
                // This is at the end of data
                xmlChecklistData.append( "</Unit>" );
                firstUnitFlag = true;
            }
            
            if( data.trim().startsWith( "<![CDATA" ) || data.trim().startsWith( "]]>" ) )
            {
                // don't write CDATA to xml-checklist
            }
            else
            {
                xmlChecklistData.append( data );
                xmlChecklistData.append( "\n" );
            }
            StringBuffer s = new StringBuffer( data );
            
            // quote characters
            int pos = 0;
            while( pos < s.length() )
            {
                if( s.charAt(pos) == '<' )
                {
                    s.replace( pos, pos + 1,"&lt;" );
                }
                else
                if( s.charAt(pos) == '>' )
                {
                    s.replace( pos, pos + 1,"&gt;" );
                }
                pos++;
            }
            htmlChecklistHeaderData.append( "<pre>" + s.toString() + "\n</pre>\n" );
            /*
            data = data.replace( '<', '[' );
            data = data.replace( '>', ']' );
            htmlChecklistData.append( "\n<pre>\n" + data + "\n</pre>\n" );
            */
            return false;
	}
        //----------------------------------------------------------------------
        // parse taxon data line
        //----------------------------------------------------------------------

        StringTokenizer dataLine = new StringTokenizer( data, " ()\"\t\n\r\f", true );
        String lastToken = "";
        String token;

        Vector v = new Vector();
        java.util.Iterator iterator = v.listIterator();
        int tokenPos = 0;
        int parseIndex = 0;
        int parseStatus = 0;

        boolean firstTokenFlag  = true;
        boolean firstAuthorFlag = false;
        boolean apostropheFlag  = false;
        boolean authorFlag      = false;
        boolean nameFlag        = false;
        boolean firstDescriptorFoundFlag = false;

        StringBuffer strBuffer = new StringBuffer( "" );

        while( dataLine.hasMoreElements() )
        {
            token = dataLine.nextToken();
            
            if( token.compareTo( "\t" ) == 0 ) token = " ";
            if( token.compareTo( "\n" ) == 0 ) token = " ";
            if( token.compareTo( "\r" ) == 0 ) token = " ";
            if( token.compareTo( "\f" ) == 0 ) token = " ";
            
            error.append( token );
            
            if( token.trim().compareTo( "" ) != 0 )
            {
                // System.out.println( tokenPos + ". " + token );
            }
            else
            {
                continue;
            }

            if( token.compareTo( "\"" ) == 0 )
            {
                setErrorMark();
                apostropheFlag = ! apostropheFlag;
            }
    	    //-----------------------------------------------
	    // check code information ( First 4 character )
	    //-----------------------------------------------
            if( tokenPos == 0 )
            {
                if( token.length() != 4 )
                {
                    setErrorMark();
                }
                else
                {
                    // validate codes
                    if( taxon.set( token ) == false )
                    {
                        setErrorMark();
                        // return errorFlag;
                    }
                }
                v.add( token );
                tokenPos++;
                parseIndex = codeIndex + 1;
                continue;
            }
	    //-----------------------------------------------
	    //-----------------------------------------------
            if( tokenPos == genusIndex )
            {
                    // Special IdentificationQualifier
                    if( checkAndSetIdentificationQualifier( token, IDENT_QUALI_GENUS ) )
                    {
                        continue;
                    }
                    taxon.setGenus( token );
                    // System.out.println( "Genus: " + taxon.getGenus() );
                    v.add( token );
                    tokenPos++;
                    if( taxon.isGenus() )
                        parseIndex = authorIndex;
                    else
                        parseIndex = genusIndex + 1;
                    continue;
            }
	    //-----------------------------------------------
            // Species
	    //-----------------------------------------------
            if( tokenPos == speciesIndex && !taxon.isGenus() )
            {
                    // Special IdentificationQualifier
                    if( checkAndSetIdentificationQualifier( token, IDENT_QUALI_SPECIES ) )
                    {
                        continue;
                    }
                    //--------------------------------------------------------------
                    // This is also a qualifier, the name of species is not defined
                    //--------------------------------------------------------------
                    if( token.compareTo( "spec.") == 0  || token.compareTo( "sp.") == 0 || token.compareTo( "spp.") == 0  )
                    {   
                        if( token.compareTo( "spp.") != 0  )
                            token = "sp.";
                        taxon.setIdentificationQualifier( token );                        
                        v.add( token );
                        tokenPos++;
                        parseIndex = authorIndex;
                        continue;
                    }
                    else
                    {
                        if( token.trim().endsWith( "." ) )
                        {
                            setErrorMark();
                        }    
                    }
                    taxon.setSpecies( token.toLowerCase() );
                    // System.out.println( "Species: " + taxon.getSpecies() );                    
                    v.add( token );
                    tokenPos++;
                    parseIndex = authorIndex;
                    continue;
            }
            //------------------------------------------------------------------
            // Special IdentificationQualifier
            //------------------------------------------------------------------
            if( parseIndex == authorIndex  )
            {
                if( token.compareTo( "nov.") == 0 )
                {
                    // this is also part of qualifier, add this token                        
                    taxon.setIdentificationQualifier( taxon.getIdentificationQualifier() + " " + token );                        
                    v.add( token );
                    tokenPos++;
                    parseIndex = authorIndex;
                    continue;                        
                 }            
            }                 
            
            if( parseIndex <= authorIndex && (!firstDescriptorFoundFlag) && token.compareTo( "(" ) == 0 )
            {
                // strBuffer = new StringBuffer( "" );
                nameFlag = true;
                firstAuthorFlag = true;
                parseIndex = firstDesIndex;

                // System.out.println( "First description" );
            }
            if( firstAuthorFlag && (!firstDescriptorFoundFlag) &&  token.compareTo( ")" ) == 0 )
            {
                firstAuthorFlag = false;
                firstDescriptorFoundFlag = true;
                // 
                if( strBuffer.toString().charAt( 0 ) != '('  )
                {
                    // Fist author must start with '('                    
/* BEGIN: SCHOLZ LISTEN 2004.06.29 */           
                    if( strBuffer.toString().startsWith( "auct." ) )
                    {
                        // System.out.println( token );
                        // set all folow data to SUFFIX
                        String suffix = strBuffer.toString();
                        while( dataLine.hasMoreElements() ) suffix += dataLine.nextElement();
                        // ??? add suffix ( not well formated ) format must be done ???
                        suffix = taxon.formatAuct( suffix + ')' );
                        taxon.setSuffix( suffix );
                        // System.out.println( suffix );
                        taxon.setFirstDescription( "" );
                    }               
                    else
                    {
                        setErrorMark();
                    }
/* END: SCHOLZ LISTEN */                                
                }
                else
                {
                    taxon.setFirstDescription( strBuffer.toString().substring( 1 ) );
                }
                strBuffer.append( token );
                // taxon.setFirstDiscription( strBuffer.toString() );
                // nameFlag = true;
                nameFlag = false;
                v.add( strBuffer.toString() );
                tokenPos++;
                strBuffer = new StringBuffer( "" );
                parseIndex = authorIndex;

                // System.out.println( "End of first description" );
                continue;
            }
            if( ! firstAuthorFlag && ! apostropheFlag )
            {
                // Special IdentificationQualifier
                if( checkIdentificationQualifier( token ) )
                {
                    rankIdentificationQualifier = token;
                    continue;
                }
                if( token.compareTo( "ssp.") == 0 && taxon.isSubSpecies() )
                {
                    //----------------------------------------------------------
                    // subspecies
                    //----------------------------------------------------------
                    if( rankIdentificationQualifier.compareTo( "" ) != 0 )
                    {
                        checkAndSetIdentificationQualifier( rankIdentificationQualifier, IDENT_QUALI_SSP );
                    }    
                    if( parseIndex >= varIndex )
                    {
                        v.add( token );
                        tokenPos++;
                        break;
                    }                        
                    v.add( token );
                    tokenPos++;
                    if( strBuffer.length() > 0 )
                    {
                        setTaxonField( strBuffer.toString(), parseIndex );
                        strBuffer = new StringBuffer( "" );
                    }
                    while( dataLine.hasMoreTokens() )
                    {
                        token = dataLine.nextToken();
                        error.append( token );
                        if( token.trim().compareTo( "" ) == 0 )
                            continue;
                        else
                            break;
                    }
                    subSpeciesPos = tokenPos;
                    taxon.setSubSpecies( token );
                    taxon.setInfraspecific( token );
                    v.add( token );
                    tokenPos++;

                    // special author test
                    if( taxon.getSpecies().compareTo( taxon.getSubSpecies() ) == 0 )
                    {
                        // use author from species
                        parseIndex = authorIndex + 1;
                    }
                    else
                    {
                        taxon.setFirstDescription( "" );
                        taxon.setAuthor( "" );
                        firstDescriptorFoundFlag = false;
                        parseIndex = authorIndex;
                    }
                    continue;
                }
                else
                if( token.compareTo( "var.") == 0 && taxon.isVariety() )
                {
                    //----------------------------------------------------------
                    // Variety
                    //----------------------------------------------------------                    
                    if( rankIdentificationQualifier.compareTo( "" ) != 0 )
                    {
                        checkAndSetIdentificationQualifier( rankIdentificationQualifier, IDENT_QUALI_VARITY );
                    }    
                    if( parseIndex >= formIndex )
                    {
                        v.add( token );
                        break;
                    }    
                    
                    
                    v.add( token );
                    tokenPos++;
                    if( strBuffer.length() > 0 )
                    {
                        setTaxonField( strBuffer.toString(), parseIndex );
                        strBuffer = new StringBuffer( "" );
                    }
                    while( dataLine.hasMoreTokens() )
                    {
                        token = dataLine.nextToken();
                        error.append( token );
                        if( token.trim().compareTo( "" ) == 0 )
                            continue;
                        else
                            break;
                    }
                    taxon.setVariety( token );
                    taxon.setInfraspecific( token );
                    v.add( token );
                    tokenPos++;
                    // special author test
                    if( taxon.getSpecies().compareTo( taxon.getVariety() ) == 0 )
                    {
                        // use author from species
                        parseIndex = authorIndex + 1;
                    }
                    else
                    {
                        taxon.setFirstDescription( "" );
                        taxon.setAuthor( "" );
                        firstDescriptorFoundFlag = false;
                        parseIndex = authorIndex;
                    }
                    continue;
                }
                else
                if( token.compareTo( "f.") == 0 && taxon.isForm() )
                {
                    //----------------------------------------------------------
                    // form
                    //----------------------------------------------------------
                    if( rankIdentificationQualifier.compareTo( "" ) != 0 )
                    {
                        checkAndSetIdentificationQualifier( rankIdentificationQualifier, IDENT_QUALI_FORMA );
                    }    
                    formPos = tokenPos;
                    tokenPos++;
                    if( strBuffer.length() > 0 )
                    {
                        setTaxonField( strBuffer.toString(), parseIndex );
                        strBuffer = new StringBuffer( "" );
                    }
                    while( dataLine.hasMoreTokens() )
                    {
                        token = dataLine.nextToken();
                        error.append( token );
                        if( token.trim().compareTo( "" ) == 0 )
                            continue;
                        else
                        {
                            break;
                        }
                    }
                    taxon.setForm( token );
                    taxon.setInfraspecific( token );
                    v.add( token );
                    tokenPos++;
                    // special author test
                    if( taxon.getSpecies().compareTo( taxon.getForm() ) == 0 )
                    {
                        // use author from species
                        parseIndex = authorIndex + 1;
                    }
                    else
                    {
                        taxon.setFirstDescription( "" );
                        taxon.setAuthor( "" );
                        firstDescriptorFoundFlag = false;
                        parseIndex = authorIndex;
                    }
                    continue;
                }
                //--------------------------------------------------------------
                //  the rest of line is comment
                //--------------------------------------------------------------
                if( token.compareTo( NOTES_TAG) == 0 )
                {
                    infoPos = tokenPos;
                    if( strBuffer.length() > 0 )
                    {
                        setTaxonField( strBuffer.toString(), parseIndex );
                        strBuffer = new StringBuffer( "" );
                    }
                    strBuffer.append( token );
                    while( dataLine.hasMoreTokens() )
                    {
                        token = dataLine.nextToken();
                        strBuffer.append( token );
                    }
                    taxon.setComment( strBuffer.toString() );
                    v.add( strBuffer.toString() );
                    tokenPos++;
                    parseIndex = commentIndex;
                    continue;
                }
                else
                //--------------------------------------------------------------
                // Concept Note
                //--------------------------------------------------------------
                if( parseIndex < litFlorIndex && token.compareTo( CONCEPT_NOTE_TAG ) == 0 )
                {
                    conceptNotePos = tokenPos;
                    if( strBuffer.length() > 0 )
                    {
                        setTaxonField( strBuffer.toString(), parseIndex );
                        strBuffer = new StringBuffer( "" );
                    }
                    parseIndex = conceptNoteIndex;
                }
                else
                //--------------------------------------------------------------
                // taxonomische Literature    
                //--------------------------------------------------------------
                if( parseIndex < litTaxonIndex && token.compareTo( TAXONOMIC_REFERENCE_TAG) == 0 )
                {
                    literatureTaxonPos = tokenPos;
                    if( strBuffer.length() > 0 )
                    {
                        setTaxonField( strBuffer.toString(), parseIndex );
                        strBuffer = new StringBuffer( "" );
                    }
                    parseIndex = litTaxonIndex;
                }
                else
                //--------------------------------------------------------------
                // floristische Literature    
                //--------------------------------------------------------------
                if( parseIndex < litFlorIndex && token.compareTo( FLORISTIC_REFERENCES_TAG) == 0 )
                {
                    literatureFlorPos = tokenPos;
                    if( strBuffer.length() > 0 )
                    {
                        setTaxonField( strBuffer.toString(), parseIndex );
                        strBuffer = new StringBuffer( "" );
                    }
                    parseIndex = litFlorIndex;
                }
                else
                //--------------------------------------------------------------
                // Typus    
                //--------------------------------------------------------------
                if( parseIndex < typusIndex && token.startsWith( TYPUS_INFORMATION_TAG)  )
                {
                    if( strBuffer.length() > 0 )
                    {
                        setTaxonField( strBuffer.toString(), parseIndex );
                        strBuffer = new StringBuffer( "" );
                    }
                    parseIndex = typusIndex;
                }
                else
                //--------------------------------------------------------------
                // location    
                //--------------------------------------------------------------
                if( parseIndex <= locationIndex && token.compareTo( GEOGRAPHICAL_UNIT_TAG) == 0 )
                {
                    literatureFlorPos = tokenPos;
                    if( strBuffer.length() > 0 )
                    {
                        setTaxonField( strBuffer.toString(), parseIndex );
                        strBuffer = new StringBuffer( "" );
                    }
                    parseIndex = locationIndex;
                }
            }
            strBuffer.append( token.toString() + " " );
        }        
        if( strBuffer.length() > 0  )
        {
            setTaxonField( strBuffer.toString(), parseIndex );
        }
        
        //----------------------------------------------------------------------
        // Check for more Errors
        //----------------------------------------------------------------------
        /*
        if( (taxon.getFirstDescription().length() != 0) && (! taxon.getFirstDescription().startsWith( "(" )) )
        {
            errorFlag = true;
        }
         */    
        // test taxon Author format validity
        if( taxon.getErrorFlag() == true )
        {
            setErrorMark();                        
        }    
        
        
        
        //----------------------------------------------------------------------
        // Synonymie Marker
        //----------------------------------------------------------------------
        int    synonymLevel = 0;
        String synonymMark = "&nbsp;";
                                             
        if( taxon.isGenus() ) synonymMark = "g";
        if( taxon.isStartUnit() ) 
        {
            synonymMark = "&nbsp;";
            synonymLastName = "";
        }    
                                
        if( synonymLastName.length() > 0 )
        {
            if( taxon.getInfraspecific().length() > 0 )
            {
                synonymLevel = getSynonymLevel( synonymLastName, taxon.getInfraspecific() );
                synonymMark  = getSynonymMarker( synonymLevel );
            }
            else if( taxon.getSpecies().length() > 0 )
            {
                synonymLevel = getSynonymLevel( synonymLastName, taxon.getSpecies() );
                synonymMark  = getSynonymMarker( synonymLevel );
            }
        }    
        
                                
        if( taxon.getInfraspecific().compareTo( "" ) == 0 )
        {    
            synonymLastName = taxon.getSpecies();
        }
        else
        {
            synonymLastName = taxon.getInfraspecific();
        }
        
        // "\u0333 \u039e "
        // System.out.println( mark + " " + t.getGenus() + " " + t.getSpecies() + " " + t.getInfraspecific() );        
        
        //----------------------------------------------------------------------
        // XML
        //----------------------------------------------------------------------
        outPutLine = new StringBuffer();        
        
        //----------------------------------------------------------------------
        // Mark Taxon-Units
        //----------------------------------------------------------------------
        if( taxon.isStartUnit() )    
        {   
            if( firstUnitFlag )
            {   
                // This is the first unit we found by parsing the input data
                firstUnitFlag = false;
                outPutLine.append( "\n<Unit>\n" );
            }
            else
            {   
                // we already found an unit before, so colose and start an new unit
                outPutLine.append( "</Unit>\n<Unit>\n" );                                
            }    
        }            
        // Output the Taxon Data
        outPutLine.append( "\t<Taxon>" );        
        outPutLine.append( "<Code>"          + taxon.getText()              + "</Code>"      );        
        outPutLine.append( "<SynLevel>"      + synonymLevel                 + "</SynLevel>"  );        
        outPutLine.append( "<Genus>"         + taxon.getGenus()             + "</Genus>"     );
        outPutLine.append( "<Species>"       + taxon.getSpecies()           + "</Species>"   );
        //----------------------------------------------------------------------
        // Rank of InfraspecificEpithet
        //----------------------------------------------------------------------
        boolean  InfraspecificEpithetFlag = false;
        
        String cdataStart = "<![CDATA[";
        String cdataEnd = "]]>";
        
        //----------------------------------------------------------------------
        // HTML
        //----------------------------------------------------------------------
        
        StringBuffer htmlOutput = new StringBuffer();
        if( errorFlag )
        {
            htmlOutput.append( "</table>" );
            htmlOutput.append( "<FONT color=#FF0000>" );
            htmlOutput.append( error.toString() );
            htmlOutput.append( "</FONT>" );
            
            htmlOutput.append( htmlTableHeader() );
        }    
        htmlOutput.append( "<tr>\n" );
        // Mark Taxon-Units
        if( taxon.isStartUnit() )
        {    
            htmlOutput.append( "<td><FONT color=#FF0000>" + taxon.getText() + "&nbsp;" + synonymMark + "&nbsp;</FONT></td>" );
        }
        else    
        {
            htmlOutput.append( "<td><FONT color=#FFF000>" + taxon.getText() + "&nbsp;" + synonymMark + "&nbsp;</FONT></td>" );
        }
        htmlOutput.append( "<td>" + taxon.getGenus()                    + "&nbsp;</td>" );
        htmlOutput.append( "<td>" + taxon.getSpecies()                  + "&nbsp;</td>" );
        
        InfraspecificEpithetFlag = false;
        if( taxon.isForm() )
        {    
            outPutLine.append( "<Rank>f.</Rank><InfraspecificEpithet>"     + taxon.getForm()       + "</InfraspecificEpithet>" );
            htmlOutput.append( "<td>f.</td><td>"   + taxon.getForm()       + "&nbsp;</td>" );
            InfraspecificEpithetFlag = true;
        }
        else
        if( taxon.isVariety() )
        {
            if( InfraspecificEpithetFlag ) errorFlag = true;
            outPutLine.append( "<Rank>var.</Rank><InfraspecificEpithet>"   + taxon.getVariety()    + "</InfraspecificEpithet>" );
            htmlOutput.append( "<td>var.</td><td>" + taxon.getVariety()    + "&nbsp;</td>" );
            InfraspecificEpithetFlag = true;
        }
        else
        if( taxon.isSubSpecies() )
        {
            if( InfraspecificEpithetFlag ) errorFlag = true;
            outPutLine.append( "<Rank>ssp.</Rank><InfraspecificEpithet>"       + taxon.getSubSpecies() + "</InfraspecificEpithet>" );
            htmlOutput.append( "<td>ssp.</td><td>" + taxon.getSubSpecies() + "&nbsp;</td>" );
            InfraspecificEpithetFlag = true;
        }    
        
        if( InfraspecificEpithetFlag == false )
        {
            if( taxon.isGenus() )
            {    
                outPutLine.append( "<Rank>gen.</Rank><InfraspecificEpithet></InfraspecificEpithet>" );            
                htmlOutput.append( "<td>gen.</td><td>&nbsp;</td>" );
            }
            else
            if( taxon.isSpecies() )
            {    
                outPutLine.append( "<Rank>sp.</Rank><InfraspecificEpithet></InfraspecificEpithet>" );            
                htmlOutput.append( "<td>sp.</td><td>&nbsp;</td>" );
            }
            else
            {    
                outPutLine.append( "<Rank></Rank><InfraspecificEpithet></InfraspecificEpithet>" );            
                htmlOutput.append( "<td><FONT color=#FF0000>?&nbsp;</FONT></td><td><FONT color=#FF0000>?&nbsp;</FONT></td>" );
            }    
        }    
        
        htmlOutput.append( "<td>" + taxon.getFirstDescription()   + "&nbsp;</td>" );
        htmlOutput.append( "<td>" + taxon.getAuthor()             + "&nbsp;</td>" );
        htmlOutput.append( "<td>" + taxon.getSuffix()             + "&nbsp;</td>" );
        htmlOutput.append( "<td>" + taxon.getStatus()             + "&nbsp;</td>" );
        htmlOutput.append( "<td>" + taxon.getConceptNote()        + "&nbsp;</td>" );
        htmlOutput.append( "<td>" + taxon.getIdentificationQualifier()  + "&nbsp;</td>" );
        htmlOutput.append( "<td>" + taxon.getLiteratureTaxon()    + "&nbsp;</td>" );
        htmlOutput.append( "<td>" + taxon.getLiteratureFlor()     + "&nbsp;</td>" );
        htmlOutput.append( "<td>" + taxon.getTypus()              + "&nbsp;</td>" );
        htmlOutput.append( "<td>" + taxon.getLocation()           + "&nbsp;</td>" );
        htmlOutput.append( "<td>" + taxon.getComment()            + "&nbsp;</td>" );
        htmlOutput.append( "\n</tr>\n" );
        
        outPutLine.append( "<First>"         + cdataStart + taxon.getFirstDescription()  + cdataEnd + "</First>"    );
        outPutLine.append( "<Author>"        + cdataStart + taxon.getAuthor()            + cdataEnd + "</Author>"   );
        outPutLine.append( "<Suffix>"        + cdataStart + taxon.getSuffix()            + cdataEnd + "</Suffix>"   );
        outPutLine.append( "<Status>"        + cdataStart + taxon.getStatus()            + cdataEnd + "</Status>"   );
        outPutLine.append( "<ConceptNote>"   + cdataStart + taxon.getConceptNote()       + cdataEnd + "</ConceptNote>" );
        outPutLine.append( "<IdQual>"        + cdataStart + taxon.getIdentificationQualifier()            + cdataEnd + "</IdQual>"   );
        outPutLine.append( "<TaxonRef>"      + cdataStart + taxon.getLiteratureTaxon()   + cdataEnd + "</TaxonRef>");
        outPutLine.append( "<FlorRef>"       + cdataStart + taxon.getLiteratureFlor()    + cdataEnd + "</FlorRef>" );
        outPutLine.append( "<Typus>"         + cdataStart + taxon.getTypus()             + cdataEnd + "</Typus>"    );
        outPutLine.append( "<GeoUnit>"       + cdataStart + taxon.getLocation()          + cdataEnd + "</GeoUnit>" );
        outPutLine.append( "<Notes>"         + cdataStart + taxon.getComment()           + cdataEnd + "</Notes>"  );
        outPutLine.append( "</Taxon>" );
	outPutLine.append( "\n" );

        xmlChecklistData.append( outPutLine.toString() );
        htmlChecklistData.append( htmlOutput.toString() );
                
	// System.out.print( outPutLine.toString() );
        if( errorFlag ) System.out.println( error );
        System.out.flush();
	return ! errorFlag;
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------

    
    java.lang.String getSynonymMarker( java.lang.String lastName, java.lang.String thisName )
    {
        final String threeLines = "&#8801;";
        final String   twoLines = "=";

        switch( getSynonymLevel( lastName, thisName ) )
        {
            case 2:     return  twoLines;
            case 3:     return  threeLines;
            default:    return  "";                   
        }        
    }
    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------      
    java.lang.String getSynonymMarker( int level )
    {
        final String threeLines = "&#8801;";
        final String   twoLines = "=";

        switch( level )
        {
            case 2:     return  twoLines;
            case 3:     return  threeLines;
            default:    return  "";                   
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------      
    int getSynonymLevel( java.lang.String lastName, java.lang.String thisName )
    {
        
        int     cutLastLength = 0;
        int     cutThisLength = 0;
        
        if( lastName.compareTo( thisName ) == 0 )
        {
            return 3;
        }
        
        //----------------------------------------------------------------------
        // er, ra, rum
        //----------------------------------------------------------------------
        if( lastName.endsWith( "er" ) )  { cutLastLength = 2; }
        if( lastName.endsWith( "ra" ) )  { cutLastLength = 2; }
        if( lastName.endsWith( "rum" ) ) { cutLastLength = 3; }
        
        if( thisName.endsWith( "er" ) )  { cutThisLength = 2; }
        if( thisName.endsWith( "ra" ) )  { cutThisLength = 2; }
        if( thisName.endsWith( "rum" ) ) { cutThisLength = 3; }
        
        if( cutLastLength > 0 && cutThisLength > 0 )
        {   
            java.lang.String lastStr = lastName.substring( 0, lastName.length() - cutLastLength );
            java.lang.String thisStr = thisName.substring( 0, thisName.length() - cutThisLength );
            
            if( lastStr.compareTo( thisStr ) == 0 )
            {
                return 3;
            }    
        }
        //----------------------------------------------------------------------
        // a, us, um
        //----------------------------------------------------------------------
        if( lastName.endsWith( "a" ) )  { cutLastLength = 1; }
        if( lastName.endsWith( "us" ) ) { cutLastLength = 2; }
        if( lastName.endsWith( "um" ) ) { cutLastLength = 2; }
        
        if( thisName.endsWith( "a" ) )  { cutThisLength = 1; }
        if( thisName.endsWith( "us" ) ) { cutThisLength = 2; }
        if( thisName.endsWith( "um" ) ) { cutThisLength = 2; }
        
        if( cutLastLength > 0 && cutThisLength > 0 )
        {   
            java.lang.String lastStr = lastName.substring( 0, lastName.length() - cutLastLength );
            java.lang.String thisStr = thisName.substring( 0, thisName.length() - cutThisLength );
            
            if( lastStr.compareTo( thisStr ) == 0 )
            {
                return 3;
            }    
        }
        //----------------------------------------------------------------------
        // is, e
        //----------------------------------------------------------------------        
        cutLastLength = 0;
        cutThisLength = 0;
        
        if( lastName.endsWith( "e" ) )  { cutLastLength = 1; }
        if( lastName.endsWith( "is" ) ) { cutLastLength = 2; }
        
        if( thisName.endsWith( "e" ) )  { cutThisLength = 1; }
        if( thisName.endsWith( "is" ) ) { cutThisLength = 2; }
        
        if( cutLastLength > 0 && cutThisLength > 0 )
        {   
            java.lang.String lastStr = lastName.substring( 0, lastName.length() - cutLastLength );
            java.lang.String thisStr = thisName.substring( 0, thisName.length() - cutThisLength );
            
            if( lastStr.compareTo( thisStr ) == 0 )
            {
                return 3;
            }    
        }                
        return 2;
    }   
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    private java.lang.String htmlTableHeader()
    {
        java.lang.StringBuffer str = new java.lang.StringBuffer();
        
        str.append( "<table BORDER NOSAVE>\n" );
        // Write comment in header line            
        str.append( "<tr>" );
        str.append( "<td>Code</td>" );
        str.append( "<td>Genus</td>" );
        str.append( "<td>Species</td>" );
        str.append( "<td>R.</td>" );
        str.append( "<td>Rank</td>" );
        str.append( "<td>First</td>" );
        str.append( "<td>Author</td>" );
        str.append( "<td>Suf.</td>" );
        str.append( "<td>Status</td>" );
        str.append( "<td>Concept Note</td>" );
        str.append( "<td>Id. Qual.</td>" );
        str.append( "<td>Ta. Ref</td>" );
        str.append( "<td>Fl. Ref.</td>" );
        str.append( "<td>Typus</td>" );
        str.append( "<td>GU</td>" );
        str.append( "<td>Note</td>" );
        str.append( "</tr>" );
        
        return str.toString();
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args)
    {
        // only for testing
        ParseImportLine pil = new ParseImportLine();
        
        
    }
    
}