/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Title:        Checklist
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      University Bayreuth
 * @author Dieter Neubacher
 * @version 1.0
 */

package net.metadiversity.diversity.navigator.checklists.taxon;

import java.util.StringTokenizer;

public class TaxonData extends TaxonFlags
{
// checklist entry data

private boolean errorFlag;

private int     genusId;
private int     firstDiscriptionId;
private int     nameId;
private int     typusId;

private String  genus;
private String  species;
private String  infraspecific;
private String  subSpecies;
private String  variety;
private String  form;
private String  firstDiscription;
private String  author;
private String  identificationQualifier;
private String  suffix;
private String  status;
private String  conceptNote;
private String  literatureTaxon;
private String  literatureFlor;
private String  reference;
private String  typus;
private String  location;
private String  comment;

    //--------------------------------------------
    // replease this Author Names in the String
    //--------------------------------------------
/*
 * von Zwackh-Holzhausen
 * von Hazslin
 * von Esenbeck
 * von Martius
 * de Saint-Amans
 * de Candolle
 * d'Urville
 * McCarthy
 *
 * 'et al.' wird in & Al. verwandelt, muss 'et al.' bleiben
 * 'Hook. Fil.', 'Hook. F.', 'Hook. fil.' soll  'Hook. f.' sein.
*/
    final static String [] repleasePart = { "De Lesd", "V. D. Boom",  "Dc.", "Mccarthy", "D'urville", "De Candolle", "De Saint-Amans", "Fil.", "Hook. F.","Linn� F.", "Ny.",  "Von ",  "& Al."  };
    final static String [] repleaseWith = { "de Lesd", "v. d. Boom",  "DC.", "McCarthy", "d'Urville", "de Candolle", "de Saint-Amans", "f.",   "Hook. f.","Linn� f.", "Nyl.", "von ",  "at al."  };

    // Suffix List
    // This List must start whis combind tokens
    
    final static String [] specialSuffixPart     = { "S. Ampl.",  "S. Auct.", "S. L.",  "S. Lat.", "S. Str.", "Agg.", "Auct.",    "Complex", "Emend",   "Non",   "Sensu",  "P. P.",  "P. Pt.", "P. Max. P.",    "P. Min. P."  };
    final static String [] specialSuffixReplease = { "s. ampl. ", "s. auct.", "s. l. ", "s. l. ",  "s. str.", "agg. ","s. auct. ","complex", "emend",   "non ",  "sensu ", "p. p. ", "p. p. ", "p. max. p. ",   "p. min. p. " };
        
    public TaxonData()
    {
        genusId     = 0;
        firstDiscriptionId = 0;
        nameId      = 0;
        typusId     = 0;
        
        errorFlag       = false;

        genus           = new String();
        species         = new String();
        infraspecific   = new String();
        
        subSpecies      = new String();
        variety         = new String();
        form            = new String();
        
        firstDiscription= new String();
        author          = new String();
        suffix          = new String();
        status          = new String();
        identificationQualifier  = new String();
        conceptNote     = new String();
        literatureTaxon = new String();
        literatureFlor  = new String();
        reference       = new String();
        typus           = new String();
        location        = new String();
        comment         = new String();
    }

    public void setErrorFlag( boolean s ) { errorFlag = s; };

    public void setGenus( String s )            { genus             = formatGenus( s ); }
    public void setSpecies( String s )          { species           = s.trim().toLowerCase(); }
    public void setInfraspecific( String s )    { infraspecific     = s.trim().toLowerCase(); }
    
    public void setSubSpecies( String s )       { subSpecies        = s.trim().toLowerCase(); }
    public void setVariety( String s )          { variety           = s.trim().toLowerCase(); }
    public void setForm( String s )             { form              = s.trim().toLowerCase(); }
    
    public void setFirstDescription( String s ) { firstDiscription  = formatAuthor( s ); }
    public void setAuthor( String s )           { author            = formatAuthor( s ); }
    public void setIdentificationQualifier( String s )           { identificationQualifier = s.trim(); }  
    public void setSuffix( String s )           { suffix            = s.trim(); }  
    public void setStatus( String s )           { status            = s.trim(); }  
    public void setConceptNote( String s )      { conceptNote       = s.trim(); }  
    public void setLiteratureTaxon( String s )  { literatureTaxon   = formatLiterature( s ); }
    public void setLiteratureFlor( String s )   { literatureFlor    = formatSentencCase( formatLiterature( s ) ); }
    public void setReference( String s )        { reference         = s.trim(); }
    public void setTypus( String s )            { typus             = s.trim(); }
    public void setLocation( String s )         { location          = s.trim(); }
    public void setComment( String s )          { comment           = s.trim(); }

    public String getGenus()                    { return genus; }
    public String getSpecies()                  { return species; }
    public String getInfraspecific()            { return infraspecific; }
    
    public String getSubSpecies()               { return subSpecies; }
    public String getVariety()                  { return variety; }
    public String getForm()                     { return form; }
    
    public String getFirstDescription()         { return firstDiscription; }
    public String getAuthor()                   { return author; }
    public String getIdentificationQualifier()  { return identificationQualifier; }
    public String getSuffix()                   { return suffix; }
    public String getStatus()                   { return status; }
    public String getConceptNote()              { return conceptNote; }
    public String getLiteratureTaxon()          { return literatureTaxon; }
    public String getLiteratureFlor()           { return literatureFlor; }
    public String getReference()                { return reference; }
    public String getTypus()                    { return typus; }
    public String getLocation()                 { return location; }
    public String getComment()                  { return comment; }

    public boolean getErrorFlag()       { return errorFlag; }
    
    private String formatLiterature( String s )
    {
        StringBuffer  lit = new StringBuffer();
        StringTokenizer t = new StringTokenizer( s, "()., \t\n\r\f", true );
        String lastToken  = "";
        String token;

        while( t.hasMoreElements() )
        {
            token = t.nextToken();
            
            if( token.compareTo( "\t" ) == 0 ) token = " ";
            if( token.compareTo( "\n" ) == 0 ) token = " ";
            if( token.compareTo( "\r" ) == 0 ) token = " ";
            if( token.compareTo( "\f" ) == 0 ) token = " ";
            
            if( lastToken.compareTo( "(" ) == 0 && token.compareTo( " " ) == 0 )
            {
                continue;
            }
            if( lastToken.compareTo( " " ) == 0 && token.compareTo( " " ) == 0 )
            {    
                continue;
            }    
            //------------------------
            // delete blank before '.'
            //------------------------
            if( lastToken.compareTo( " " ) == 0 && token.compareTo( "." ) == 0 )
            {
                if( lit.length() > 0 ) lit.delete( lit.length() - 1, lit.length() );
            }
            //------------------------
            // delete blank before ','
            //------------------------
            if( lastToken.compareTo( " " ) == 0 && token.compareTo( "," ) == 0 )
            {
                if( lit.length() > 0 ) lit.delete( lit.length() - 1, lit.length() );
            }
            //------------------------
            // delete blank befor ')'
            //------------------------
            if( token.startsWith( ")" ) == true && lastToken.compareTo( " " ) == 0 )
            {
                if( lit.length() > 0 ) lit.delete( lit.length() - 1, lit.length() );
            }
            
            lit.append( token );
            lastToken = token;
        }
        String str = lit.toString().trim();
        //------------------
        // delete last "."
        //------------------
        
        if( str.length() > 0 && str.charAt( str.length() -1 ) == '.' )
        {
            return str.substring( 0 , str.length() -1 );   
        }
        else
        {    
            return str;
        } 
    }
    
    //--------------------------------------------------------------------------
    // Format all tokens whis only upper-case-letters to first letter in uppercase
    // and all other characters to lowercase
    //--------------------------------------------------------------------------
    
    private String formatSentencCase( String s )
    {
        StringBuffer  output = new StringBuffer();
        StringTokenizer t = new StringTokenizer( s, " !�$%&/?<>|{}[]().,;:-+*#\'\"@\t\n\r\f", true );
        String token;

        while( t.hasMoreElements() )
        {
            token = t.nextToken();            
            
            if( token.compareTo( "\t" ) == 0 ) { output.append(" " ); continue; }
            if( token.compareTo( "\n" ) == 0 ) { output.append(" " ); continue; }
            if( token.compareTo( "\r" ) == 0 ) { output.append(" " ); continue; }
            if( token.compareTo( "\f" ) == 0 ) { output.append(" " ); continue; }
            
            if( token.length() > 1 )
            {    
                String upper = token.toUpperCase();
                
                if( token.compareTo( upper ) == 0 )
                {
                    if( token.compareTo( "DC" ) == 0 )
                    {    
                        output.append( token );
                    }
                    else
                    {
                        output.append( token.charAt( 0 ) + token.substring( 1 ).toLowerCase() );
                    }
                }
                else
                {
                    output.append( token );
                }    
            }
            else
            {
                output.append( token );
            }                
        }    
        return output.toString();
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public String formatAuct( String s )
    {
        return formatAuthor( s, false );                
    }
    private String formatAuthor( String s )
    {
        return formatAuthor( s, true );
    }
    private String formatAuthor( String s, boolean setStatusSuffixFlag )
    {
        errorFlag = false;
        StringBuffer name = new StringBuffer();
        StringBuffer suffix = new StringBuffer();        
        StringBuffer status = new StringBuffer();
        StringTokenizer t = new StringTokenizer( s, "-():., \t\n\r\f", true );
        String lastToken = "";
        String token;
        
        if( ! setStatusSuffixFlag ) 
        {
            status = suffix;
        }

        boolean skipDotFlag = false;

        while( t.hasMoreElements() )
        {            
            token = t.nextToken();
            
            if( token.compareTo( "\t" ) == 0 ) token = " ";
            if( token.compareTo( "\n" ) == 0 ) token = " ";
            if( token.compareTo( "\r" ) == 0 ) token = " ";
            if( token.compareTo( "\f" ) == 0 ) token = " ";
            
            if( skipDotFlag == true )
            {   
                skipDotFlag = false;
                if( token.compareTo( "." ) == 0 )
                    continue;
            }                    
            //----------------------
            // delete blank after (
            //----------------------
            if( lastToken.compareTo( "(" ) == 0 && token.compareTo( " " ) == 0 )
            {
                continue;
            }
            if( lastToken.compareTo( " " ) == 0 && token.compareTo( " " ) == 0 )
                continue;
            //------------------------
            // delete blank before '.'
            //------------------------
            if( lastToken.compareTo( " " ) == 0 && token.compareTo( "." ) == 0 )
            {
                if( name.length() > 0 ) name.delete( name.length() - 1, name.length() );
            }
            //------------------------
            // delete blank before ','
            //------------------------
            if( lastToken.compareTo( " " ) == 0 && token.compareTo( "," ) == 0 )
            {
                if( name.length() > 0 ) name.delete( name.length() - 1, name.length() );
            }
            if( lastToken.compareTo( "." ) == 0 && token.compareTo( " " ) != 0  )
            {
                if( token.startsWith( ")" ) == false )
                {
                    lastToken = " ";
                    name.append( " " );
                }
            }
            if( lastToken.compareTo( "," ) == 0 && token.compareTo( " " ) != 0  )
            {
                if( token.startsWith( ")" ) == false )
                {
                    lastToken = " ";
                    name.append( " " );
                }
            }
            //------------------------
            // delete blank befor ')'
            //------------------------
            if( token.startsWith( ")" ) == true && lastToken.compareTo( " " ) == 0 )
            {
                if( name.length() > 0 ) name.delete( name.length() - 1, name.length() );
            }
            if( token.startsWith( "-" ) == true && lastToken.compareTo( " " ) == 0 )
            {
                if( name.length() > 0 ) name.delete( name.length() - 1, name.length() );
            }


            if( token.compareTo( "and" ) == 0 )     { name.append( "& "  ); lastToken = " "; continue; }
            if( token.compareTo( "apud" ) == 0 )    { name.append( "in " ); lastToken = " "; continue; }
            if( token.compareTo( "et" ) == 0 )      { name.append( "& "  ); lastToken = " "; continue; }
            if( token.compareTo( "ex" ) == 0 )      { name.append( "ex " ); lastToken = " "; continue; }
            if( token.compareTo( "in" ) == 0 )      { name.append( "in " ); lastToken = " "; continue; }

           
            //------------------------------------------------------------------
            //  Nomenklatur-Status
            //------------------------------------------------------------------
            /*
            if( token.compareTo( "nom" ) == 0 )     { name.append( "nom " );    lastToken = " ";continue; }
            if( token.compareTo( "rej" ) == 0 )     { name.append( "rej " );    lastToken = " ";continue; }
            if( token.compareTo( "prop" ) == 0 )    { name.append( "prop " );   lastToken = " ";continue; }
            if( token.compareTo( "reject" ) == 0 )  { name.append( "rej. " );    lastToken = " ";continue; }
            if( token.compareTo( "illeg" ) == 0 )   { name.append( "illegit " ); lastToken = " ";continue; }
            if( token.compareTo( "inval" ) == 0 )   { name.append( "illegit " ); lastToken = " ";continue; }
            if( token.compareTo( "illegit" ) == 0 ) { name.append( "illegit " ); lastToken = " ";continue; }
            if( token.compareTo( "inval" ) == 0 )   { name.append( "invalid " ); lastToken = " ";continue; }
            if( token.compareTo( "invalid" ) == 0 ) { name.append( "invalid " ); lastToken = " ";continue; }
            if( token.compareTo( "nud" ) == 0 )     { name.append( "nud " );    lastToken = " ";continue; }
            if( token.compareTo( "nudum" ) == 0 )   { name.append( "nud. " );   lastToken = " ";continue; }
            if( token.compareTo( "nomen" ) == 0 )   { name.append( "nom. " );   lastToken = " ";continue; }
            if( token.compareTo( "excl" ) == 0 )    { name.append( "excl " );   lastToken = " ";continue; }
            if( token.compareTo( "cons" ) == 0 )    { name.append( "cons " );   lastToken = " ";continue; }
            if( token.compareTo( "confus" ) == 0 )  { name.append( "confus " ); lastToken = " ";continue; }
            if( token.compareTo( "comb" ) == 0 )    { name.append( "comb " );   lastToken = " ";continue; }
            */
            
            if( token.compareTo( "nom" ) == 0 )     { status.append( "nom. " );      skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "rej" ) == 0 )     { status.append( "rej. " );      skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "prop" ) == 0 )    { status.append( "prop. " );     skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "reject" ) == 0 )  { status.append( "rej. " );      skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "illeg" ) == 0 )   { status.append( "illegit. " );  skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "illegit" ) == 0 ) { status.append( "illegit. " );  skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "inval" ) == 0 )   { status.append( "invalid. " );  skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "invalid" ) == 0 ) { status.append( "invalid. " );  skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "nud" ) == 0 )     { status.append( "nud. " );      skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "nudum" ) == 0 )   { status.append( "nud. " );      skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "nomen" ) == 0 )   { status.append( "nom. " );      skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "excl" ) == 0 )    { status.append( "excl. " );     skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "cons" ) == 0 )    { status.append( "cons. " );     skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "confus" ) == 0 )  { status.append( "confus. " );   skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "comb" ) == 0 )    { status.append( "comb. " );     skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "ined" ) == 0 )    { status.append( "ined. " );     skipDotFlag = true;  lastToken = " ";continue; }
            if( token.compareTo( "unpubl" ) == 0 )  { status.append( "ined. " );     skipDotFlag = true;  lastToken = " ";continue; }
            // f. ?--> fil.
            if( token.compareTo( "f" ) == 0 )  { name.append( "Fil " ); lastToken = " ";continue; }
            // convert first character to upper case
            else
            {
                String  first = token.substring( 0, 1 ).toUpperCase();
                String  part  = token.substring( 1 ).toLowerCase();
                token = first + part;
            }
            // name.append( "[" ).append( token ).append("]");
            name.append( token );
            lastToken = token;
        }

        String str = name.toString().trim();
        int index;
        //----------------------------------------------------------------------
        // Replease some not well formated Author Names
        //----------------------------------------------------------------------
        for( int i=0; i < repleasePart.length; i++ )
        {
            while( (index = str.indexOf( repleasePart[ i ] )) != -1 )
            {
                str = str.substring( 0, index ) + repleaseWith[ i ] + str.substring( index + repleasePart[ i ].length() );
            }
        }
        //----------------------------------------------------------------------
        // Find and split Suffix tokens
        //----------------------------------------------------------------------
        
        int suffixStartIndex = -1;
        
        for( int i=0; i < specialSuffixPart.length; i++ )
        {
            if( (index = str.indexOf( specialSuffixPart[ i ] )) != -1 )
            {                
                // sonderfall S. L. F. Meyer      
                if( str.substring( index ).startsWith( "S. L. F. Meyer" ) )
                {
                    // this is an author name
                    continue;
                }
                else
                {                    
                    if( suffixStartIndex == -1 ) suffixStartIndex = index;
                    str = str.substring( 0, index ) + specialSuffixReplease[ i ] + str.substring( index + specialSuffixPart[ i ].length() );
                }    
            }
        }
        //----------------------------------------------------------------------
        // Suffix found, split AuthorString and append to SuffixString
        //----------------------------------------------------------------------
        if( setStatusSuffixFlag && suffixStartIndex != -1 )
        {   
                suffix.append( str.substring( suffixStartIndex  ) ); 
                str = str.substring( 0, suffixStartIndex );
                setSuffix( suffix.toString() );        
        }
        //----------------------------------------------------------------------
        // Status
        //----------------------------------------------------------------------
        if( setStatusSuffixFlag )
        {
            setStatus( status.toString() );
            // System.out.println( ">>>" + getStatus() + "<<<" );
            checkNomenclaturalStatus();
        }
        return str;
    }

    private void checkNomenclaturalStatus()
    {
        // all posible Nomenclatural status
        
        if( isCombInval()      && getStatus().compareTo( "comb. invalid." ) != 0 )
            setErrorFlag( true );
        else
        if( isIned()           && getStatus().compareTo( "ined."         ) != 0 )
            setErrorFlag( true );
        else
        if( isNomenConfus()    && getStatus().compareTo( "nom. confus."  ) != 0 )
            setErrorFlag( true );
        else
        if( isNomenCons()      && getStatus().compareTo( "nom. cons."    ) != 0 )
            setErrorFlag( true );
        else
        if( isNomenExcl()      && getStatus().compareTo( "nom. excl."    ) != 0 )
            setErrorFlag( true );
        else
        if( isNomenIlleg()     && getStatus().compareTo( "nom. illegit." ) != 0 )
            setErrorFlag( true );
        else
        if( isNomenInvalid()   && getStatus().compareTo( "nom. invalid." ) != 0 )
            setErrorFlag( true );
        else
        if( isNomenNudum()     && getStatus().compareTo( "nom. nud."     ) != 0 )
            setErrorFlag( true );
        else
        if( isNomenRej()       && getStatus().compareTo( "nom. rej."     ) != 0 )
            setErrorFlag( true );
        else
        if( isValid()          && getStatus().compareTo( ""              ) != 0 )
            setErrorFlag( true );
    }
    
    private String formatGenus( String str )
    {
        String  first = str.substring( 0, 1 ).toUpperCase();
        String  part  = str.substring( 1 ).trim().toLowerCase();
        return( first + part );
    }
    public String insertDataString()
    {
        StringBuffer insert = new StringBuffer();
        StringBuffer data   = new StringBuffer();

        insert.append(  "INSERT into first_list ( "     );
        // insert.append(  "id, "            		); data.append( "\'" +  + "\'," ); // serial UNIQUE NOT NULL,
        // insert.append(  "checklist_id, "		); data.append( "\'" +  + "\'," ); // int4,
        // insert.append(  "checked_flag, " 		); data.append( "\'" +  + "\'," ); // bool default false,
        insert.append(  "error_flag, "                  ); data.append( "\'" + errorFlag + "\'," ); // bool default true,
        // -- synonymietry
        insert.append(  "synonym_flag, "          	); data.append( "\'" + isSynonym() + "\'," ); // bool default false,
        insert.append(  "non_synonym_flag, "          	); data.append( "\'" + isNonSynonym() + "\'," ); // bool default false,
        insert.append(  "synonymietry_unknown_flag, "   ); data.append( "\'" + isSynonymietryUnkown() + "\'," ); // bool default true,
        // -- basionymietry
        insert.append(  "basionym_flag, " 		); data.append( "\'" + isBasionym() + "\'," ); // bool default false,
        insert.append(  "combined_flag, " 		); data.append( "\'" + isCombined() + "\'," ); // bool default false,
        insert.append(  "basionymietry_unknown_flag, "  ); data.append( "\'" + isBasionymietryUnkown() + "\'," ); // bool default true,
        // -- status
        insert.append(  "genus_flag, " 			); data.append( "\'" + isGenus() + "\'," ); // bool default false,
        insert.append(  "species_flag, " 		); data.append( "\'" + isSpecies() + "\'," ); // bool default false,
        insert.append(  "subspecies_flag, " 		); data.append( "\'" + isSubSpecies() + "\'," ); // bool default false,
        insert.append(  "variety_flag, "		); data.append( "\'" + isVariety() + "\'," ); // bool default false,
        insert.append(  "form_flag, "			); data.append( "\'" + isForm() + "\'," ); // bool default false,
        insert.append(  "status_unknown_flag, "         ); data.append( "\'" + isStatusUnkown() + "\'," ); // bool default true,
        // -- validy
        insert.append(  "valid_flag, "			); data.append( "\'" + isValid() + "\'," ); // bool default false,
        insert.append(  "nomen_nudum_flag, " 		); data.append( "\'" + isNomenNudum() + "\'," ); // bool default false,
        insert.append(  "nomen_illeg_flag, " 		); data.append( "\'" + isNomenIlleg() + "\'," ); // bool default false,
        insert.append(  "nomen_invalid_flag, "		); data.append( "\'" + isNomenInvalid() + "\'," ); // bool default false,
        insert.append(  "auct_flag, "       		); data.append( "\'" + isAuct() + "\'," ); // bool default false,
        insert.append(  "agg_flag, "			); data.append( "\'" + isAgg() + "\'," ); // bool default false,
        insert.append(  "ined_flag, " 			); data.append( "\'" + isIned() + "\'," ); // bool default false,
        // --
        // insert.append(  "genus_id, "			); data.append( "\'" +  + "\'," ); // int4 NOT NULL,
        insert.append(  "genus, "			); data.append( "\'" + genus + "\'," ); // character varying( 256 ),
        insert.append(  "species, "			); data.append( "\'" + species + "\'," ); // character varying( 256 ) NOT NULL,
        insert.append(  "subspecies, "			); data.append( "\'" + subSpecies + "\'," ); // character varying( 256 ),
        insert.append(  "variety, "			); data.append( "\'" + variety + "\'," ); // character varying( 256 ),
        insert.append(  "form, "			); data.append( "\'" + form + "\'," ); // character varying( 256 ),
        insert.append(  "name_id, "			); data.append( "\'" + author + "\'," ); // character varying( 256 ),
        insert.append(  "first_name_id, "    		); data.append( "\'" + firstDiscription + "\'," ); // character varying( 256 ),

        insert.append(  "reference, "      		); data.append( "\'" + reference + "\'," ); // text,
        // --
        insert.append(  "typus, "			); data.append( "\'" + typus + "\'," ); // int4,
        insert.append(  "literature_taxon, "		); data.append( "\'" + literatureTaxon + "\'," ); // int4,
        insert.append(  "literature_flor, "		); data.append( "\'" + literatureFlor + "\'," ); // int4,
        insert.append(  "location, "			); data.append( "\'" + location + "\'," ); // int4,
        // --
        insert.append(  "comment"       		); data.append( "\'" + comment +  "\'" ); // text,
        insert.append(  " ) values " );

        return insert.toString() + "( " + data.toString() + " );";
    }
    /*
    int getSynonymMarker( java.lang.String lastName, java.lang.String thisName )
    {
        int     cutLastLength = 0;
        int     cutThisLength = 0;
        
        if( lastName.compareTo( thisName ) == 0 )
        {
            return 3;
        }
        
        //----------------------------------------------------------------------
        // er, ra, rum
        //----------------------------------------------------------------------
        if( lastName.endsWith( "er" ) )  { cutLastLength = 2; }
        if( lastName.endsWith( "ra" ) )  { cutLastLength = 2; }
        if( lastName.endsWith( "rum" ) ) { cutLastLength = 3; }
        
        if( thisName.endsWith( "er" ) )  { cutThisLength = 2; }
        if( thisName.endsWith( "ra" ) )  { cutThisLength = 2; }
        if( thisName.endsWith( "rum" ) ) { cutThisLength = 3; }
        
        if( cutLastLength > 0 && cutThisLength > 0 )
        {   
            java.lang.String lastStr = lastName.substring( 0, lastName.length() - cutLastLength );
            java.lang.String thisStr = thisName.substring( 0, thisName.length() - cutThisLength );
            
            if( lastStr.compareTo( thisStr ) == 0 )
            {
                return 3;
            }    
        }
        //----------------------------------------------------------------------
        // a, us, um
        //----------------------------------------------------------------------
        if( lastName.endsWith( "a" ) )  { cutLastLength = 1; }
        if( lastName.endsWith( "us" ) ) { cutLastLength = 2; }
        if( lastName.endsWith( "um" ) ) { cutLastLength = 2; }
        
        if( thisName.endsWith( "a" ) )  { cutThisLength = 1; }
        if( thisName.endsWith( "us" ) ) { cutThisLength = 2; }
        if( thisName.endsWith( "um" ) ) { cutThisLength = 2; }
        
        if( cutLastLength > 0 && cutThisLength > 0 )
        {   
            java.lang.String lastStr = lastName.substring( 0, lastName.length() - cutLastLength );
            java.lang.String thisStr = thisName.substring( 0, thisName.length() - cutThisLength );
            
            if( lastStr.compareTo( thisStr ) == 0 )
            {
                return 3;
            }    
        }
        //----------------------------------------------------------------------
        // is, e
        //----------------------------------------------------------------------        
        cutLastLength = 0;
        cutThisLength = 0;
        
        if( lastName.endsWith( "e" ) )  { cutLastLength = 1; }
        if( lastName.endsWith( "is" ) ) { cutLastLength = 2; }
        
        if( thisName.endsWith( "e" ) )  { cutThisLength = 1; }
        if( thisName.endsWith( "is" ) ) { cutThisLength = 2; }
        
        if( cutLastLength > 0 && cutThisLength > 0 )
        {   
            java.lang.String lastStr = lastName.substring( 0, lastName.length() - cutLastLength );
            java.lang.String thisStr = thisName.substring( 0, thisName.length() - cutThisLength );
            
            if( lastStr.compareTo( thisStr ) == 0 )
            {
                return 3;
            }    
        }
        return 2;
    } 
     */   
}
