/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Title:        Checklist
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:      University Bayreuth
 * @author Dieter Neubacher
 * @version 1.0
 */

package net.metadiversity.diversity.navigator.checklists.taxon;

public class TaxonFlags
{
public final static int UNKOWN         = 0x800;


public final static int NON_SYNONYM    = 0x01;
public final static int SYNONYM        = 0x02;

private boolean NonSynonym;         // a    accepted name
private boolean Synonym;            // s    synonym   

// private boolean SynonymietryUnkown;  // ??

public final static int BASIONYM       = 0x01;
public final static int COMBINED       = 0x02;

private boolean Basionym;           // b    basionym
private boolean Combined;           // c    combined name

// private boolean BasinomietryUnkown;  // ??

public final static int FLAG_STRING_LENGTH = 4;

//------------------------------------------------------------------------------
// Stauts
//------------------------------------------------------------------------------
private int status;

public final static int GENUS          = 0x01;
public final static int SPECIES        = 0x02;
public final static int SUB_SPECIES    = 0x04;
public final static int VARIETY        = 0x08;
public final static int FORM           = 0x10;

private boolean Genus;        // g
private boolean Species;      // z
private boolean SubSpecies;   // t
private boolean Variety;      // r
private boolean Form;         // f
private boolean StatusUnkown; // ?

//------------------------------------------------------------------------------
// validy
//------------------------------------------------------------------------------

private int validy;

public final static int VALID          = 0x001;
public final static int NOMEN_NUDUM    = 0x002;
public final static int NOMEN_ILLEG    = 0x004;
public final static int NOMEN_INVALID  = 0x008;
public final static int NOMEN_CONS     = 0x010;
public final static int NOMEN_REJ      = 0x020;
public final static int NOMEN_EXCL     = 0x040;
public final static int NOMEN_CONFUS   = 0x080;
public final static int COMB_INVAL     = 0x100;
public final static int INED           = 0x200;
// ??? Concept
public final static int AUCT           = 0x400;
public final static int AGG            = 0x800;

private boolean Valid;        // [g,z,t,r,f] i
private boolean NomenNudum;   // [g,z,t,r,f] n
private boolean NomenIlleg;   // [g,z,t,r,f] p
private boolean NomenInvalid; // [g,z,t,r,f] y
private boolean NomenCons;    // [g,z,t,r,f] o
private boolean NomenRej;     // [g,z,t,r,f] j
private boolean NomenExcl;    // [g,z,t,r,f] 
private boolean NomenConfus;  // [g,z,t,r,f] 
private boolean CombInval;    // [g,z,t,r,f] 
private boolean Ined;         // [g,z,t,r,f] u
private boolean ValidyUnkown; // [g,z,t,r,f] ?
// ??? Concept
private boolean Auct;         // [g,z,t,r,f] a
private boolean Agg;          // [g,z,t,r,f] e

    //--------------------------------------------------------------------------
    // Synonymietry: Synonym / NonSynonym (exclusive)
    //--------------------------------------------------------------------------
    public boolean isNonSynonym()
    {
        return NonSynonym;
    }
    public boolean isSynonym()
    {
        return Synonym;
    }
    public boolean isSynonymietryUnkown()
    {
        return NonSynonym | Synonym;
    }
    public void setSynonymietryUnkown()
    {
        NonSynonym  = false;
        Synonym     = false;
    }
    public void setNonSynonym()
    {
        NonSynonym  = true;
        Synonym     = false;
    }
    public void setSynonym()
    {
        NonSynonym  = false;
        Synonym     = true;
    }
    //--------------------------------------------------------------------------
    //  Basionymietry: Basionym / Combiend (exclusive)
    //--------------------------------------------------------------------------
    public boolean isBasionym()
    {
        return Basionym;
    }
    public boolean isCombined()
    {
        return Combined;
    }
    public boolean isBasionymietryUnkown()
    {
        return Basionym | Combined;
    }
    public void setBasionymietryUnkown()
    {
        Basionym    = false;
        Combined    = false;
    }
    public void setBasionym()
    {
        Basionym    = true;
        Combined    = false;
    }
    public void setCombined()
    {
        Basionym    = false;
        Combined    = true;
    }
    //--------------------------------------------------------------------------
    //  Status
    //--------------------------------------------------------------------------

    private void clearStatus()
    {
        Genus       = false;
        Species     = false;
        SubSpecies  = false;
        Variety     = false;
        Form        = false;
        status      = 0;
    }
    public boolean isGenus()
    {
        return Genus;
    }
    public void setGenus()
    {
        Genus = true;
    }
    public boolean isSpecies()
    {
        return Species;
    }
    public void setSpecies()
    {
        Species = true;
    }
    public boolean isSubSpecies()
    {
        return SubSpecies;
    }
    public void setSubSpecies()
    {
        SubSpecies = true;
    }
    public boolean isVariety()
    {
        return Variety;
    }
    public void setVariety()
    {
        Variety = true;
    }
    public boolean isForm()
    {
        return Form;
    }
    public void setForm()
    {
        Form = true;
    }
    public boolean isStatusUnkown()
    {
        return StatusUnkown;
    }
    public void setStatusUnkown()
    {
        StatusUnkown = true;
    }
    //--------------------------------------------------------------------------
    // Validy
    //--------------------------------------------------------------------------

    private void clearValidy()
    {
        Valid           = false;
        NomenNudum      = false;
        NomenIlleg      = false;
        NomenInvalid    = false;
        NomenRej        = false;
        NomenExcl       = false;
        NomenConfus     = false;
        NomenCons       = false;
        Ined            = false;
        CombInval       = false;
        // ??? Concept
        Auct            = false;
        Agg             = false;
        validy          = 0;
    }
    public boolean isValid()
    {
        return Valid;
    }
    public boolean isNomenNudum()
    {
        return NomenNudum;
    }
    public boolean isNomenIlleg()
    {
        return NomenIlleg;
    }
    public boolean isNomenInvalid()
    {
        return NomenInvalid;
    }
    public boolean isNomenRej()
    {
        return NomenRej;
    }
    public boolean isNomenExcl()
    {
        return NomenExcl;
    }
    public boolean isNomenConfus()
    {
        return NomenConfus;
    }
    public boolean isNomenCons()
    {
        return NomenCons;
    }
    public boolean isIned()
    {
        return Ined;
    }
    public boolean isCombInval()
    {
        return CombInval;
    }
    // ??? Concept
    public boolean isAgg()
    {
        return Agg;
    }
    public boolean isAuct()
    {
        return Auct;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void clearAll()
    {
        NonSynonym  = false;
        Synonym     = false;
        Basionym    = false;
        Combined    = false;
        clearStatus();
        clearValidy();
    }

    private int getStatus()
    {
        return status;
    }
    private int getValidy()
    {
        return validy;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    
    public boolean isStartUnit()
    {
        //-------------------------------------
        // Synonymietry:  Synonym / NonSynonym
        //-------------------------------------
        if( isSynonym() )
            return false;
        else if( isNonSynonym() )
            return true;  
        else
            return false;

    }        
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    
    public java.lang.String getText()
    {
        java.lang.StringBuffer b = new java.lang.StringBuffer();

        //-------------------------------------
        // Synonymietry:  Synonym / NonSynonym
        //-------------------------------------
        if( isSynonym() )
            b.append( "s" );
        else if( isNonSynonym() )
            b.append( "a" );
        else
            b.append( "?" );

        //-------------------------------------
        // Basionymietry: Basionym / Combined
        //-------------------------------------
        if( isBasionym() )
            b.append( "b" );
        else if( isCombined() )
            b.append( "c" );
        else
            b.append( "?" );

        // Status

        switch( getStatus() )
        {
        case GENUS :        b.append( "g" ); break;
        case SPECIES :      b.append( "z" ); break;
        case SUB_SPECIES :  b.append( "t" ); break;
        case VARIETY :      b.append( "r" ); break;
        case FORM :         b.append( "f" ); break;

        default :           b.append( "?" ); break;
        }
        //----------------------------------------------------------------------
        // Validy
        //----------------------------------------------------------------------
        switch( getValidy() )
        {
        case COMB_INVAL :   b.append( "d" ); break;
        case INED :         b.append( "u" ); break;
        case NOMEN_CONFUS : b.append( "m" ); break;
        case NOMEN_CONS :   b.append( "o" ); break;
        case NOMEN_EXCL :   b.append( "x" ); break;
        case NOMEN_ILLEG :  b.append( "p" ); break;
        case NOMEN_INVALID: b.append( "y" ); break;
        case NOMEN_NUDUM :  b.append( "n" ); break;
        case NOMEN_REJ :    b.append( "j" ); break;
        case VALID :        b.append( "i" ); break;        
        case UNKOWN :       b.append( "?" ); break;

        default :           b.append( "*" ); break;
        }

        return b.toString();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public boolean set( String s )
    {
        String str = s.trim();
        char   ch;

        clearAll();

        if( str.length() != FLAG_STRING_LENGTH )
            return false;
            
        //----------------------------------------------------------------------
        // synonymietry
        //----------------------------------------------------------------------
        ch = str.charAt( 0 );

        switch( ch )
        {
            case 'a':   setNonSynonym();
                        break;
            case 's':   setSynonym();
                        break;
            case '?':   setSynonymietryUnkown();
                        break;
            default :             
                        return false;
        }                
        //----------------------------------------------------------------------
        // basionymietry
        //----------------------------------------------------------------------

        ch = str.charAt( 1 );
        switch( ch )
        {
            case 'b':   setBasionym();
                        break;
            case 'c':   setCombined();
                        break;
            case '?':   setBasionymietryUnkown();
                        break;
            default :             
                        return false;
        }                
        //----------------------------------------------------------------------
        // taxonomic rank
        //----------------------------------------------------------------------
        ch = str.charAt( 2 );

        switch( ch )
        {
        case 'g':   Genus = true;       status |= GENUS;
                    break;
        case 'z':   Species = true;     status |= SPECIES;
                    break;
        case 't':   SubSpecies = true;  status |= SUB_SPECIES;
                    break;
        case 'r':   Variety = true;     status |= VARIETY;
                    break;
        case 'f':   Form = true;        status |= FORM;
                    break;
        case '?':
                    StatusUnkown = true;status |= UNKOWN;
                    break;
        default :   return false;
        }

        //----------------------------------------------------------------------
        // Validy (Nomenclatureal Status )
        //----------------------------------------------------------------------
        ch = str.charAt( 3 );

        switch( ch )
        {
        case 'd':   CombInval = true;   validy |= COMB_INVAL;
                    break;
        case 'u':   Ined = true;        validy |= INED;
                    break;
        case 'm':   NomenConfus = true; validy |= NOMEN_CONFUS;
                    break;
        case 'o':   NomenCons = true;   validy |= NOMEN_CONS;
                    break;
        case 'x':   NomenExcl = true;   validy |= NOMEN_EXCL;
                    break;
        case 'p':   NomenIlleg = true;  validy |= NOMEN_ILLEG;
                    break;
        case 'y':   NomenInvalid = true;validy |= NOMEN_INVALID;
                    break;
        case 'n':   NomenNudum = true;  validy |= NOMEN_NUDUM;
                    break;
        case 'j':   NomenRej = true;    validy |= NOMEN_REJ;
                    break;                    
        case 'i':   Valid = true;       validy |= VALID;
                    break;
        case '?':   ValidyUnkown = true;validy |= UNKOWN;
                    break;
        case '*':   validy = 0;
                    break;
        default :   return false;
        }
        return true;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public TaxonFlags()
    {
        clearAll();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public TaxonFlags( String s )
    {
        set( s );
    }
}