/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package net.metadiversity.diversity.navigator.connection;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class ConnectionDialog extends JDialog 
{
	/**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = 1L;

	boolean OK = false;
        
	javax.swing.JTextField      edName, edDescription;
	javax.swing.JComboBox		edDriver, edUrl;
	javax.swing.JTextField		edUser;
    javax.swing.JPasswordField  edPwd;
        
    String [] driverStrings = new String[] { "", 
    										 "org.postgresql.Driver", 
    										 "org.hsqldb.jdbcDriver", 
											 "net.sourceforge.jtds.jdbc.Driver" 
    									   };
    String [] urlStrings = new String[] { "", 
    		                              "jdbc:postgresql://<server>:5432/<database>", 
    		                              "jdbc:hsqldb:hsql://<server>:9001/<database>", 
										  "jdbc:jtds:sqlserver://<server>:1433/<database>", 
										  "jdbc:jtds:sybase://<server>:7100/<database>", 
										  "jdbc:jtds:<server_type>://<server>[:<port>][/<database>][;<property>=<value>[;...]]" 
										};
    
        final static int marginWidth =  10;
        final static int lableWidth  = 120;
        final static int lableHeight =  30;
        final static int inputWidth  = 480;
        final static int inputHeight =  25;
        
	public ConnectionDialog() 
        {
		this(null);
	}
	public boolean getOK()          { return OK; }
	public String getName()         { return edName.getText().trim(); }
	public String getDescription()  { return edDescription.getText().trim(); }
	/*
	public String getDriver()       { return edDriver.getText(); }
	public String getUrl()          { return edUrl.getText(); }
	*/
	public String getDriver()       { return (( String ) edDriver.getSelectedItem()).trim(); }
	public String getUrl()          { return (( String ) edUrl.getSelectedItem()).trim(); }
	
	public String getUser()         { return edUser.getText().trim(); }
	public String getPwd()      
        { 
            return new String( edPwd.getPassword() ); 
        }

	public ConnectionDialog( ConnectionInfo conInfo ) 
        {
		super();
                
		setTitle( "Diversity Connection Info" );
                
                int y = 10; int increment = 25;               
		setBounds( 100, 100, 2 * marginWidth + lableWidth + inputWidth, 6 * lableHeight + 2 * y + lableHeight );
                
		getContentPane().setLayout( new BorderLayout() );

                //--------------------------------------------------------------
		//  Edit pane
                //--------------------------------------------------------------
		JPanel Edit = new JPanel();
                
		Edit.setLayout(null);
                
		JLabel lblName          = new JLabel( "Resource" );             // JLabel( "Connection name");
		JLabel lblDescription   = new JLabel( "Description");
		JLabel lblDriver        = new JLabel( "Driver class");
		JLabel lblUrl           = new JLabel( "Accesspoint" );          // new JLabel( "Database URL");
		JLabel lblUser          = new JLabel( "Database user");
		JLabel lblPwd           = new JLabel( "Database password");
                
		lblName.setBounds(          marginWidth, y, lableWidth, lableHeight ); y += increment;
		lblDescription.setBounds(   marginWidth, y, lableWidth, lableHeight ); y += increment;
		lblDriver.setBounds(        marginWidth, y, lableWidth, lableHeight ); y += increment;
		lblUrl.setBounds(           marginWidth, y, lableWidth, lableHeight ); y += increment;
		lblUser.setBounds(          marginWidth, y, lableWidth, lableHeight ); y += increment;
		lblPwd.setBounds(           marginWidth, y, lableWidth, lableHeight ); y += increment;
                
		Edit.add( lblName );
		Edit.add( lblDescription );
		Edit.add( lblDriver );
		Edit.add( lblUrl );
		Edit.add( lblUser );
		Edit.add( lblPwd );

		edName          = new javax.swing.JTextField();
		edDescription   = new javax.swing.JTextField();
		/*
		edDriver        = new javax.swing.JTextField();
		edUrl           = new javax.swing.JTextField();
		*/
		edDriver        = new javax.swing.JComboBox();
		edUrl           = new javax.swing.JComboBox();
		
		edUser          = new javax.swing.JTextField();
		edPwd           = new javax.swing.JPasswordField();
                
                y = 15;
		edName.setBounds(       marginWidth + lableWidth, y, inputWidth, inputHeight );  y += increment;
		edDescription.setBounds(marginWidth + lableWidth, y, inputWidth, inputHeight );  y += increment;
		edDriver.setBounds(     marginWidth + lableWidth, y, inputWidth, inputHeight );  y += increment;
		edUrl.setBounds(        marginWidth + lableWidth, y, inputWidth, inputHeight );  y += increment;
		edUser.setBounds(       marginWidth + lableWidth, y, inputWidth, inputHeight );  y += increment;
		edPwd.setBounds(        marginWidth + lableWidth, y, inputWidth, inputHeight );  y += increment;

                if( conInfo != null ) 
                {
                    edName.setText( conInfo.getName() );
                    edName.setEnabled(false);
                        
                    edDescription.setText(  conInfo.getDescription() );
                    /*
                    edDriver.setText(       conInfo.getDriver() );
                    edUrl.setText(          conInfo.getUrl() );
                    */
                    
                    
            		driverStrings[ 0 ] 	= conInfo.getDriver();
            		urlStrings[ 0 ] 	= conInfo.getUrl();
            		
                    edUser.setText(         conInfo.getUser() );
                    edPwd.setText(          conInfo.getPwd() );
		}
  		edDriver.setModel( new javax.swing.DefaultComboBoxModel( driverStrings ) );
   		edUrl.setModel(    new javax.swing.DefaultComboBoxModel( urlStrings ) );
   		
   		edDriver.setEditable( true );
   		edUrl.setEditable( true );
   		
        edDriver.setSelectedIndex( 0 );
        edUrl.setSelectedIndex( 0 );
        
        		
		Edit.add( edName );
		Edit.add( edDescription );
		Edit.add( edDriver );
		Edit.add( edUrl );
		Edit.add( edUser );
		Edit.add( edPwd );

		getContentPane().add( Edit, BorderLayout.CENTER );
                
                //--------------------------------------------------------------
		// Buttons
                //--------------------------------------------------------------
		Box Buttons = new Box(BoxLayout.X_AXIS);
		Buttons.add(Box.createGlue());
		JButton cmdOK = new JButton("OK");
		cmdOK.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {OK = true; setVisible(false);}
		});
		JButton cmdCancel = new JButton("Cancel");
		cmdCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {OK = false; setVisible(false);}
		});
		Buttons.add(cmdCancel);		
		Buttons.add(Box.createHorizontalStrut(5));
		Buttons.add(cmdOK);
		getContentPane().add(Buttons, BorderLayout.SOUTH);
	}
}
