/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package net.metadiversity.diversity.navigator.connection;

import org.apache.log4j.Logger;

//import net.metadiversity.diversity.navigator.util.TestUrlConnection;

public class ConnectionInfo 
{
    static Logger logger = Logger.getLogger( ConnectionInfo.class.getName() ); 
    
    private String name;
    private String description;
    private String driver;
    private String url;
    private String user;
    private String pwd;
    // parsed field from Database-URL
    private String serverName;
    private String databaseName;
    private String databasePort;
    
    private static boolean driverLoaded = false;
    
    private static java.util.HashSet loadedDriverHashSet = null;
    
    /** Conection Info
     * @param name
     * @param driver
     * @param url
     * @param user
     * @param pwd
     */    
    public ConnectionInfo( String name, String description, String driver, String url, String user, String pwd ) 
    {
        // logger.debug( name + " " +  driver + " " + url + " " + user + " " + pwd );
        logger.debug( name + " " + description + " " +  driver + " " + url + " " + user );
        this.name   = name;
        this.description = description;
        this.driver = driver;
        this.url    = url;
        this.user   = user;
        this.pwd    = pwd;
        
        parseUrl( url );
        
        if( loadedDriverHashSet == null )
        {
            // init hash for loaded drivers
            loadedDriverHashSet = new java.util.HashSet( 10, (float)0.75 );
        }
        
        try
        {
            // ??? only works whis one database-Dirver
            // should be changed
            // if( driverLoaded == false )
            //     driverLoaded = true;
            if( loadedDriverHashSet.contains( driver ) != true )
            {
                Class.forName( driver );
                loadedDriverHashSet.add( driver );
                logger.info( "Load Driver: " + driver + " ..." );                
            }
        }
        catch( java.lang.ClassNotFoundException ee )
        {
        	logger.error( "ClassNotFoundException: " );
        	logger.error( ee.getMessage() );
        }
    }
    //----------------------------------------------------------------------------
    //----------------------------------------------------------------------------
    private void parseUrl( String url )
    {
        // format jdbc:postgresql://141.84.65.158:5432/DiversityDescriptions_LIASimages
    	logger.info( "parseUrl: " + url );
        try
        {
            int startPos = url.indexOf( "://" ) + 3;
            int endPos   = url.indexOf( ':' , startPos );
            serverName   = url.substring( startPos, endPos );
            startPos = ++endPos; 
            endPos = url.indexOf( '/' , startPos );
            databasePort = url.substring( startPos, endPos );
            startPos = ++endPos; 
            databaseName = url.substring( startPos );
            
            boolean showParams = false;
            
            if( showParams )
            {
                logger.info( "Database server : " + serverName );
                logger.info( "Database name :   " + databaseName );
                logger.info( "Database port :   " + databasePort );
            }
        }
        catch( java.lang.IndexOutOfBoundsException ex )
        {
            logger.error( "invalid database url: " + url );
        }
    }
    public String getServerName()
    {
        return serverName;
    }
    public String getDatabasePort()
    {
        return databasePort;
    }
    public String getDatabaseName()
    {
        return databaseName;
    }
    /** conection name
     * @return
     */    
    public String getName()     
    { 
        return name; 
    }
    /** conection description
     * @return
     */    
    public String getDescription()     
    { 
        return description; 
    }
    /** conection Driver
     * @return
     */    
    public String getDriver()   
    { 
        return driver; 
    }
    /** Conection URL
     * @return
     */    
    public String getUrl()      
    { 
        return url; 
    }
    /** Conection user
     * @return
     */    
    public String getUser()     
    { 
        return user; 
    }
    /** Conection password
     * @return
     */    
    public String getPwd()      
    { 
        return pwd; 
    }
    //----------------------------------------------------------------------
    // create a new connection to database. First test connection to server 
    // is posible.
    //----------------------------------------------------------------------
    public java.sql.Connection getConnection()  throws java.sql.SQLException
    {
/*
	try
	{
            Class.forName( "org.postgresql.Driver" );
	    logger.info( "Load PostgreSQL Driver..." );
	}
	catch( java.lang.ClassNotFoundException ee )
        {
	    logger.error( "ClassNotFoundException: " );
	    logger.error( ee.getMessage() );
	    System.err.print( "ClassNotFoundException: " );
	    System.err.println( ee.getMessage() );
	}
 */
        java.lang.String url = getUrl();
        url = url.substring( url.indexOf( "//" ) );

/***        
        TestUrlConnection test = new TestUrlConnection( "http:" + url );
        if( test.isConnected() )
        {
            return java.sql.DriverManager.getConnection( getUrl(), getUser(), getPwd() );
        }
        else
        {
            return null;
        }
 */
        
        // java.sql.DriverManager.setLoginTimeout( 200 );
        // logger.info( "Connection timeout is set to: " + java.sql.DriverManager.getLoginTimeout() + " sec." );
        
        return java.sql.DriverManager.getConnection( getUrl(), getUser(), getPwd() );
    }
}
