/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * CharacterStateDescriptionEntry.java
 *
 * Created on 3. M�rz 2004, 15:10
 */

package net.metadiversity.diversity.navigator.db.delta_editor;

import org.apache.log4j.Logger;

/**
 *
 * @author  didi
 */
public class CharacterStateDescriptionEntry extends java.lang.Object implements java.lang.Comparable
{
    static Logger logger = Logger.getLogger( CharacterStateDescriptionEntry.class.getName() ); 
    
    private int     iid;
    private int     cid;
    private String  modifier;
    private String  cs;
    private double  x;
    private String  text;
    private String  notes;
    private int     seq;
    
    // private String  characterType;  // TE, OM, UM,...
           
    
    private static String SQL_QUOTE = "'";
    
    
    /** Creates a new instance of CharacterStateDescriptionEntry */
    public CharacterStateDescriptionEntry()
    {
    }
    public CharacterStateDescriptionEntry( int iid, int cid, String modifier, String cs, double x, String text, String notes, int seq )
    {
        this.iid        = iid;
        this.cid        = cid;
        this.modifier   = (modifier == null) ? "" : modifier.trim();
        this.cs         = (cs == null) ? "" : cs;
        this.x          = x;
        this.text       = (text == null) ? "" : text.trim();
        this.notes      = (notes == null) ? "" : notes.trim();
        this.seq        = seq;        
        // this.characterType = characterType;
    }
    public CharacterStateDescriptionEntry( int iid, int cid, String cs )
    {
        this.iid        = iid;
        this.cid        = cid;
        this.modifier   = "";
        this.cs         = cs;
        this.x          = 0;
        this.text       = "";
        this.notes      = "";
        this.seq        = 0;
        // this.characterType = characterType;
    }
    //--------------------------------------------------------------------------
    public int getIID()
    {
        return iid;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int getCID()
    {
        return cid;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getModifier()
    {
        {
            return modifier;
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getModifierSqlValue()
    {
        if( modifier.compareTo( "" ) == 0 )
        {
            return "null";
        }
        else
        {
            return SQL_QUOTE + modifier + SQL_QUOTE;
        }        
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getCS()
    {
        return cs;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getCSSqlValue()
    {
        return SQL_QUOTE + cs + SQL_QUOTE;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public double getX()
    {
        return x;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getXSqlValue()
    {
        return SQL_QUOTE + x + SQL_QUOTE;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getText()
    {
        return text;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getTextSqlValue()
    {
        if( text.compareTo( "" ) == 0 )
        {
            return "null";
        }
        else
        {
            return SQL_QUOTE + text + SQL_QUOTE;
        }        
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getNotes()
    {
        return notes;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public String getNotesSqlValue()
    {
        if( notes.compareTo( "" ) == 0 )
        {
            return "null";
        }
        else
        {
            return SQL_QUOTE + notes + SQL_QUOTE;
        }        
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int getSEQ()
    {
        return seq;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setIID( int val )
    {
        iid = val;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setCID( int val )
    {
        cid = val;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setModifier( String str )
    {
        modifier = str.trim();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setCS( String str )
    {
        cs = str;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setX( double val )
    {
        x = val;
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setText( String str )
    {
        text = str.trim();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setNotes( String str )
    {
        notes = str.trim();
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setSEQ( int val )
    {
        seq = val;
    }
    //--------------------------------------------------------------------------
    public boolean isTextElement()
    {
        // return characterType.equals( "TE" );
        return cs.equals( "TE" );
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public int compareTo( Object o )
    {
        String cs = ((CharacterStateDescriptionEntry) o ).cs;
        int x, y;
        try
        {          
            x = Integer.parseInt( cs );
        }
        catch( java.lang.NumberFormatException ex )
        {          
            if( cs.equalsIgnoreCase("-" ) )       x = Integer.MAX_VALUE -1;
            else
            if( cs.equalsIgnoreCase("U" ) )       x = Integer.MAX_VALUE -2;
            else
            if( cs.equalsIgnoreCase("V" ) )       x = Integer.MAX_VALUE -3;
            else
            if( cs.equalsIgnoreCase("TE" ) )      x = Integer.MAX_VALUE -4;
            else
            if( cs.equalsIgnoreCase("Min" ) )     x = Integer.MAX_VALUE -5;
            else
            if( cs.equalsIgnoreCase("-Low" ) )    x = Integer.MAX_VALUE -6;
            else
            if( cs.equalsIgnoreCase("Mean" ) )    x = Integer.MAX_VALUE -7;
            else
            if( cs.equalsIgnoreCase("+High" ) )   x = Integer.MAX_VALUE -8;
            else
            if( cs.equalsIgnoreCase("Max" ) )     x = Integer.MAX_VALUE -9;
            else
            x = Integer.MAX_VALUE;
        }
        cs = this.cs;
        try
        {          
            y = Integer.parseInt( cs );
        }
        catch( java.lang.NumberFormatException ex )
        {  
            if( cs.equalsIgnoreCase("-" ) )  y = Integer.MAX_VALUE -1;
            else
            if( cs.equalsIgnoreCase("U" ) )  y = Integer.MAX_VALUE -2;
            else
            if( cs.equalsIgnoreCase("V" ) )  y = Integer.MAX_VALUE -3;
            else
            if( cs.equalsIgnoreCase("TE" ) ) y = Integer.MAX_VALUE -4;
            else
            if( cs.equalsIgnoreCase("Min" ) ) y = Integer.MAX_VALUE -5;
            else
            if( cs.equalsIgnoreCase("-Low" ) ) y = Integer.MAX_VALUE -6;
            else
            if( cs.equalsIgnoreCase("Mean" ) ) y = Integer.MAX_VALUE -7;
            else
            if( cs.equalsIgnoreCase("+High" ) ) y = Integer.MAX_VALUE -8;
            else
            if( cs.equalsIgnoreCase("Max" ) ) y = Integer.MAX_VALUE -9;
            else
            y = Integer.MAX_VALUE;
        }
        if( x == y ) return 0;
        if( x < y )  return 1;
        else return -1;
    }
}
