/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * ChecklistsGisVisualisation.java
 *
 * Created on 13. August 2003, 16:40
 */

package net.metadiversity.diversity.navigator.gui;

//import net.metadiversity.diversity.navigator.checklists.database.DBChecklist;

//import java.net.URL;          // F�r Datenbank-URL
import java.sql.*;            // JDBC

import org.apache.log4j.Logger;
//import org.apache.log4j.Priority;
import org.apache.log4j.Level;

import org.apache.axis.client.Call;
import org.apache.axis.client.Service;
import org.apache.axis.encoding.XMLType;
//import org.apache.axis.utils.Options;

import javax.xml.rpc.ParameterMode;

/**
 *
 * @author  Dieter Neubacher
 */
public class ChecklistsGisVisualisation extends javax.swing.JPanel
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3905808569440875321L;

    static Logger logger = Logger.getLogger( ChecklistsGisVisualisation.class.getName() ); 
       
    // Variables declaration
    
//    private final static String EMPTY_MAP_URL = "http://localhost/icons/grass-empty-map.jpg";
//    private final static String GRASS_MAP_URL = "http://localhost/icons/grass-map.jpg";
    
    final static java.lang.String bundleFullPath = "net/metadiversity/diversity/navigator/gui/ChecklistWebServiceBundle";
    static java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle( bundleFullPath );
    
    private final static String GRASS_EMPTY_MAP_NAME    = bundle.getString( "GRASS_EMPTY_MAP_NAME" );
    private final static String GRASS_MAP_LOCATION      = bundle.getString( "GRASS_MAP_LOCATION" );    
    private final static String GRASS_MAP_EXTENSION     = bundle.getString( "GRASS_MAP_EXTENSION" );    
    private final static String WEB_SERVICE_ENDPOINT    = bundle.getString( "WEB_SERVICE_ENDPOINT" );            
    // GUI Strings
    private final static String GENUS_LABEL             = bundle.getString( "LABEL_GENUS" );
    private final static String SPECIES_LABEL           = bundle.getString( "LABEL_SPECIES" );
    private final static String SHOW_IMAGE_LABEL        = bundle.getString( "LABEL_SHOW_IMAGE" );
    
    // GUI elements
    private javax.swing.JPanel      mainPanel;
    private javax.swing.JPanel      jPanel;     // selection
    private javax.swing.JLabel      jLabelGenus;
    private javax.swing.JButton     jButtonShowImage;
    private javax.swing.JLabel      jLabelEpithet;
    private javax.swing.JComboBox   jComboBoxGenus;
    private javax.swing.JComboBox   jComboBoxEpithet;
    //----------------------------------------------
    private java.lang.String genusSelection;
    private java.lang.String epithetSelection;
    
    private java.util.Vector genusModel;
    private java.util.Vector epithetModel;
    
    private ImageViewer              imageViewer;
//    private DBChecklist db;
    private Connection con;     // Checklists DB-Connection
    
    /** Creates new form ChecklistsGisVisualisation */
    public ChecklistsGisVisualisation( Connection connection )
    {
        genusModel = new java.util.Vector();
        epithetModel = new java.util.Vector();
        epithetModel.add( "ALL" );
        con = connection;
        try
        {
            if( con == null )
            {
                logger.error( "Can't create connection " );
            }
	}
	catch( Exception ex )
	{
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	} 
        genusModel = setupGenusNames();        
        initComponents();
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    protected void finalize() throws java.lang.Throwable
    {
        // close DataBaseConnection
        if( con != null )
        {
            logger.info( "ChecklistGisVisualisation: close database connection" );
            con.close();
        }    
        super.finalize();        
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    private void initComponents()
    {
        mainPanel           = new javax.swing.JPanel();
        jPanel              = new javax.swing.JPanel();
        jLabelGenus         = new javax.swing.JLabel();
        jComboBoxGenus      = new javax.swing.JComboBox();
        jLabelEpithet       = new javax.swing.JLabel();
        jComboBoxEpithet    = new javax.swing.JComboBox();
        jButtonShowImage    = new javax.swing.JButton();
        try
        {
            // load empty map
            imageViewer     = new ImageViewer( new java.net.URL( GRASS_MAP_LOCATION + GRASS_EMPTY_MAP_NAME + GRASS_MAP_EXTENSION) );
        }    
        catch( java.net.MalformedURLException ex )
        {
            logger.error( GRASS_MAP_LOCATION + GRASS_EMPTY_MAP_NAME );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        //-----------------
        // Genus selection
        //-----------------
        jLabelGenus.setText( GENUS_LABEL );
        jPanel.add( jLabelGenus );
        jComboBoxGenus.setEditable( true );
        jComboBoxGenus.setModel( new javax.swing.DefaultComboBoxModel( genusModel ) );
        jComboBoxGenus.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                jComboBoxGenusActionPerformed( evt );
            }
        });
        jPanel.add( jComboBoxGenus );
        //---------------------------
        // Species Epithet selection
        //---------------------------
        jLabelEpithet.setText( SPECIES_LABEL );
        jPanel.add( jLabelEpithet );
        jComboBoxEpithet.setModel( new javax.swing.DefaultComboBoxModel( epithetModel ) );
        jComboBoxEpithet.setEditable( true );
        jPanel.add( jComboBoxEpithet );
        //-------------------------------
        // get the image wia WEB-Service
        //-------------------------------
        jButtonShowImage.setText( SHOW_IMAGE_LABEL );
        jButtonShowImage.addActionListener( new java.awt.event.ActionListener()
        {
            public void actionPerformed( java.awt.event.ActionEvent evt )
            {
                jButtonShowImageActionPerformed(evt);
            }
        });
        jPanel.add( jButtonShowImage );
        
        // mainPanel
        this.setLayout( new java.awt.BorderLayout() );
        this.add( jPanel, java.awt.BorderLayout.NORTH );
        this.add( imageViewer, java.awt.BorderLayout.CENTER );
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    private void jComboBoxGenusActionPerformed(java.awt.event.ActionEvent evt)
    {
        // get epithet Selections List
        logger.debug( "Genus Action: " + evt );
        genusSelection = (java.lang.String) jComboBoxGenus.getSelectedItem();
        epithetModel = setupEpithetNames();
        jComboBoxEpithet.setModel( new javax.swing.DefaultComboBoxModel( epithetModel ) );        
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    private void jButtonShowImageActionPerformed(java.awt.event.ActionEvent evt)
    {
        // Build a new grass-gis- map 
        boolean debugFlag = false;
        StringBuffer buffer = new StringBuffer();
        
        epithetSelection = (java.lang.String) jComboBoxEpithet.getSelectedItem();
        
	try
	{                
	    String cmd = null;
            if( epithetSelection.compareTo( "ALL" ) == 0 )
            {
                cmd = "SELECT DISTINCT \"ChecklistId\" FROM \"checklists_data\"  WHERE \"Genus\" = '" + genusSelection + "' AND \"Code\" LIKE 'a%' ORDER BY \"ChecklistId\" ASC;";
            }
            else
            {
                cmd = "SELECT DISTINCT \"ChecklistId\" FROM \"checklists_data\"  WHERE \"Genus\" = '" + genusSelection + "' AND \"Species\" = '" + epithetSelection + "' AND \"Code\" LIKE 'a%' ORDER BY \"ChecklistId\" ASC;";
            }
                
	    Statement stmt = con.createStatement();
	    if( debugFlag ) System.out.println( cmd );
	    ResultSet rs = stmt.executeQuery( cmd );
	    if( debugFlag ) System.out.println( "result:" );
            String gettyId = "";
            boolean firstEntryFlag = true ;
	    while( rs.next() == true  )
	    {
		try
		{
		    gettyId = rs.getString( 1 );
                  
                    if( firstEntryFlag )
                    {
                        buffer.append( gettyId );
                        firstEntryFlag = false;
                    }
                    else
                    {
                        buffer.append( "," + gettyId );
                    }                        
		}
		catch( Exception ex )
		{
                    logger.error( ex.getMessage(), ex.fillInStackTrace() );
		}
	    }
	    stmt.close();
	}
	catch( Exception ex )
	{
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	} 
        //----------------------------------------------------------------------
        // WEB-SERVICE CLIENT
        //
        // return Map-Filename        
        //----------------------------------------------------------------------
        String mapFileName = null;
        try
        {
            String locations = buffer.toString();
            logger.info( "Build GRASS-GIS map for location(s): " + locations );
            java.net.URL url = new java.net.URL( WEB_SERVICE_ENDPOINT );
            // ???
            // test connection to server
            try
            {
                // ??? test connection to server
                java.net.Socket s = new java.net.Socket();
                if( url.getPort() == -1 )
                {   
                    // use port 80
                    s.connect( new java.net.InetSocketAddress( url.getHost(), 80 ), 3000 );
                }
                else
                {    
                    s.connect( new java.net.InetSocketAddress( url.getHost(), url.getPort() ), 3000 );
                }
                s.close();
            }
            catch( java.io.IOException ex )
            {
                // DnOptionPane.showMessageDialog( null, ex.toString(), "Error" , DnOptionPane.ERROR_MESSAGE );     
                DnOptionPane.showMessageDialog( null, "Timeout: Can't connection to Webservice", "Error" , DnOptionPane.ERROR_MESSAGE );     
                return;
            }
            
            Service service = new Service();
            Call    call    = ( Call ) service.createCall();            
            // ???
            call.setTimeout( new java.lang.Integer( 20000 ) );
            call.setTargetEndpointAddress( url );
            call.setOperationName( "visualisation" );
            call.addParameter( "parameter", XMLType.XSD_STRING, ParameterMode.IN );
            call.setReturnType( XMLType.XSD_STRING );

            mapFileName = (String) call.invoke( new Object [] { locations } );

            logger.info( "Get map: " + GRASS_MAP_LOCATION + mapFileName + GRASS_MAP_EXTENSION  );
        }
        catch( java.lang.Exception ex )
        {
            logger.error( ex.getLocalizedMessage() );
            DnOptionPane.showMessageDialog( null, ex.toString(), "Error" , DnOptionPane.ERROR_MESSAGE );     
        }   
        System.out.println( buffer );
        // Show the new map with this locations
        try
        {
            imageViewer.showUrl( new java.net.URL( GRASS_MAP_LOCATION + mapFileName + GRASS_MAP_EXTENSION ) );
        }    
        catch( java.net.MalformedURLException ex )
        {
            logger.error( GRASS_MAP_LOCATION + mapFileName + GRASS_MAP_EXTENSION );
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        
    }
    //--------------------------------------------------------------------------
    // 
    //--------------------------------------------------------------------------
    private java.util.Vector setupGenusNames()
    {
        // Build a new grass-gis- map 
        boolean debugFlag = false;
        boolean newLineFlag = false;
        java.util.Vector values = new java.util.Vector();
        
	try
	{                
	    String cmd = "SELECT DISTINCT \"Genus\" FROM \"checklists_data\" WHERE \"Code\" LIKE 'a%' ORDER BY \"Genus\" ASC;";
	    Statement stmt = con.createStatement();
	    if( debugFlag ) System.out.println( cmd );
	    ResultSet rs = stmt.executeQuery( cmd );
	    if( debugFlag ) System.out.println( "result:" );

            String genus   = "";
	    while( rs.next() == true  )
	    {
                newLineFlag = false;
		try
		{
		    genus   = rs.getString( 1 );
		}
		catch( Exception ex )
		{
                    logger.error( ex.getMessage(), ex.fillInStackTrace() );
		}
                values.add( genus );                
	    }
	    stmt.close();
	}
	catch( Exception ex )
	{
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	}        
        
        return values;
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    private java.util.Vector setupEpithetNames()
    {
        // Build a new grass-gis- map 
        boolean debugFlag = false;
        boolean newLineFlag = false;
        java.util.Vector values = new java.util.Vector();
                
	try
	{                
	    String cmd = "SELECT DISTINCT \"Species\" FROM \"checklists_data\" WHERE \"Genus\" = '" + genusSelection + "' AND \"Code\" LIKE 'a%' ORDER BY \"Species\" ASC;";
	    Statement stmt = con.createStatement();
	    if( debugFlag ) System.out.println( cmd );
	    ResultSet rs = stmt.executeQuery( cmd );
	    if( debugFlag ) System.out.println( "result:" );

            String epithet   = "ALL";
            values.add( epithet );
	    while( rs.next() == true  )
	    {
                newLineFlag = false;
		try
		{
		    epithet = rs.getString( 1 );
		}
		catch( Exception ex )
		{
                    logger.error( ex.getMessage(), ex.fillInStackTrace() );
		}
                values.add( epithet );                
	    }
	    stmt.close();
	}
	catch( Exception ex )
	{
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	}        
        
        return values;
    }
    //--------------------------------------------------------------------------
    //
    //--------------------------------------------------------------------------
    public void internalFrameClosed(javax.swing.event.InternalFrameEvent evt)
    {
        logger.debug( "Exit: colse connection" );
        try
        {
            if( con != null )
            {    
                logger.info( "ChecklistGisVisualisation: close database connection" );
                con.close();
            }    
	}
	catch( Exception ex )
	{
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
	} 
        System.exit( 0 );
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main( String args[] ) throws Exception
    {
        Connection con = null;
        try
        {
            Class.forName( "org.postgresql.Driver" );
            logger.info( "Load PostgreSQL Driver..." );
        }
        catch( java.lang.ClassNotFoundException ee )
        {
            logger.error( "ClassNotFoundException: " );
            logger.error( ee.getMessage() );
            System.err.print( "ClassNotFoundException: " );
            System.err.println( ee.getMessage() );
        }
        try
        {
            con = java.sql.DriverManager.getConnection( "jdbc:postgresql://hongkong:5432/checklists", "didi", "" );
        }
        catch( java.sql.SQLException ex )
        {
             logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }
        // only for testing
        final ChecklistsGisVisualisation cgv = new ChecklistsGisVisualisation( con );               
        
        try
        {
            org.apache.log4j.BasicConfigurator.configure();
            logger.setLevel( Level.DEBUG );
            javax.swing.JFrame frame = new javax.swing.JFrame();
            frame.getContentPane().add( cgv, java.awt.BorderLayout.CENTER );
            frame.setDefaultCloseOperation( javax.swing.JFrame.EXIT_ON_CLOSE );
            frame.setSize( 500, 100 );
            frame.pack();
            frame.setVisible( true );

            frame.addWindowListener(new java.awt.event.WindowAdapter()
            {
                public void windowClosing(java.awt.event.WindowEvent evt)
                {
                    if( cgv.con != null )
                    {
                        try
                        {
                            logger.info( "close connection" );
                            cgv.con.close();
                        }
                        catch( java.sql.SQLException ex )
                        {
                            logger.error( ex.getMessage(), ex.fillInStackTrace() );
                        }
                    }
                }
            });            
        }
        finally
        {
            
        }                
    }
}
