/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * ProjectEditPanel.java
 *
 * Created on 25. Mai 2004, 15:07
 */

package net.metadiversity.diversity.navigator.gui;

import com.jgoodies.plaf.plastic.Plastic3DLookAndFeel;

import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.factories.ButtonBarFactory;

import java.awt.event.ActionListener;

/**
 *
 * @author  didi
 */
public class FormsProjectEditPanel extends DnDialog
{    
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3257846575899489072L;
    private javax.swing.JLabel  nameLabel;
    private javax.swing.JLabel  infoLabel;
    private javax.swing.JLabel  classLabel;
    private javax.swing.JLabel  selectionLabel;
    
    private javax.swing.JTextField nameTextField;
    private javax.swing.JTextField infoTextField;
    private javax.swing.JTextField classTextField;
    
    // Selection    
    private javax.swing.DefaultListModel model;    
    private java.util.Vector    selectedItems;
    private java.util.Vector    unselectedItems;
    private javax.swing.JFrame  frame;    
    // Selection GUI
    private javax.swing.JButton addButton;
    private javax.swing.JButton deleteButton;
    private javax.swing.JButton helpButton;
    private javax.swing.JList list;
    private javax.swing.JPanel panel;
    private javax.swing.JScrollPane scrollPane;    
    // Dialog
    private javax.swing.JButton saveDialogButton;
    private javax.swing.JButton cancelDialogButton;
    private javax.swing.JButton helpDialogButton;
    
    // Flags
    boolean showHelpFlag = false;
    // Listeners
    private LocalActionListener localActionListener;
    
    // public FormsProjectEditPanel( java.awt.Frame owner, boolean modal )
    public FormsProjectEditPanel( javax.swing.JFrame owner, boolean modal, String name, String info, String clazz )
    {
        super( owner, modal );
        frame = owner;
        nameTextField   = new javax.swing.JTextField();
        infoTextField   = new javax.swing.JTextField();
        classTextField  = new javax.swing.JTextField();
        if( name  == null )  
            nameTextField.setText( "" );
        else                 
            nameTextField.setText( name );
        if( info  == null )  
            infoTextField.setText( "" );
        else
            infoTextField.setText( info );
        if( clazz == null )  
            classTextField.setText( "" );
        else
            classTextField.setText( clazz );
        initComponents();
        setDialogSize();
        setDefaultCloseOperation( DISPOSE_ON_CLOSE );
        setResizable( false );
        center();
    }
    
    private void initComponents()
    {
        localActionListener = new LocalActionListener();
        
        nameLabel       = new javax.swing.JLabel();
        infoLabel       = new javax.swing.JLabel();
        classLabel      = new javax.swing.JLabel();
        selectionLabel  = new javax.swing.JLabel();

        CellConstraints cc = new CellConstraints();
        
        getContentPane().setLayout( new FormLayout( /* columnes */ "10dlu, 40dlu, 10dlu, 80dlu, 10dlu, 40dlu, 10dlu", 
                                           //             1       2      3  4      5  6      7  8      9  10     11 12      13 14      15 16     17 18  
                                           //                            <--    input fields          -->     <-- selection          -->   
                                           /* rows */     "10dlu, 45dlu, 12dlu, 15dlu, p, 15dlu, p, 15dlu, p, 15dlu, p, 15dlu , p, 15dlu , 12dlu, 15dlu, p, 10dlu"  
                                 ) );

        
        nameLabel.setText("Name:");
        infoLabel.setText("Info");
        classLabel.setText("Class");
        selectionLabel.setText("Selection");
        
        final int fieldPosY =  4;
        
        getContentPane().add( new javax.swing.JSeparator(), cc.xywh( 2, 3, 5, 1 ) );
        
        getContentPane().add( nameLabel,         cc.xy( 2, fieldPosY + 0 ) );
        getContentPane().add( infoLabel,         cc.xy( 2, fieldPosY + 2 ) );
        getContentPane().add( classLabel,        cc.xy( 2, fieldPosY + 4 ) );
        
        getContentPane().add( nameTextField,     cc.xywh( 4, fieldPosY + 0, 3, 1 ) );     
        getContentPane().add( infoTextField,     cc.xywh( 4, fieldPosY + 2, 3, 1 ) );
        getContentPane().add( classTextField,    cc.xywh( 4, fieldPosY + 4, 3, 1 ) );     
        
        //--------------------------------------
        // Selection
        //--------------------------------------
        final int selectionPosX = 4;
        final int selectionPosY = fieldPosY + 6;        
        
        getContentPane().add( selectionLabel,    cc.xy( 2, selectionPosY ) );
        
        addButton       = new javax.swing.JButton();
        deleteButton    = new javax.swing.JButton();        
        scrollPane      = new javax.swing.JScrollPane();
        list            = new javax.swing.JList();
        
        selectedItems = new java.util.Vector();        
        selectedItems.add( "Diversity Workbench 1" );
        selectedItems.add( "Diversity Workbench 2" );
        selectedItems.add( "Diversity Workbench 3" );
        selectedItems.add( "Diversity Workbench 4" );
        selectedItems.add( "Diversity Workbench 5" );
        selectedItems.add( "Diversity Workbench 6" );
        
        unselectedItems = new java.util.Vector();        
        unselectedItems.add( "Data 1" );
        unselectedItems.add( "Data 2" );
        unselectedItems.add( "Data 3" );
        unselectedItems.add( "Data 4" );

        model           = new javax.swing.DefaultListModel();
        // init items        
        java.util.ListIterator iter = selectedItems.listIterator();
        while( iter.hasNext() )
        {
            model.addElement( iter.next() );
        }        
        
        addButton.setMnemonic('A');
        addButton.setText("Add");
        addButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                addButtonActionPerformed(evt);
            }
        });

        deleteButton.setMnemonic('D');
        deleteButton.setText("Delete");
        deleteButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                deleteButtonActionPerformed(evt);
            }
        });
        getContentPane().add( addButton,    cc.xywh( selectionPosX + 2, selectionPosY + 0, 1, 1 ) );
        getContentPane().add( deleteButton, cc.xywh( selectionPosX + 2, selectionPosY + 2, 1, 1 ) );

        if( showHelpFlag )
        {
            helpButton      = new javax.swing.JButton();
            helpButton.setMnemonic('H');
            helpButton.setText("Help");
            helpButton.addActionListener(new java.awt.event.ActionListener()
            {
                public void actionPerformed(java.awt.event.ActionEvent evt)
                {
                    helpButtonActionPerformed(evt);
                }
            });
            getContentPane().add( helpButton,   cc.xywh( selectionPosX + 2, selectionPosY + 4,1, 1 ) );
        }                
        
        list.setModel( model );
        scrollPane.setViewportView(list);

        getContentPane().add( scrollPane,   cc.xywh( selectionPosX, selectionPosY, 2, 5 ) );
        
        getContentPane().add( new javax.swing.JSeparator(), cc.xywh( 2, selectionPosY + 5, 5, 1 ) );
        // Dialog
            helpDialogButton      = new javax.swing.JButton();
            helpDialogButton.setMnemonic('H');
            helpDialogButton.setText("Help");
            helpDialogButton.addActionListener( localActionListener );

            cancelDialogButton      = new javax.swing.JButton();
            cancelDialogButton.setMnemonic('C');
            cancelDialogButton.setText("Cancel");
            cancelDialogButton.addActionListener( localActionListener );

            saveDialogButton      = new javax.swing.JButton();
            saveDialogButton.setMnemonic('S');
            saveDialogButton.setText("Save");
            saveDialogButton.addActionListener( localActionListener );

            getContentPane().add( buildButtonBar(), cc.xywh( 2, selectionPosY + 6, 5, 1));
    }          
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    /**
     * Builds the button bar.  Uses jgoodies.com's ButtonBarFactory class.
     */
    private java.awt.Component buildButtonBar() 
    {
        return ButtonBarFactory.buildHelpOKCancelBar( helpDialogButton, saveDialogButton, cancelDialogButton);
    }
    /**
     * Sets the dialog's size.
     */
    void setDialogSize() 
    {
        pack();
    }
    //--------------------------------------------------------------------------
    private void closeDialog()
    {
        setVisible(false);
        dispose();
    }    
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------       
    private class LocalActionListener implements ActionListener 
    {
        public void actionPerformed( java.awt.event.ActionEvent actionEvent )
        {
            Object source = actionEvent.getSource();
            // SAVE BUTTON
            if( source == saveDialogButton ) 
            {
                System.out.println( "save button" );
            }
            // CANCEL BUTTEN
            if( source == cancelDialogButton ) 
            {
                System.out.println( "cancel button" );
                closeDialog();                
            }
            // HELP BUTTON
            if( source == helpDialogButton ) 
            {
                System.out.println( "help button" );            
            }            
        }        
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
        
    private void helpButtonActionPerformed(java.awt.event.ActionEvent evt)
    {
        // Add your handling code here:
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    private void deleteButtonActionPerformed(java.awt.event.ActionEvent evt)
    {
        Object [] sel = list.getSelectedValues();
        if( sel != null )
        {
            for( int i = 0; i < sel.length; i++ )
            {
                if( DnOptionPane.showConfirmDialog( frame, "Do you really want to delete this entry: " + sel[ i ], "Confirmation", DnOptionPane.YES_NO_OPTION ) == DnOptionPane.OK_OPTION )
                {
                    selectedItems.remove( sel[ i ] );
                    model.removeElement(  sel[ i ] );
                    unselectedItems.add(  sel[ i ] );
                }
            }
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    private void addButtonActionPerformed(java.awt.event.ActionEvent evt)
    {
        // Add your handling code here:
        PopupSelectionList psl = new PopupSelectionList( frame, true, unselectedItems );
        /*
        System.out.println( frame.getLocation() );
        psl.setLocation( frame.getLocation() );
        */
        psl.setVisible( true );
        
        System.out.println( "Add:" );        
        java.util.Vector v = psl.getSelectedValues();
        if( v != null )
        {
            for( int i = 0; i < v.size(); i++ )
            {
                 selectedItems.add( v.get( i ) );
                 model.addElement( v.get( i ) );
                 unselectedItems.remove( v.get( i ) );
            }                 
        }
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public static void main(String args[])
    {
        try
        {
            javax.swing.UIManager.setLookAndFeel( new Plastic3DLookAndFeel() );
        }
        catch( javax.swing.UnsupportedLookAndFeelException ex )
        {
        }
        javax.swing.JFrame frame = new javax.swing.JFrame();                
        // frame.getContentPane().add( new FormsProjectEditPanel( frame, true ) );
        FormsProjectEditPanel pep = new FormsProjectEditPanel( frame, true, "Name", "Info", "Class" );        
        frame.pack();
        frame.setVisible( true );
        pep.setVisible( true );
        System.out.println( "Return from dialog" );
        frame.dispose();
    }    
}
