/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*
 * ProjectQueryPanel.java
 *
 * Created on 14. November 2003, 15:15
 */

package net.metadiversity.diversity.navigator.gui;

import org.apache.log4j.Logger;
import org.apache.log4j.Level;

import net.metadiversity.diversity.navigator.project.*;
import net.metadiversity.diversity.navigator.gui.DnOptionPane;

// ???
import net.metadiversity.diversity.navigator.connection.ConnectionsManager;
import net.metadiversity.diversity.navigator.connection.ConnectionInfo;

/**
 *
 * @author  Dieter Neubacher
 */
public class ProjectQueryPanel extends javax.swing.JPanel
{
    /**
     * Comment for <code>serialVersionUID</code>
     */
    private static final long serialVersionUID = 3257283651749031987L;

    static Logger logger = Logger.getLogger( ProjectQueryPanel.class.getName() ); 
    
    DnDbProjectRoot projectRoot;
    DnDbProject     project;
    DnDbGroup       group;
    
    int size;
    
    java.awt.Component parent;
    
    // ??? Query Results ???
    private javax.swing.JTabbedPane     queryResultComponent;
    private java.sql.Connection         queryDatabaseConnection;
    
    //--------------------------------------------------------------------------
    // 
    //--------------------------------------------------------------------------
    public void setQueryResultComponent( javax.swing.JTabbedPane component )
    {
        queryResultComponent = component;
    }    
    //--------------------------------------------------------------------------
    // Creates new form ProjectQueryPanel
    //--------------------------------------------------------------------------
    public ProjectQueryPanel( java.awt.Component parent, String projectName, String groupName )
    {
        logger.debug( "TextQuery()" );
        
        this.parent = parent;
        
        queryResultComponent = null;
        size = 0;
        try
        {
            projectRoot = new DnDbProjectRoot();
            setProjectName( projectName );
            setGroupName( groupName );
            project.addDnGroup( group );
            projectRoot.addDnProject( project );
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }        
        initComponents();
        setSize( 640, 400 );
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setProjectName( String projectName )
    {
        project = new DnDbProject( projectName );
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void setGroupName( String groupName )
    {
        group = new DnDbGroup( groupName );        
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    public void addDatabase( String groupName, String database ) throws java.sql.SQLException
    {
        // ??? group is fix
        group.addDnDatabase( new DnDatabase( database ) );        
    }
    //--------------------------------------------------------------------------
    //--------------------------------------------------------------------------
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel1 = new javax.swing.JLabel();
        executeQueryButton = new javax.swing.JButton();
        jSplitPane1 = new javax.swing.JSplitPane();
        queryTreeScrollPane = new javax.swing.JScrollPane();
        queryTree = new javax.swing.JTree();
        queryPanelScrollPane = new javax.swing.JScrollPane();
        queryPanel = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jLabel2 = new javax.swing.JLabel();

        setLayout(new java.awt.BorderLayout());

        jLabel1.setText("Query");
        add(jLabel1, java.awt.BorderLayout.NORTH);

        executeQueryButton.setText("Execute Query");
        executeQueryButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                executeQueryButtonActionPerformed(evt);
            }
        });

        add(executeQueryButton, java.awt.BorderLayout.SOUTH);

        queryTree.setCellRenderer(new ProjectQueryTreeRenderer());
        queryTree.setModel(new QueryTreeModel( projectRoot ) );
        queryTree.setRootVisible(false);
        queryTree.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                queryTreeKeyReleased(evt);
            }
        });
        queryTree.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                queryTreeMouseReleased(evt);
            }
        });

        queryTreeScrollPane.setViewportView(queryTree);

        jSplitPane1.setLeftComponent(queryTreeScrollPane);

        queryPanel.setLayout(new javax.swing.BoxLayout(queryPanel, javax.swing.BoxLayout.Y_AXIS));

        queryPanel.setName("queryPanel");
        queryPanel.setAutoscrolls(true);
        queryPanelScrollPane.setViewportView(queryPanel);

        jSplitPane1.setRightComponent(queryPanelScrollPane);

        add(jSplitPane1, java.awt.BorderLayout.CENTER);

        jLabel2.setIcon(new javax.swing.JLabel() {
            public javax.swing.Icon getIcon() {
                try {
                    return new javax.swing.ImageIcon(
                        new java.net.URL("http://localhost/icons/test.png")
                    );
                } catch (java.net.MalformedURLException e) {
                }
                return null;
            }
        }.getIcon());
        jLabel2.setAutoscrolls(true);
        jScrollPane1.setViewportView(jLabel2);

        add(jScrollPane1, java.awt.BorderLayout.EAST);

    }//GEN-END:initComponents

    private void executeQueryButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_executeQueryButtonActionPerformed
        
        // Build query string
        int count = queryPanel.getComponentCount();
        boolean firstWhereFlag = true;
        
        StringBuffer query = new StringBuffer();
        StringBuffer where = new StringBuffer();
        java.util.HashSet tables = new java.util.HashSet(); 
        
        java.lang.String database = null;
        query.append( "SELECT " );
        for( int i = 0; i < count; i++ )
        {
            // System.out.println( queryPanel.getComponent( i ).toString() );
            BasicQueryField bqf = null;
            try
            {
                bqf = ( BasicQueryField ) queryPanel.getComponent( i );
            }
            catch( java.lang.ClassCastException ex )
            {
                continue;
            }
            // we can only use one database
            if( database == null )
            {    
                database = bqf.getDatabaseName();
            }
            else
            {
                if( database.compareTo( bqf.getDatabaseName() ) != 0 )
                {
                    logger.error( "Query now can use only one database" );
                }
            }
            String table = bqf.getTableName();
            
            tables.add( table );
            
            String field = bqf.getFieldName();
            
            query.append( "\"" + table + "\".\"" + field + "\"" );
            if( i < count -1 )
            {
                query.append( ", " );
            }            
            // where
            String w = bqf.getWhere();
            if( w == null )
            {
                continue; // no where cause
            }
            else
            {
                if( ! firstWhereFlag )
                {
                    where.append( bqf.getLogicalOperator() );
                }
                where.append( w );
                firstWhereFlag = false;
            }
            
        }
        //--- FROM--------------------------------------------------------------
        query.append( " FROM " );
        
        int ts = tables.size();
        java.util.Iterator iter = tables.iterator();
        while( iter.hasNext() )
        {
            query.append( "\"" + iter.next() + "\" " );            
           
            if( iter.hasNext() ) query.append( ", " );
        }
        //--- WHERE ------------------------------------------------------------
        if( ! firstWhereFlag )
        {
            // if there is a where clause, append it to query
            query.append( " WHERE " + where.toString() );
        }
        //--- GROUP BY ---------------------------------------------------------
        //--- HAVING -----------------------------------------------------------
        //--- ORDER BY ---------------------------------------------------------
        //---- LIMIT -----------------------------------------------------------
        int limitCount = 10000;
        query.append( " LIMIT " + limitCount  );
        //----------------------------------------------------------------------
        logger.error( query.toString() );
        
        // Execute Query and show Results
        // ???
        // get Connection
        // java.beans.EventHandler.create( ActionListener.class, , "dn query event" );
        // Show result TabSheet
        // ???
        // we need the queryDatabaseConnection
        // how to fine/cearte this conection ???

        try
        {
            // get conection info for this database       
            ConnectionInfo ci;
            ci = ConnectionsManager.getConnectionInfoByName( database );        // databaseName 
            queryDatabaseConnection = ci.getConnection();
            
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }    
        
        queryResultComponent.remove( 2 );    

        QueryResultSetTable qrst = new QueryResultSetTable( queryDatabaseConnection, query.toString() );
        qrst.setName( "Grid view" );
        queryResultComponent.add( qrst, 2 );

        queryResultComponent.setVisible( true );
    }//GEN-LAST:event_executeQueryButtonActionPerformed

    private void queryTreeMouseReleased(java.awt.event.MouseEvent evt)//GEN-FIRST:event_queryTreeMouseReleased
    {//GEN-HEADEREND:event_queryTreeMouseReleased
        // System.out.println( evt );
        // logger.debug( "mousePressed( " + e.getModifiers() + " )" );
        // Apple Mac used (ALT) ??? mouse for right butten simulation
        if( ( evt.getModifiers() & java.awt.event.InputEvent.BUTTON3_MASK ) == java.awt.event.InputEvent.BUTTON3_MASK ||
            ( evt.getModifiers() & java.awt.event.InputEvent.BUTTON2_MASK ) == java.awt.event.InputEvent.BUTTON2_MASK   
          )

        {    
            //------------------
            // show popup menu
            //------------------
            int xx = evt.getX();
            int yy = evt.getY();
            int x = xx - queryTreeScrollPane.getHorizontalScrollBar().getValue();
            int y = yy - queryTreeScrollPane.getVerticalScrollBar().getValue();
            
            javax.swing.JPopupMenu p = new javax.swing.JPopupMenu();
            DnMenuItem  i = new DnMenuItem( "query.add" );
            i.addActionListener( new java.awt.event.ActionListener()
            {
                public void actionPerformed( java.awt.event.ActionEvent evt ) 
                {
                    String str = (String) queryTree.getSelectionPath().getLastPathComponent();
                    
                    // field
                    int pos = str.lastIndexOf( '.' );
                    String field = str.substring( pos + 1 ).trim();                    
                    // table
                    str = str.substring( 0 , pos );
                    pos = str.lastIndexOf( '.' );
                    String table = str.substring( pos + 1 ).trim();
                    // database                    
                    str = str.substring( 0 , pos );
                    pos = str.lastIndexOf( '.' );
                    String database = str.substring( pos + 1 ).trim();
                    addToQuery( database, table, field );
                }
            } );
            i.setText( "add to query" );
            p.add( i );
            p.show( queryTreeScrollPane, x, y );
            // p.setLocation( xx, yy );
            // p.setVisible( true );
            
            if( queryTree.getSelectionPath() == null ) 
            {
                return;
            }
//            javax.swing.tree.TreeNode selection = ( javax.swing.tree.TreeNode ) queryTree.getSelectionPath().getLastPathComponent();
            
            Object selection = queryTree.getSelectionPath().getLastPathComponent();
        
            if( selection == null ) 
            {
                return;
            }
            else
            {
                System.out.println( selection );
            }
            /*
            if( Sel.getNodeType() == DBNode.NODE_TYPE_CAPTION_CONNECTIONS ) popConnections.show( this, x,y );
            if( Sel.getNodeType() == DBNode.NODE_TYPE_DATABASE            ) popDatabase.show( this, x,y );
            if( Sel.getNodeType() == DBNode.NODE_TYPE_TABLE               ) popTable.show( this, x,y );
            */
        }                               
    }//GEN-LAST:event_queryTreeMouseReleased

    private void queryTreeKeyReleased(java.awt.event.KeyEvent evt)//GEN-FIRST:event_queryTreeKeyReleased
    {//GEN-HEADEREND:event_queryTreeKeyReleased
        // Add your handling code here:

    }//GEN-LAST:event_queryTreeKeyReleased
    
    private void addToQuery( java.lang.String database, java.lang.String table, java.lang.String field ) 
    {
        size += 50;
        // show an internal dialog
        // int i = DnOptionPane.showInternalConfirmDialog( queryPanel, "Add new entry" );
        int i = DnOptionPane.showConfirmDialog( parent, "Add new entry" );
        if( i == 0 )
        {    
            // queryPanal.setSize( new java.awt.Dimension( size, 200 ) );                        
            queryPanel.add( new BasicQueryField( database, table, field ) );
        }
//        queryPanel.setVisible( true );
//        queryPanel.repaint();
        parent.setVisible( true );
        parent.repaint();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton executeQueryButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JPanel queryPanel;
    private javax.swing.JScrollPane queryPanelScrollPane;
    private javax.swing.JTree queryTree;
    private javax.swing.JScrollPane queryTreeScrollPane;
    // End of variables declaration//GEN-END:variables
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) 
    {
        try
        {
            org.apache.log4j.BasicConfigurator.configure();
            logger.getRoot().setLevel( Level.DEBUG );
            logger.setLevel( Level.DEBUG );

            javax.swing.JFrame frame = new javax.swing.JFrame();
            ProjectQueryPanel queryPanel = new ProjectQueryPanel( frame, "BIOTA S04 Lichens", "Diversity Workbench" );
            
            queryPanel.addDatabase( null, "LIAS Checklists" );
            queryPanel.addDatabase( null, "LIAS Descriptive Data (disabled)" );
            
            frame.getContentPane().add( queryPanel );
            
            frame.setVisible( true );
        }
        catch( java.sql.SQLException ex )
        {
            ex.printStackTrace();
        }
        finally
        {
            
        }                
    }
}
