/*
 * Copyright (C) 2001-2006 University Bayreuth, BayCEER, Dept. of Mycology 
 * 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package net.metadiversity.diversity.navigator.project;

//import javax.swing.event.*;
//import javax.swing.tree.*;
import java.util.*;

import java.sql.*;

import org.apache.log4j.Logger;

import net.metadiversity.diversity.navigator.connection.*;

/**
 * 
 * @author didi
 */
public class ProjectNode
{
    static Logger    logger                     = Logger.getLogger( ProjectNode.class.getName() );

    static final int NODE_TYPE_PROJECTS_CAPTION = 1;
    static final int NODE_TYPE_PROJECT          = 2;
    static final int NODE_TYPE_DB_GROUP         = 3;
    static final int NODE_TYPE_DATABASE         = 4;
    static final int NODE_TYPE_TABLE            = 5;
    static final int NODE_TYPE_FIELD            = 6;

    String           name                       = "";
    int              nodeType                   = 0;

    Object           data                       = null;
    Object           data2                      = null;
    Vector           childs                     = null;
    ProjectNode      parentNode                 = null;
    ProjectPane      inPane;

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    /**
     * @param inPane
     * @param parentNode
     * @param nodeType
     * @param name
     */
    public ProjectNode( ProjectPane inPane, ProjectNode parentNode, int nodeType, String name )
    {
        this( inPane, parentNode, nodeType, name, null );
    }

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    /**
     * @param inPane
     * @param parentNode
     * @param nodeType
     * @param name
     * @param data
     */
    public ProjectNode( ProjectPane inPane, ProjectNode parentNode, int nodeType, String name, Object data )
    {
        this.nodeType = nodeType;
        this.name = name;
        this.parentNode = parentNode;
        this.inPane = inPane;
        this.data = data;
    }

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    /**
     * @return
     */
    public String getName()
    {
        return this.name;
    }

    /**
     * @return
     */
    public int getNodeType()
    {
        return this.nodeType;
    }

    /**
     * @return
     */
    public Object getData()
    {
        return data;
    }

    /**
     * @param data
     */
    public void setData( Object data )
    {
        this.data = data;
    }

    /**
     * @return
     */
    public Object getData2()
    {
        return data2;
    }

    /**
     * @param data
     */
    public void setData2( Object data )
    {
        this.data2 = data;
    }

    /**
     * @return
     */
    public ProjectNode getParentNode()
    {
        return this.parentNode;
    }

    // ----------------------------------------------------------------------
    public String getProjectName()
    {
        if( nodeType == NODE_TYPE_DB_GROUP )
        {
            return null;
        }
        ProjectNode node = this;
        while( node.getNodeType() != NODE_TYPE_PROJECT )
        {
            node = node.getParentNode();
        }
        return node.getName();
    }

    // ----------------------------------------------------------------------
    /**
     * @return
     */
    public ConnectionInfo getConnectionInfo()
    {
        if( nodeType == NODE_TYPE_DB_GROUP )
        {
            return null;
        }
        ProjectNode node = this;
        while( node.getNodeType() != NODE_TYPE_DATABASE )
        {
            node = node.getParentNode();
        }
        return (ConnectionInfo) node.getData();
    }

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    /**
     * @return
     */
    public Connection getConnection()
    {
        if( nodeType == NODE_TYPE_DB_GROUP )
        {
            return null;
        }
        ProjectNode node = this;
        while( node.getNodeType() != NODE_TYPE_DATABASE )
        {
            node = node.getParentNode();
        }
        if( node.getData2() == null )
        {
            node.load();
        }
        return (Connection) node.getData2();
    }

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    /**
     * @return
     */
    public String getTable()
    {
        if( nodeType == NODE_TYPE_DB_GROUP || nodeType == NODE_TYPE_DATABASE )
        {
            return null;
        }

        ProjectNode node = this;
        while( node.getNodeType() != NODE_TYPE_TABLE )
        {
            node = node.getParentNode();
        }
        return node.getName();
    }

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    private void load()
    {
        try
        {
            if( childs != null )
            {
                return;
            }
            Connection con = null;

            switch( this.getNodeType() )
            {
            // root
            case NODE_TYPE_PROJECTS_CAPTION:

                childs = new Vector();
                LinkedHashMap projects = ProjectManager.getProjects();
                Iterator iter = projects.keySet().iterator();
                while( iter.hasNext() )
                {
                    ProjectInfo pi = (ProjectInfo) projects.get( iter.next() );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_PROJECT, pi.getName() ) );
                }
                break;

            case NODE_TYPE_PROJECT:
            {
                childs = new Vector();

                if( this.getName().startsWith( "LIAS" ) )
                {
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DB_GROUP, "Diversity Workbench" ) );
                }
                if( this.getName().startsWith( "DEEMY" ) )
                {
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DB_GROUP, "Diversity Workbench" ) );
                }
                if( this.getName().startsWith( "BIOTA" ) )
                {
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DB_GROUP, "Diversity Workbench" ) );
                }
                else if( this.getName().startsWith( "Lichen" ) )
                {
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DB_GROUP, "Diversity Workbench" ) );
                }
                else if( this.getName().startsWith( "Bryophyte" ) )
                {
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DB_GROUP, "Diversity Workbench" ) );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DB_GROUP, "Specify [not yet implemented]" ) );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DB_GROUP, "Systax [not yet implemented]" ) );
                }
            }
                break;

            case NODE_TYPE_DB_GROUP:
            {
                childs = new Vector();

                Map connections = ConnectionsManager.getConnections();
                ConnectionInfo ci;

                if( this.getParentNode().getName().startsWith( "LIAS Checklists" ) )
                {
                    ci = (ConnectionInfo) connections.get( "LIAS Checklists" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "LIAS Checklists", ci ) );
                    
                    ci = (ConnectionInfo) connections.get( "LIAS Descriptive Data (disabled)" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "LIAS Descriptive Data (disabled)", ci ) );
                    
                    ci = (ConnectionInfo) connections.get( "Diversity Taxon Names Fungi" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Diversity Taxon Names Fungi", ci ) );
                }
                if( this.getParentNode().getName().startsWith( "LIAS Light" ) )
                {
                    ci = (ConnectionInfo) connections.get( "LIAS Light Descriptions" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "LIAS Light Descriptive Data", ci ) );
                    
                    ci = (ConnectionInfo) connections.get( "Diversity Taxon Names Fungi" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Diversity Taxon Names Fungi", ci ) );
                }
                else if( this.getParentNode().getName().startsWith( "LIAS Main" ) )
                {
                    ci = (ConnectionInfo) connections.get( "LIAS Main Descriptions" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "LIAS Main Descriptive Data", ci ) );
                    
                    ci = (ConnectionInfo) connections.get( "Diversity Taxon Names Fungi" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Diversity Taxon Names Fungi", ci ) );
                }
                else if( this.getParentNode().getName().startsWith( "DEEMY" ) )
                {
                    ci = (ConnectionInfo) connections.get( "DEEMY Descriptions" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "DEEMY Descriptive Data", ci ) );
                    
                    ci = (ConnectionInfo) connections.get( "Diversity Taxon Names Fungi" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Diversity Taxon Names Fungi", ci ) );
                }
                else if( this.getParentNode().getName().startsWith( "BIOTA" ) )
                {
                    ci = (ConnectionInfo) connections.get( "UBT BIOTA Lichen Collections" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "UBT BIOTA Lichen Collections", ci ) );
                    
                    ci = (ConnectionInfo) connections.get( "UBT BIOTA Lichen Descriptions" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "UBT BIOTA Lichen Descriptions", ci ) );
                    
                    ci = (ConnectionInfo) connections.get( "Diversity Taxon Names Fungi" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Diversity Taxon Names Fungi", ci ) );
                    /*
                     * ci = (ConnectionInfo) connections.get( "UBT BIOTA Lichen
                     * Exsiccata" ); childs.add( new ProjectNode( inPane, this,
                     * NODE_TYPE_DATABASE, "UBT BIOTA Lichen Exsiccata", ci ));
                     */
                }
                else if( this.getParentNode().getName().startsWith( "Checklists" ) )
                {
                    ci = (ConnectionInfo) connections.get( "Taxonomy M" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Taxonomy M", ci ) );
                    ci = (ConnectionInfo) connections.get( "References M" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "References M", ci ) );
                    ci = (ConnectionInfo) connections.get( "Gazetteer M" );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Gazetteer M", ci ) );
                }
                else if( this.getParentNode().getName().startsWith( "Lichen" ) )
                {
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Taxonomy M" ) );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Collection M" ) );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Collection HBG" ) );
                    childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Collection UBT" ) );
                }
                else if( this.getParentNode().getName().startsWith( "Bryophyte" ) )
                {
                    if( this.getName().startsWith( "Diversity" ) )
                    {
                        childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Taxonomy M" ) );
                        childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Collection M" ) );
                    }
                    else if( this.getName().startsWith( "Specify" ) )
                    {
                        childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Speciy M" ) );
                    }
                    else if( this.getName().startsWith( "Systax" ) )
                    {
                        childs.add( new ProjectNode( inPane, this, NODE_TYPE_DATABASE, "Systax M" ) );
                    }
                }
            }
                break;

            case NODE_TYPE_DATABASE:

                childs = new Vector();
                ConnectionInfo ci = getConnectionInfo();
                /*
                 * ??? try { inPane.getMainWindow().setConnectionStatus("Loading
                 * driver ..."); Class.forName( ci.getDriver() ).newInstance();
                 * Class.forName( ci.getDriver() ); } catch( Exception ex ) {
                 * inPane.getMainWindow().setConnectionError( "Can't load
                 * driver: " + ci.getDriver() ); logger.error( ex.getMessage(),
                 * ex.fillInStackTrace() ); return; }
                 */
                try
                {
                    inPane.getMainWindow().setConnectionStatus( "Connecting ..." );
                    con = java.sql.DriverManager.getConnection( ci.getUrl(), ci.getUser(), ci.getPwd() );
                    setData2( con );
                }
                catch( Exception ex )
                {
                    inPane.getMainWindow().setConnectionError( "Can't connect to: " + ci.getUrl() );
                    logger.error( ex.getMessage(), ex.fillInStackTrace() );
                    return;
                }
                try
                {
                    inPane.getMainWindow().setConnectionStatus( "Retrieving tables ..." );
                    DatabaseMetaData md = con.getMetaData();
                    String [] types = { "TABLE" };
                    ResultSet tables = md.getTables( null, null, null, types );
                    while( tables.next() )
                    {
                        String tableName = tables.getString( "TABLE_NAME" );
                        ProjectNode child = new ProjectNode( inPane, this, NODE_TYPE_TABLE, tableName );
                        childs.add( child );
                    }
                    tables.close();
                }
                catch( Exception ex )
                {
                    inPane.getMainWindow().setConnectionError( "Can't get tables: " + ex.getMessage() );
                    logger.error( ex.getMessage(), ex.fillInStackTrace() );
                    return;
                }
                inPane.getMainWindow().setConnectionStatus( "Connected to " + ci.getName() );
                break;

            case NODE_TYPE_TABLE:

                childs = new Vector();
                con = (Connection) getParentNode().getData2();
                try
                {
                    inPane.getMainWindow().setConnectionStatus( "Retrieving fields ..." );
                    DatabaseMetaData md = con.getMetaData();
                    String tableName = getName();
                    ResultSet columns = md.getColumns( null, null, tableName, null );
                    while( columns.next() )
                    {
                        String columnName = columns.getString( "COLUMN_NAME" );
                        ProjectNode child = new ProjectNode( inPane, this, NODE_TYPE_FIELD, columnName );
                        childs.add( child );
                    }
                    columns.close();
                }
                catch( Exception ex )
                {
                    inPane.getMainWindow().setConnectionStatus( "Can't get fields: " + ex.getMessage() );
                    logger.error( ex.getMessage(), ex.fillInStackTrace() );
                    return;
                }
                inPane.getMainWindow().setConnectionStatus( "Connected to " + getConnectionInfo().getName() );
                break;
            }
        }
        catch( Exception ex )
        {
            logger.error( ex.getMessage(), ex.fillInStackTrace() );
        }

    }

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    /**
     * @return
     */
    public boolean isLeaf()
    {
        return getNodeType() == NODE_TYPE_FIELD;
    }

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    /**
     * @return
     */
    public int getChildCount()
    {
        load();
        return childs.size();
    }

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    /**
     * @param i
     * @return
     */
    public ProjectNode getChild( int i )
    {
        load();
        return (ProjectNode) childs.elementAt( i );
    }

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    /**
     * @param ochild
     * @return
     */
    public int getChildIndex( Object ochild )
    {
        load();
        ProjectNode child = (ProjectNode) ochild;
        for( int i = 0; i < getChildCount(); i++ )
        {
            if( getChild( i ).getName().equals( child.getName() ) )
                return i;
        }
        return -1;
    }

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    /**
     * @return
     */
    public String toString()
    {
        return this.getName();
    }

    // ----------------------------------------------------------------------
    // ----------------------------------------------------------------------
    public void reloadChilds()
    {
        childs = null;
    }
}
